#!/usr/bin/perl
#
# WacsInstUtil.PM
#
#    (C) Copyright 2006,2008 B King
#    This file is part of WACS.
#
#    WACS is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 3 of the License, or
#    (at your option) any later version.
#
#    WACS is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with WACS.  If not, see <http://www.gnu.org/licenses/>.
#

package Wacs::WacsInstUtil;

require Exporter;

@ISA            = qw(Exporter);
@EXPORT         = qw(checkpkg installpkg cpaninst cgiinst clonecgi
		     copyifabsent);

# global variables

@srcs=('index','manage','models','presentation','retrieval','search',
	'security','tag','migrate','download','install');

# check a package
sub checkpkg( $$ )
{
	my( $pkgname, $rpm_name ) = @_;
	my( $pkg );

	if( $rpm_name =~ /apt-get/ )
	{
		# we enter the Ubuntu zone
		open( RPM, "$rpm_name $pkgname |" ) ||
		  die "Cannot run package query command: ".
			$rpm_name." ".$pkgname.".\n";
		while( $pkg = <RPM> )
		{
			if( $pkg =~ /newest version/ )
			{
				return( "latest" );
			}
		}
		return( undef );
	}
	open( RPM, "$rpm_name $pkgname |" ) ||
		die "Cannot run package query command: $rpm_name $pkgname.\n";
	$pkg = <RPM>;
	close RPM;
	chomp $pkg;
	if( $pkg =~ /is not installed/ )
	{
		return( undef );
	}
	$pkg =~ s/$pkgname//g;
	$pkg =~ s/^\-//;
	if( $debug > 0 )
	{
		print "Found package $pkgname version: $pkg\n";
	}
	return( $pkg );
}

sub installpkg( $$ )
{
        my( $pkgname, $rpminst_name ) =@_;

        print "Package $pkgname is not installed - installing...\n";
        if( system( $rpminst_name." ".$pkgname ) != 0 )
        {
                print "Install of $pkgname failed.  Please seek help.\n";
                exit -2;
        }
	#print "DO: ".$rpminst_name." ".$pkgname."\n";
}

sub cpaninst( $$ ) 
{
	my( $pkgname, $cpan_name )=@_;

	if( $debug > 0 )
	{
		print "Starting CPAN install of $pkgname....\n";
	}
	if( system( $cpan_name." ".$pkgname ) != 0 )
        {
                print "CPAN Install of $pkgname failed.  Please seek help.\n";
                exit -2;
        }
}

sub cgiinst( $$ )
{
	my( $pkgname, $cgibindir )=@_;
	my( $srcdir, $found );
	
	$found = 0;
	if( $debug > 0 )
	{
		print "Installing cgi script $pkgname...\n";
	}
	foreach $srcdir (@srcs)
	{
		if( -f "../$srcdir/$pkgname" )
		{
			if( $debug > 1 )
			{
				print "installing $pkgname from $srcdir...\n";
			}
			system( "cp -p ../$srcdir/$pkgname $cgibindir" );
			system( "chmod 755 $cgibindir/$pkgname" );
			return 1;
		}
	}
	return( 0 );
}

sub clonecgi( $$$$$ )
{
	my( $src, $dst, $var, $val, $dir )=@_;

	if( $debug > 0 )
	{
		print "Cloning $src to make $dst...\n";
	}
	# clone the file and mode
	system( "cp -p $dir/$src $dir/$dst" );

	# change the specified variable
	open( EDITOR, "| ed $dir/$dst" );
	print EDITOR "/$var\n";
	print EDITOR ".s/$var=0/$var=$val/p\n";
	print EDITOR "w\n";
	print EDITOR "q\n";
	close EDITOR;
}

sub copyifabsent( % )
{
	my( %params )=@_;
	my( $srcfile, $srcdir, $destdir );

	$srcfile=$params{'file'};
	$srcdir=$params{'srcdir'};
	$destdir=$params{'destdir'};

	# make the destination directory if it doesn't exist
	if( ! -d $destdir )
	{
		system( "mkdir -p ".$destdir );
	}
	
	# check for the existance of the destination file
	if( -f $destdir."/".$srcfile )
	{
		if( $debug > 0 )
		{
			print STDERR "copyifabsent: ".$srcfile." exists in ".
					$destdir."\n";
		}
		return 0;
	}
	system( "cp -rp ".$srcdir."/".$srcfile." ".$destdir );
	return 1;
}

# show that we parsed correctly
1;

