#
#    WACS.PM - core perl modules
#
#    (C) Copyright 2006,2007,2008,2009,2010,2011 B King
#    This file is part of WACS.
#
#    WACS is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 3 of the License, or
#    (at your option) any later version.
#
#    WACS is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with WACS.  If not, see <http://www.gnu.org/licenses/>.
#

package Wacs;

require Exporter;
use XML::Simple;
use Data::Dumper;

@ISA 		= qw(Exporter);
@EXPORT 	= qw(read_conf conf_get_attr check_auth auth_user
			auth_get_attr auth_error add_auth
			find_config_location dberror gettoday timecomps
			vendlink 
			getvaluename geticonlist gettypecolour divideup 
			checkexclude checkindex makedbsafe
			addheadercss setgroupperms treemkdir );

# the debug level is now set through the wacs config file.
# actual assignment is done in two steps below because of self-referential
# concerns

# Assumptions
my $fssep = '/';

# Where to find the WACS configuration
my $default_location1="/etc/wacs.d";
my $default_location2="/usr/local/etc/wacs.d";
my $default_location3="/opt/wacs/etc/wacs.d";
my $default_specifier="WACS_CONFIG";
my $main_conf="wacs.cfg";
my $main_auth="wacs.acl";

# Package globals
my $treehandle;
my $authhandle;
my $leasehandle;

# DEFAULTS SECTION:
# the default values for the configuration files
my( %dblist )=(
	'dateformat'=>'YYYY-MM-DD',
	'dbname'=>'wacs',
	'dbuser'=>'wacs',
	'dbpass'=>'wacs'
	);
my( %tablelist )=(
	'models'=>'models',
	'idmap'=>'idmap',
	'assoc'=>'assoc',
	'sets'=>'sets',
	'download'=>'download',
	'photographer'=>'photographer',
	'tag'=>'tag',
	'vendor'=>'vendor',
	'conn'=>'conn',
	'keyword'=>'keyword',
	'user'=>'user',
	'attrib'=>'attrib',
	'notes'=>'notes'
	);
my( %serverlist )= (
	'baseurl'=>'http://localhost/',
	'cgiurl'=>'http://localhost/cgi-bin/',
	'iconurl'=>'http://localhost/glyphs/',
	'siteurl'=>'http://localhost/wacs/',
	'directurl'=>'',
	'wacsurl'=>'http://localhost/wacs/',
	'wacsmain'=>'http://localhost/cgi-bin/wacsfp',
	'wacsallm'=>'http://localhost/cgi-bin/wacsmodelthumbs'
	);
my( %fsloclist )= (
	'images'=>'/home/images',
	'videos'=>'/home/videos',
	'glyphs'=>'/var/www/html/wacs/glyphs',
	'seticons'=>'/var/www/data/wacs/iconcache',
	'officons'=>'/var/www/data/wacs/officons',
	'vidicons'=>'/var/www/data/wacs/vidicons',
	'dvdicons'=>'/var/www/data/wacs/dvdicons',
	'addicons'=>'/var/www/html/wacs/addicons',
	'modicons'=>'/var/www/html/wacs/icons',
	'modbigicons'=>'/var/www/html/wacs/bigicons',
	'thumbcache'=>'/var/www/html/wacs/cache/thumbs',
	'minicache'=>'/var/www/html/wacs/cache/minithumbs',
	'scalecache'=>'/var/www/data/wacs/cache/scaled',
	'download'=>'/home/wacs/download',
	'archive'=>'/home/wacs/archive/zips',
	'leasedir'=>'/var/run/wacs',
	'tooldir'=>'/usr/local/bin',
	'imgtooldir'=>'',
	'cgibindir'=>'/var/www/cgi-bin',
	'htmldir'=>'/var/www/html/wacs',
	'wacsdir'=>'/usr/share/wacs'
	);
my( %seclist )= (
	'authmode'=>'host',
	'barredusers'=>'',
	'powerusers'=>'',
	'adminusers'=>'',
	'admingroup'=>'wacs',
	'leasetime'=>'',
	'leasefile'=>'leases.acl'
	);
my( %dnldlist )= (
	'actithresh'=>'2',
	'dormthresh'=>'4',
	'idlethresh'=>'90',
	'checkcount'=>'10'
	);
my( %colourlist )= (
	'foreground'=>'#000000',
	'background'=>'#FFE8FF',
	'setbackground'=>'#FFB0FF',
	'vidbackground'=>'#FFFFB0',
	'dvdbackground'=>'#B0FFB0',
	'mastheadforeground'=>'#F0D8F0',
	'mastheadbackground'=>'#000040',
	'mastheadhighlight'=>'#F0B8B8',
	'divider'=>'#FFA0A0',
	'indexcolours'=>'Y',
	);
my( %laylist )= (
	'vendormode'=>'N',
	'style'=>'modelattr',
	'thumbsize'=>'220',
	'thumbcount'=>'4',
	'thumbrows'=>'3',
	'modelcount'=>'7',
	'minisize'=>'110',
	'setspergallery'=>'20',
	'units'=>'metric',
	'mpdividers'=>'Y',
	'newperiod'=>'30'
	);
my( %preclist )= ( 
	'default'=>'sets'
	);
my( %extlist )= (
	'imageviewer'=>'gqview',
	'movieplayer'=>'mplayer -fs'
	);
my( %debuglist )= (
	'module_wacs'=>'0',
	'module_wacsid'=>'0',
	'module_wacsui'=>'0',
	'global'=>'0',
	);

my( %iconlist )=(
	'young'=>'icon3-2.png',
	'tinytit'=>'icon3-3.png',
	'nopanties'=>'icon3-4.png',
	'shaven'=>'icon3-5.png',
	'dildo'=>'icon3-6.png',
	'handsoff'=>'icon3-7.png',
	'piercedtit'=>'icon3-8.png',
	'seethru'=>'icon3-9.png',
	'outdoors'=>'icon3-10.png',
	'shorthair'=>'icon3-11.png',
	'fetish'=>'icon3-12.png',
	'nude'=>'icon3-13.png',
	'public'=>'icon3-14.png',
	'anal'=>'icon3-15.png',
	'fuck'=>'icon3-16.png',
	'lesbian'=>'icon3-17.png',
	'country'=>'icon3-18.png',
	'fisting'=>'icon3-19.png',
	'clothed'=>'icon3-20.png',
	'pissing'=>'icon3-21.png',
	'uniform'=>'icon3-22.png',
	'schoolgirl'=>'icon3-23.png',
	'tattoo'=>'icon3-24.png',
	'piercedclit'=>'icon3-25.png',
	'lipring'=>'icon3-26.png',
	'pregnant'=>'icon3-27.png',
	'bondage'=>'icon3-28.png',
	'feet'=>'icon3-29.png',
	'cheerleader'=>'icon3-30.png',
	'transport'=>'icon3-31.png',
	'weapon'=>'icon3-32.png',
	'pantiesfirst'=>'icon3-33.png',
	'blowjob'=>'icon3-34.png',
	'brazilian'=>'icon3-35.png',
	'hairy'=>'icon3-36.png',
	'trimmed'=>'icon3-37.png',
	'strapon'=>'icon3-38.png',
	'pussylick'=>'icon3-39.png',
	'footjob'=>'icon3-40.png',
	'visiblecum'=>'icon3-41.png',
	'boobjob'=>'icon3-42.png',
	'eyebrowring'=>'icon3-43.png'
	);
my( %modelattr )= (
        'young'=>'icon3-2.png',
        'tinytit'=>'icon3-3.png',
        'shaven'=>'icon3-5.png',
        'piercedtit'=>'icon3-8.png',
        'shorthair'=>'icon3-11.png',
        'tattoo'=>'icon3-24.png',
	'piercedclit'=>'icon3-25.png',
	'lipring'=>'icon3-26.png',
	'pregnant'=>'icon3-27.png',
	'brazilian'=>'icon3-35.png',
	'hairy'=>'icon3-36.png',
	'trimmed'=>'icon3-37.png',
	'boobjob'=>'icon3-42.png',
	'eyebrowring'=>'icon3-43.png'
	);
my( %typelist )= (
	'B'=>'Backstage',
	'C'=>'Clothed',
	'F'=>'Straight',
	'G'=>'Group Orgy',
	'I'=>'Interview',
	'L'=>'Lesbian',
	'M'=>'Masturbation',
	'S'=>'Solo',
	'T'=>'Toys',
	'D'=>'Duplicate'
	);
my( %media )= (
	'I'=>'Image Set',
	'V'=>'Video Clip',
	'A'=>'Audio File',
	'S'=>'DVD Scene'
	);
my( %dstatus )= (
	'U'=>'Not Yet Attempted',
	'F'=>'Failed - Retryable',
	'S'=>'Successful - in database',
	'P'=>'Pending - awaiting unpacking',
	'I'=>'In Progress - currently downloading',
	'A'=>'Aborted - unknown reason',
	'D'=>'Deferred - held from downloading',
	'R'=>'Relationship Entry',
	'L'=>'Liason - not yet active R entry',
	'E'=>'Error - not the right model',
	'X'=>'Incomplete - not enough details'
	);
my( %regions )= (
	'Europe'=>'Europe',
	'North America'=>'North America: USA And Canada',
	'South America'=>'South and Central America',
	'Middle East'=>'The Middle East',
	'Asia'=>'Asia (India and the Indian Sub-continent ONLY)',
	'Orient'=>'Orient (excluding Indian Sub-contient)',
	'Australasia'=>'Australia and New Zealand',
	'Africa'=>'Africa',
	'Other'=>'Other'
	);
my( %flaglist )= (
	'P'=>'Placeholder',
	'S'=>'Favourite Solo',
	'L'=>'Favourite Lesbian',
	'C'=>'Favourite Cutie',
	'F'=>'Favourite Straight',
	'M'=>'Current Featured Model'
	);
my( %pussylist )= (
	'S'=>'Shaven',
	'B'=>'Brazilian Shaved',
	'T'=>'Trimmed',
	'H'=>'Hairy',
	'V'=>'Varies'
	);
my( %buildlist )= (
	'V'=>'Very Slim',
	'S'=>'Slim',
	'M'=>'Medium',
	'H'=>'Heavy'
	);
my( %piconlist )= (
	'S'=>'icon1-1.png',
	'B'=>'icon1-2.png',
	'T'=>'icon1-3.png',
	'H'=>'icon1-4.png',
	'V'=>'icon1-5.png'
	);
my( %ticonlist )= (
	'Tiny'=>'icon2-1.png',
	'Small'=>'icon2-2.png',
	'Normal'=>'icon2-3.png',
	'Large'=>'icon2-4.png'
	);
my( %liconlist )= (
	'Short'=>'icon7-1.png',
	'Shoulder'=>'icon7-2.png',
	'Normal'=>'icon7-3.png',
	'Long'=>'icon7-4.png'
	);
my( %hiconlist )= (
	'Blonde'=>'icon8-1.png',
	'Brunette'=>'icon8-2.png',
	'Dark Hair'=>'icon8-3.png',
	'Redhead'=>'icon8-4.png'
	);
my( %biconlist )= (
	'Very Slim'=>'icon9-1.png',
	'Slim'=>'icon9-2.png',
	'Medium'=>'icon9-3.png',
	'Heavy'=>'icon9-4.png'
	);
my( %ciconlist )= (
	'Caucasian'=>'icon10-1.png',
	'Latina'=>'icon10-2.png',
	'Asian'=>'icon10-2.png',
	'Oriental'=>'icon10-3.png',
	'Negroid'=>'icon10-4.png',
	'Mixed'=>'icon10-2.png'
	);


my( @exclude_list )=( '.', '..', '.info', '.exclude', 'spares', 'Spares',
	'.thumbs', '.xvpics', '.faulty','index.jpg','.index','.unpack',
	'.errors','.DS_Store','Thumbs.db' );
my( @index_list )=( 'ndex','ContactSheet','idx','DX' );
my( @exclusions )=( 'Ingridxx' );
my( @monnames )=('Err','Jan','Feb','Mar','Apr','May','Jun','Jul','Aug',
		 'Sep','Oct','Nov','Dec' );


# Set the debug level 
# at this point we set it to the default value specified above - in read_conf
# we reset it as soon as we have the necessary variables imported from XML
my $debug = $debuglist{"module_wacs"};

# MAIN ROUTINES START HERE
# ========================
#
# read_conf
#
# This tries to figure out where the config file is and then reads
# it.  It should always be called early in your code.
sub read_conf
{
	my( $find_config, $find_acl, $find_lease );

	# read configuration files
	$find_config = find_config_location( $main_conf );
	if( -f $find_config )
	{
		# XMLin croaks the program if the file isn't there
		$treehandle = XMLin( $find_config );
		# this should now have read properly so we can now
		# test for the debug config value for this module
		# having changed as the result of a config file setting
		$debug = conf_get_attr("debug","module_wacs");
		if( $debug > 0 )
		{
			print STDERR "readconf: module_wacs debug level ".
					$debug."\n";
		}
	}
	else
	{
		# this debug is controlled by the default settings ONLY
		if( $debug > 0 )
		{
			print STDERR "Did not find config: $main_conf\n";
		}
	}
	# configuration based debug settings should now be active
	if( $debug > 3)
	{
		print STDERR Dumper( $treehandle );
	}

	# read access control list
	$find_acl = find_config_location( $main_auth );
	if( -f $find_acl )
	{
		$authhandle = XMLin( $find_acl );
	}
	else
        {
                if( $debug > 0 )
                {
                        print STDERR "Did not find permanent ACL: $main_auth\n";
                }
        }

	if( $debug > 3)
	{
		print STDERR Dumper( $authhandle );
	}

	# read lease access control list
	if( -d conf_get_attr("fsloc","leasedir") )
	{
		$find_lease = conf_get_attr("fsloc","leasedir")."/".
				conf_get_attr("security","leasefile");
	}
	else
	{
		print STDERR "No directory! ".conf_get_attr("fsloc","leasedir");
		print STDERR "Defaulting to /tmp.";
		$find_lease = "/tmp/".conf_get_attr("security","leasefile");
	}
	if( -f $find_lease )
	{
		$leasehandle = XMLin( $find_lease );
	}
	else
        {
                if( $debug > 0 )
                {
                        print STDERR "Did not find leases: $find_lease\n";
                }
        }

}

#
# check_auth
#
# Checks that this is an authorised access to this system.
sub check_auth( $$ )
{
	my( $ipaddr, $vocal )=@_;
	my( $whennow );

	# ipaddr is the IP address the access is coming from
	# vocal is whether this should issue a warning or not
	# text outputing programs (vocal=1) should get please log in
	# embedded ones (vocal=0) should generate no output or
	# image ones (vocal=2) should give a failure indication
	# as an image (initially fail.png)
	# Special Case vocal=3 - You are already logged out
	if( $ipaddr =~ /\:/ )
	{
		# check for mistakenly labelled as IPv4, prefix with IPv6
		# flag and make XML safe
		$ipaddr =~ s/^ipv4-//;
		$ipaddr = "ipv6-".$ipaddr;
		$ipaddr =~ s/\:/_/g;
	}
	elsif( $ipaddr !~ /^ipv4-/ )
	{
		# prepend an ipv4- tag if not found (for now)
		$ipaddr = "ipv4-".$ipaddr;
	}

	if( auth_get_attr( $ipaddr, "user" ) eq "nobody" )
	{
		if( $debug > 3 )
		{
			print STDERR "auth_get_attr($ipaddr,\"user\") = ";
			print STDERR auth_get_attr( $ipaddr, "user" )."\n";
		}
		# allow a global yes if the owner wishes it by an
		# all setting 
		if( auth_get_attr("ipv4-all","user") ne "nobody" )
		{
			# effectively a permanent yes at default
			# (lowest) access levels
			return;
		}

		# authentication has failed
		if( $vocal == 0 )
		{
			# game over
			print STDERR "Unauth access attempt from $ipaddr\n";
			exit 0;
		}
		# vocal is on
		auth_error( "Authentication Required" );
		print STDERR "Unauth access attempt from $ipaddr\n";
		exit 0;
	}
	# known user
	if( auth_get_attr( $ipaddr, "type" ) eq "permanent" )
	{
		# all OK - access from this address is allowed permanently
		return;
	}
	if( auth_get_attr( $ipaddr, "type" ) eq "lease" )
	{
		# check the lease
		$whennow = time;
		if( auth_get_attr( $ipaddr, "date" ) > $whennow )
		{
			# lease current - all OK
			return;
		}
		else
		{
			# lease expired
			print STDERR "lease expired for $ipaddr\n";
			if( $vocal == 1 )
			{
				auth_error("Sorry, your lease has expired.");
				print STDERR "lease expired $ipaddr\n";
			}
			elsif( $vocal == 2 )
			{
				print "Content-Type: image/png\n";
				print "\n";
				system( "cat ".conf_get_attr("fsloc","glyphs").
					"/fail.png" );
			}
			elsif( $vocal == 3 )
			{
				auth_error("You have already logged out.");
				print STDERR "already logged out $ipaddr\n";
			}
			exit 0;
		}
	}
	print STDERR "Unrecognised type in entry for $ipaddr\n";
	exit -1;
}

#
# auth_error
#
# do a pretty error message
sub auth_error( $ )
{
	my( $message )=@_;

	print "Content-type: text/html\n";
	print "\n";
	print "<html>\n";
	print "<head>\n";
	print "<title>WACS - Authentication Required</title>\n";
	print "</head>\n";
	print "<body>\n";
	print "<h1 align=center><font color=\"#FF0000\">W</font><font ";
        print "color=\"#FFFF00\">A</font><font ";
        print "color=\"#00FF00\">C</font><font ";
        print "color=\"#00FFFF\">S</font></h1>\n";
	print "<hr>\n";
	print "<center><table width=50%><tr><td align=center>\n";
	print "Access to this WACS server is restricted.  In order to ";
	print "be able to access it, you will need appropriate ";
	print "authentication credentials from the web manager of this ";
	print "installation.  If you know these and received this message ";
	print "due to expiry of a lease or by coming from a new location, you ";
	print "can <a href=\"".conf_get_attr("server","cgiurl")."wacslogin\">";
	print "login here</a>.\n";
	print "</td></tr></table></center>\n";
	print "<center>\n";
	print "<table border=1>\n";
	print " <tr><th>Additional Information:</th></tr>\n";
	print " <tr><td align=center>$message</td></tr>\n";
	print "</table>\n";
	print "</center>\n";
	print "<hr>\n";
	print "</body>\n";
	print "</html>\n";
}

#
# auth_user
#
# returns the name of the current user based upon IP address
sub auth_user( $ )
{
	my( $ipaddr )=@_;

	if( $ipaddr =~ /\:/ )
	{
		$ipaddr =~ s/^ipv4-//;
		$ipaddr = "ipv6-".$ipaddr;
		$ipaddr =~ s/\:/_/g;
	}
	elsif( $ipaddr !~ /^ipv4-/ )
	{
		$ipaddr = "ipv4-".$ipaddr;
	}

	# get the registered user for this ip address from
	# the access control list
	return( auth_get_attr($ipaddr,"user") );
}

#
# add_auth
#
# add a new authentication token to the access control list
sub add_auth( % )
{
        my %params = @_;
        my( $ipaddr, $user, $type, $role, $date, $prefexcl, $usedirect );
	my( $imagepage, $scaling, $size, $quality, $delay );
	my( $whennow, $outdir, $outfile, $outxml );

	$whennow = time;
	$outdir = conf_get_attr("fsloc","leasedir");
	if( ! -d $outdir )
	{
		# this we definitely should be able to create as
		# a last resort
		print STDERR "Lease file location specifed does not exist.\n";
		$outdir = "/tmp";
	}
	$outfile = $outdir."/".conf_get_attr("security","leasefile");

	$ipaddr = $params{"ipaddr"};
	$user = $params{"user"};
        $type = $params{"type"};
        $role = $params{"role"};
        $date = $params{"date"};
	$prefexcl = $params{"prefexcl"};
	$usedirect = $params{"usedirect"};
	$imagepage = $params{"imagepage"};
	$scaling = $params{"scaling"};
	$size = $params{"size"};
	$quality = $params{"quality"};
	$delay = $params{"delay"};

	# check and default parameter values
	if( $ipaddr =~ /\:/ )
	{
		$ipaddr =~ s/^ipv4-//;
		$ipaddr = "ipv6-".$ipaddr;
		$ipaddr =~ s/\:/_/g;
	}
	elsif( $ipaddr !~ /^ipv4-/ )
	{
		$ipaddr	= "ipv4-".$ipaddr;
	}
	if( $type eq "" )
	{
		$type = "lease";
	}
	if( $role eq "" )
	{
		$role = "viewer";
	}
	if( $usedirect eq "" )
	{
		$usedirect = "no";
	}
	if( $date eq "" )
	{
		# defaulted lease is one hour - the login application
		# should make the decision on longer leases
		$date = sprintf( "%ld", ($whennow + 3600) );
	}

	# always created
	$leasehandle->{"$ipaddr"}->{"user"} = $user;
	$leasehandle->{"$ipaddr"}->{"type"} = $type;
	$leasehandle->{"$ipaddr"}->{"role"} = $role;
	$leasehandle->{"$ipaddr"}->{"date"} = $date;
	$leasehandle->{"$ipaddr"}->{"prefexcl"} = $prefexcl;
	$leasehandle->{"$ipaddr"}->{"usedirect"} = $usedirect;
	# created if specified
	if( $imagepage ne "" )
	{
		$leasehandle->{"$ipaddr"}->{"imagepage"} = $imagepage;
	}
	if( $scaling ne "" )
	{
		$leasehandle->{"$ipaddr"}->{"scaling"} = $scaling;
	}
	if( $size ne "" )
	{
		$leasehandle->{"$ipaddr"}->{"size"} = $size;
	}
	if( $quality ne "" )
	{
		$leasehandle->{"$ipaddr"}->{"quality"} = $quality;
	}
	if( $delay ne "" )
	{
		$leasehandle->{"$ipaddr"}->{"delay"} = $delay;
	}

	$outxml = XMLout( $leasehandle, 
			  NoAttr=>1, RootName=>'accesslist' );
	open( LEASES, "> $outfile" ) ||
		die( "Can't write XML output to $outfile\n" );
	print LEASES "<?xml version='1.0' standalone='yes'?>\n";

	print LEASES $outxml;
	close( LEASES );
}

# 
# find_config_location
# 
# this routine is passed the name of a configuration file to look for
# and searches for it in the possible locations.
sub find_config_location
{
	my( $try_file )=@_;
	my( $try_config );

	# work out where the config file is
	# first see if the environment variable is set
	if( $ENV{"$default_specifier"} ne "" )
	{
		$try_config = $ENV{"$default_specifier"}.$fssep.$try_file;
		if( ! -f $try_config )
		{
			$try_config='';
		}
	}
	# first default location
	if( $try_config eq "" )
	{
		$try_config = $default_location1.$fssep.$try_file;
		if( ! -f $try_config )
		{
			$try_config='';
		}
	}
	# second default location
        if( $try_config eq "" )
        {
                $try_config = $default_location2.$fssep.$try_file;
                if( ! -f $try_config )
                {
                        $try_config='';
                }
        }
	# third default location
        if( $try_config eq "" )
        {
                $try_config = $default_location3.$fssep.$try_file;
                if( ! -f $try_config )
                {
                        $try_config='';
                }
	}

	# check and return result
	if( -f $try_config )
	{
		if( $debug > 2 )
		{
			print STDERR "wacs: using config file $try_config\n";
		}
		return( $try_config );
	}
	return undef;
}

#
# conf_get_attr
#
# try to provide the requested attribute somehow
sub conf_get_attr( $$ )
{
	my( $req_type, $req_attr )=@_;
	my( $result );

	if( $debug > 4 )
	{
		print STDERR $treehandle->{$req_type}->{"$req_attr"}."\n";
	}
	$result = $treehandle->{$req_type}->{$req_attr};

	# if no result, check built-in default values
	if( $result eq "" )
	{
		if( $debug > 4 )
		{
			print STDERR "Falling back to defaults to service:\n";
			print STDERR "  Type: $req_type\n";
			print STDERR "  Request: $req_attr\n";
		}
		# configuration file requests - note debug is first since
		# it will ALWAYS get called.
		if( $req_type eq "debug" )
		{
			$result = defaults_debug( $req_attr );
		}
		elsif( $req_type eq "database" )
		{
			$result = defaults_database( $req_attr );
		}
		elsif( $req_type eq "tables" )
		{
			$result = defaults_tables( $req_attr );
		}
		elsif( $req_type eq "server" )
		{
			$result = defaults_server( $req_attr );
		}
		elsif( $req_type eq "fsloc" )
		{
			$result = defaults_fsloc( $req_attr );
		}
		elsif( $req_type eq "security" )
		{
			$result = defaults_security( $req_attr );
		}
		elsif( $req_type eq "download" )
		{
			$result = defaults_download( $req_attr );
		}
		elsif( $req_type eq "colours" )
		{
			$result = defaults_colours( $req_attr );
		}
		elsif( $req_type eq "layout" )
		{
			$result = defaults_layout( $req_attr );
		}
		elsif( $req_type eq "precedence" )
		{
			$result = defaults_precedence( $req_attr );
		}
		elsif( $req_type eq "external" )
		{
			$result = defaults_external( $req_attr );
		}
		# embedded type requests
		elsif( $req_type eq "icons" )
		{
			$result = defaults_icons( $req_attr );
		}
		elsif( $req_type eq "modelattr" )
		{
			$result = defaults_modelattr( $req_attr );
		}
		else
		{
			print STDERR "Unknown config category: $req_attr\n";
		}
	}

	if( $result eq "" )
	{
		return undef;
	}
	return( $result );
}

# defaults provider: database
sub defaults_database( $ )
{
	my( $dbreq )=@_;

	return( $dblist{"$dbreq"} );
}

# defaults provider: tables
sub defaults_tables( $ )
{
	my( $tablereq )=@_;

	return( $tablelist{"$tablereq"} );
}

# defaults provider: server
sub defaults_server( $ )
{
	my( $servreq )=@_;

	return( $serverlist{"$servreq"} );
}

# defaults provider: fsloc
sub defaults_fsloc( $ )
{
	my( $fslocreq )=@_;

	return( $fsloclist{"$fslocreq"} );
}

# defaults provider: security
sub defaults_security( $ )
{
	my( $secreq )=@_;

	return( $seclist{"$secreq"} );
}

# defaults provider: download
sub defaults_download( $ )
{
	my( $dnlreq )=@_;

	return( $dnldlist{"$dnlreq"} );
}

# defaults provider: colours
sub defaults_colours( $ )
{
	my( $colourreq )=@_;

	return( $colourlist{"$colourreq"} );
}

# defaults provider: layout
sub defaults_layout( $ )
{
	my( $layreq )=@_;

	return( $laylist{"$layreq"} );
}

# defaults provider: precedence
sub defaults_precedence( $ )
{
	my( $precreq )=@_;

	return( $preclist{"$precreq"} );
}

# defaults provider: external
sub defaults_external( $ )
{
	my( $extreq )=@_;

	return( $extlist{"$extreq"} );
}

# defaults provider: debug
sub defaults_debug( $ )
{
	my( $dbgwhat )=@_;
	my( $ddresult );

	# per module level takes precidence, but failing that check the
	# global debug level
	$ddresult = $debuglist{"$dbgwhat"};
	if( $ddresult eq "" )
	{
		$ddresult = $debuglist{"global"};
		if( $debug > 2 )
		{
			print STDERR "defaults_debug: result for ".
				$dbgwhat." was null - using default global ".
				$ddresult."\n";
		}
	}
	return( $ddresult );
}

# defaults provider: icons
sub defaults_icons( $ )
{
	my( $iconreq )=@_;

	return( $iconlist{"$iconreq"} );
}

# defaults provider: modelattr
sub defaults_modelattr( $ )
{
	my( $modareq )=@_;

	return( $modelattr{"$modareq"} );
}


#
# auth_get_attr
#
# try to provide the requested access control list attribute somehow
sub auth_get_attr( $$ )
{
        my( $req_node, $req_attr )=@_;
        my( $result );

	# first try the permanent authority file
        if( $debug > 4 )
        {
		print STDERR "Permanent: node=$req_node, attr=$req_attr\n";
                print STDERR $authhandle->{$req_node}->{$req_attr}."\n";
        }
        $result = $authhandle->{$req_node}->{$req_attr};

	# didn't get it?  try the leases
	if( $result eq "" )
	{
		if( $debug > 4 )
		{
			print STDERR
				"Leases: node=$req_node, attr=$req_attr\n";
			print STDERR 
				$leasehandle->{$req_node}->{$req_attr}."\n";
		}
		$result = $leasehandle->{$req_node}->{$req_attr};
	}

        # if no result, check built-in default values
        if( $result eq "" )
        {
                if( $debug > 4 )
                {
                        print STDERR "Falling back to defaults to acls:\n";
                        print STDERR "  Node: $req_node\n";
                        print STDERR "  Request: $req_attr\n";
                }
		# make sure it doesn't work for everyone
		if( $req_attr eq "user" )
		{
			$result = "nobody";
		}
		elsif( $req_attr eq "prefexcl" )
		{
			$result = "";
		}
		elsif( $req_attr eq "type" )
		{
			$result = "noaccess";
		}
		elsif( $req_attr eq "role" )
		{
			$result = "viewer";
		}
		elsif( $req_attr eq "date" )
		{
			$result = 1;
		}
		elsif( $req_attr eq "usedirect" )
		{
			$result = "no";
		}
		elsif( $req_attr eq "delay" )
		{
			$result = "3";
		}
		elsif( $req_attr eq "imagepage" )
		{
			$result = "frame";
		}
		elsif( $req_attr eq "quality" )
		{
			$result = "75";
		}
		elsif( $req_attr eq "size" )
		{
			$result = "1024x768";
		}
		elsif( $req_attr eq "scaling" )
		{
			$result = "slide";
		}
		else
		{
                        print STDERR "Unknown auth attribute: $req_attr\n";
                }

	}

	# manage the return
        if( $result eq "" )
        {
                return undef;
        }
        return( $result );
}

# dberror
#
# makes a readable page about a database error
sub dberror( % )
{
	my( %params )=@_;
	my( $deheader, $demessage, $deerror, $dedbname, $dedbuser, $dedbhost );

	$deheader = $params{'header'};
	$demessage = $params{'message'};
	$deerror = $params{'error'};
	$dedbname = $params{'dbname'};
	$dedbuser = $params{'dbuser'};
	$dedbhost = $params{'dbhost'};

	if( $deheader ne "n" )
	{
		print "Content-type: text/html\n";
		print "\n";
		print "<html>\n";
		print "<head>\n";
		print "<title>WACS - Database Connection Problem</title>\n";
		print "</head>\n";
		print "<body>\n";
	}
	print "<h1 align=center><font color=\"#FF0000\">W</font><font ";
        print "color=\"#FFFF00\">A</font><font ";
        print "color=\"#00FF00\">C</font><font ";
        print "color=\"#00FFFF\">S</font></h1>\n";
	print "<hr>\n";
	print "<center><table border=1 width=90%>\n";
	print "<tr><th align=center colspan=2>\n";
	print "Database Connection Problem</th></tr>\n";
	print "<tr><td width=100% colspan=2 bgcolor=\"#D0D040\">";
	print "<img align=left src=\"".conf_get_attr("server","iconurl").
		"connfailure.png\">\n";
	print "<font size=+2>There was a problem making a connection to the ";
	print "database server which is used by this WACS installation ";
	print "to store the information it needs in order to ";
	print "work.  This website is therefore currently unavailable.  ";
	print "Sorry.</font></td></tr>\n";
	print "<tr><th colspan=2>$demessage</th></tr>\n";
	print "<tr><td colspan=2 align=center bgcolor=\"#F8F8D0\">";
	print "<i>If you are a user of this system and ";
	print "saw this message, the details of the failure have been ";
	print "recorded for the system manager to see (this does not ";
	print "include any personal information about you other than your IP ";
	print "address at the time of the incident).<br>";
	print "If they do not attend to it soon, you may wish ";
	print "to contact them via their support channel.<br>";
	print "You do not <b>NEED</b> to do anything more.<br>";
	print "Sorry - please try again later.</i>";
	print "</td></tr>\n"; 
	print "<tr><th colspan=2 bgcolor=\"".conf_get_attr("colours","divider").
		"\">";
	print "If You Are The Owner/Administrator";
	print "</th></tr>\n";
	print "<tr><td align=center colspan=2><font size=-1>";
	print "There are a number of possible causes of this failure, ";
	print "but they fall into two broad categories:</font></td></tr>";
	print "<tr><th width=50% bgcolor=\"#40D040\">Not Yet Configured</th>";
	print "<th width=50% bgcolor=\"#D04040\">Fault or Failure</th></tr>\n";
	print "<tr><td width=50%><font size=-1>";
	print "This is typically the case when WACS has ";
	print "only just been installed and the databases haven't been ";
	print "created and preloaded yet.  If you've just installed WACS ";
	print "using application packages (such as RPM or .deb files), this ";
	print "is normal - you need to complete the installation.<br>\n";
	print "New Installation?  <a href=\"".conf_get_attr("server","cgiurl").
		"/wacssetup\">click here</a> to perform initial setup ".
		"(Administrators Only)<br>\n";
	print "Please see <tt>/usr/share/wacs/README.database</tt>";
	print " for instructions on how to complete this process, or read ";
	print "the full installation guide.</font></td>\n";
	print "<td width=50%><font size=-1>";
	print "To assist you with finding the fault, please see ";
	print "the error messages in your web server error log file (typically";
	print " <tt>/var/log/httpd/errorlog</tt>) for more information. ";
	print "The entries you are looking for will be prefixed with the ";
	print "text <tt>dberror</tt>.  For more information, please see the ";
	print "file <tt>/usr/share/wacs/README.faults</tt>.</font></td></tr>\n";
	print "</table></center>\n";
	print STDERR "dberror: errstr=".$deerror."\n";
	print STDERR "dberror: dbuser=".$dedbuser."\n";
	print STDERR "dberror: dbhost=".$dedbhost."\n";
	print "<hr>\n";
	print "</body>\n";
	print "</html>\n";
	exit 0;
}

# gettoday
# 
# return todays date as a suitable string
sub gettoday( % )
{
	my %params = @_;
	my( $whennow,$rawdate,$fordate,$lwday,$lmon,$lmday,$ltime,$lyear );
	my( $format, $epoch, $offset );
	my( $datefmt, $monchk );

	$format = $params{"format"};
	$epoch = $params{"epoch"};
	$offset = $params{"offset"};

	# see if a format has been specified and only copy valid ones
	if( $format eq "DD-MON-YYYY" )
	{
		$datefmt = "DD-MON-YYYY";
	}
	elsif( $format eq "YYYY-MM-DD" )
	{
		$datefmt = "YYYY-MM-DD";
	}
	else
	{
		# no - get the wacs config value
		$datefmt = conf_get_attr("database","dateformat");
	}

	# work out the time here
	$whennow = time;
	if( $epoch ne "" )
	{
		# time was specified in Unix epoch format
		$whennow = $epoch;
	}
	$rawdate = scalar localtime( $whennow );
	if( $offset ne "" )
	{
		$rawdate = scalar localtime( ($whennow - (86400*$offset)) );
	}
	($lwday,$lmon,$lmday,$ltime,$lyear)=split( /\s+/, $rawdate, 5);

	# what date format is requested?
	if( $datefmt eq "DD-MON-YYYY" )
	{
		# Oracle style
		$fordate = $lmday . "-" . uc($lmon) . "-" . $lyear;
	}
	elsif( $datefmt eq "YYYY-MM-DD" )
	{
		# MySQL style
		for( $monchk=1; $monchk < 13 ; $monchk++ )
		{
			if( lc($monnames[$monchk]) eq lc( $lmon ))
			{
				$fordate = $lyear."-".$monchk."-".$lmday;
				break;
			}
		}
	}
	else
	{
		# unknown, report error and return ctime-style
		print STDERR "Unknown date format $datefmt.\n";
		$fordate = $rawdate;  # returning ctime style
	}

	return( $fordate );
}

# timecomps
#
# Date conversion routine - takes a date as given by the database
# and converts it to three date components suitable for further manipulation
sub timecomps( $;$ )
{
	my( $unconv, $specdfmt )=@_;
	my( $datefmt );
	my( $year, $month, $mname, $day );
	# NOTE: these are NOT zero-offset month numbers as used by the
	# perl date component tools - a $month-- is required to make it
	# compatible.
	my( %montonum )=(
		'err'=>'0',
		'jan'=>'1',
		'feb'=>'2',
		'mar'=>'3',
		'apr'=>'4',
		'may'=>'5',
		'jun'=>'6',
		'jul'=>'7',
		'aug'=>'8',
		'sep'=>'9',
		'oct'=>'10',
		'nov'=>'11',
		'dec'=>'12' );

	# sanity check first
	if( $unconv eq "" )
	{
		return undef;
	}
	# work out the input date format
	$datefmt = conf_get_attr("database","dateformat");
	if( $specdfmt ne "" )
	{
		# a different one was specified from the default for this
		# database type - probably for import conversions, etc.
		$datefmt = $specdfmt;
		if( $debug > 0 )
		{
			print STDERR "timecomps: input parser set to ".
					$specdfmt."\n";
		}
	}

	if( $datefmt eq "DD-MON-YYYY" )
	{
		( $day, $mname, $year ) = split( /-/, $unconv, 3 );
		$month = $montonum{lc( $mname )};
	}
	elsif( $datefmt eq "YYYY-MM-DD" )
	{
		( $year, $month, $day ) = split( /-/, $unconv, 3 );
	}
	else
	{
		# unknown, report error and return ctime-style
		print STDERR "Unknown date format $datefmt.\n";
		return undef;
	}

	if( $debug > 0 )
	{
		print STDERR 
		"timecomps: $unconv = day: $day month: $month year: $year\n";
	}
	return( $day, $month, $year );
}

#
# vendlink
#
# provide (if possible) a link to the vendor's site for this model
sub vendlink( % )
{
	my( %params ) = @_;
	my( $vendor, $name, $vpage, $key, $alt, $setkey, $sessionkey );
	my( $modelno, $setno, $vdbhandle );
	my( $vsql, $vcsr, @vrec, $vurl );

	$vendor = $params{'vendor'};
	$vpage = $params{'page'};
	$name = $params{'name'};
	$key = $params{'key'};
	$alt = $params{'alt'};
	$setkey = $params{'setkey'};
	$sessionkey = $params{'sessionkey'};  # for SPARTA sites
	$modelno = $params{'modelno'};    # allow use of Wacs IDs
	$setno = $params{'setno'};	  # allow use of Wacs Set No
	$vdbhandle = $params{'dbhandle'};

	# these parameters are mandatory
	if( $vendor eq "" )
	{
		return undef;
	}
	
	#                0      1           2         3            4
	$vsql = "select vsite, vshortname, vcurrent, vmdirectory, vmdiruse, ";
	#         5       6         7      8         9         10
	$vsql.= "vmpage, vmpaguse, vmbio, vmbiouse, vmvideos, vmviduse, ";
	#         11        12       13        14       15        16
	$vsql.= "vvidpage, vviduse, vimgpage, vimguse, valtpage, valtuse, ";
	#         17      18        19
	$vsql.= "vsrvimg, vsrvvid, vsignup ";
	$vsql.= "from ".conf_get_attr("tables","vendor")." ";
	$vsql.= "where vsite = '".$vendor."' ";
	
	if( $debug > 2 )
	{
		print STDERR "vendlink: SQL Query is: $vsql";
	}
	$vcsr=$vdbhandle->prepare( $vsql );
	$vcsr->execute;
	if( ! (@vrec = $vcsr->fetchrow_array ) )
	{
		return undef;
	}
	if( $vrec[2] eq "N" )
	{
		return undef;
	}
	
	if( $vpage eq "directory" )
	{
		$vurl = $vrec[3];
	}
	if( $vpage eq "modelpage" )
	{
		$vurl = $vrec[5];
	}
	if( $vpage eq "bio" )
	{
		$vurl = $vrec[7];
	}
	if( $vpage eq "vidindex" )
	{
		$vurl = $vrec[9];
	}
	if( $vpage eq "vidpage" )
	{
		$vurl = $vrec[11];
	}
	if( $vpage eq "imgpage" )
	{
		$vurl = $vrec[13];
	}
	if( $vpage eq "altpage" )
	{
		$vurl = $vrec[15];
	}
	if( $vpage eq "signup" )
	{
		$vurl = $vrec[19];
	}

	$vurl =~ s/#NAME#/$name/g;
	$vurl =~ s/#KEY#/$key/g;
	$vurl =~ s/#ALT#/$alt/g;
	$vurl =~ s/#SETKEY#/$setkey/g;
	$vurl =~ s/#SESSIONKEY#/$session/g;
	# OK, the next two are a bit out there... Zombies Rock!
	$vurl =~ s/#MODELNO#/$modelno/g;
	$vurl =~ s/#SETNO#/$setno/g;

	if( $debug > 0 )
	{
		print STDERR "vendlink: Modified URL is: $vurl\n";
	}
	return( $vurl );
}

#
# getvaluename
#
# look up the long name for the specified value of specified type
sub getvaluename( % )
{
	my( %params )=@_;
	my( $gvnresult, $gvnobject, $gvnvalue, %gvnlist );

	$gvnobject=$params{'object'};
	$gvnvalue=$params{'value'};

	%gvnlist = geticonlist( $gvnobject );
	$gvnresult = $gvnlist{$gvnvalue};
	if( $gvnresult eq "" )
	{
		$gvnresult = "Unknown (".$gvnvalue.")";
	}
	return( $gvnresult );
}

#
# geticonlist
#
# return the list of icons for those who need it, now a more generalised
# way of returning a hash of attributes -> description mappings
sub geticonlist( $ )
{
	my( $whatfor )=@_;

	if( $whatfor eq "models" )
	{
		return( %modelattr );
	}
	if( $whatfor eq "sets" )
	{
		return( %iconlist );
	}
	if( $whatfor eq "types" )
	{
		return( %typelist );
	}
	if( $whatfor eq "media" )
	{
		return( %media );
	}
	if( $whatfor eq "dstatus" )
	{
		return( %dstatus );
	}
	if( $whatfor eq "regions" )
	{
		return( %regions );
	}
	if( $whatfor eq "flags" )
	{
		return( %flaglist );
	}
	if( $whatfor eq "pussy" )
	{
		return( %pussylist );
	}
	if( $whatfor eq "build" )
	{
		return( %buildlist );
	}
	# word/value to icon mappings
	# 1. pussy shave
	if( $whatfor eq "picon" )
	{
		return( %piconlist );
	}
	# 2. tit size
	if( $whatfor eq "ticon" )
	{
		return( %ticonlist );
	}
	# 3. hair length
	if( $whatfor eq "licon" )
	{
		return( %liconlist );
	}
	# 4. hair colour
	if( $whatfor eq "hicon" )
	{
		return( %hiconlist );
	}
	# 5. build
	if( $whatfor eq "bicon" )
	{
		return( %biconlist );
	}
	# 6. complexion
	if( $whatfor eq "cicon" )
	{
		return( %ciconlist );
	}
	return undef;
}

#
# gettypecolour
#
# return background colours for display tools
sub gettypecolour( $ )
{
        my( $settype )=@_;

        if( $settype eq "I" )
        {
                return( conf_get_attr("colours","setbackground") );
        }
        if( $settype eq "V" )
        {
                return( conf_get_attr("colours","vidbackground") );
        }
	# add DVD and Audio file support here
        return( "#000000" );
}

#
# divideup
#
# make a directory name more readable
sub divideup( $$$ )
{
        my( $source, $divider, $alreadysmall )=@_;
        my( $dispname, $builddisp, $namepart );

        # install proto dividers and hide remaining spaces, then remove proto
	# both space and hyphen are proto dividers
        $source =~ s/\s/[/g;
	$source =~ s/\-/]/g;
        $source =~ s/ ([A-Za-z]*)$/]$1/;
        $source =~ s/ /+/g;
        $source =~ s/\[/ /g;
        $source =~ s/\]/-/g;

	# add a divider before hyphenated part as well
	$source =~ s/\-([A-Za-z0-9\s]*)$/$divider\-+<i>$1<\/i>/g;

        # check for overlength
        foreach $namepart ( split( /\s/, $source) )
        {
                if( length( $namepart ) > 40 )
                {
                        if( $alreadysmall == 0 )
                        {
                                $dispname .=
                                   "$divider<font~size=-1>$namepart</font>";
                        }
			elsif( $alreadysmall == -1 )
			{
				# don't resize however long
				$dispname .= $divider.$namepart;
			}
                        else
                        {
                                # small is at -2 already
                                $dispname .=
                                   "$divider<font~size=-3>$namepart</font>";
                        }
                }
                else
                {
                        $dispname .= "$divider$namepart";
                }
        }

        # only at start and end - mark where divider will go
        $dispname =~ s/\s$//;
        $dispname =~ s/^$divider//;
        $dispname =~ s/\+/ /g;

        # change holder to divider
        $dispname =~ s/\~/ /;

	# prettifier from GKK
	$dispname =~ s,(\w)([A-Z][a-z]),$1 $2,g;
	$dispname =~ s,([a-z])([A-Z0-9]),$1 $2,g;

        return( $dispname );
}

#
# checkexclude
#
# look for "magic" file and directory names that we ignore
sub checkexclude( $ )
{
        my( $about )=@_;
        my( $exc_count );

        $exc_count = 0;
        while( $exclude_list[$exc_count] ne "" )
        {
                if( $about eq $exclude_list[$exc_count] )
                {
                        # it's in the exclude list so return true
                        return 1;
                }
                $exc_count++;
        }
        return 0;
}

#
# checkindex
#
# take a guess at what might be an index file
sub checkindex( $ ) 
{
	my( $about )=@_;
        my( $index_count );

        # first loop through the exclusions list
        $index_count = 0;
        $_ = $about;
        while( $exclusions[$index_count] ne "" )
        {
                if( /$exclusions[$index_count]/ )
                {
                        # it's a known false positive for index test
                        return 0;
                }
                $index_count++;
        }

        $index_count = 0;
        while( $index_list[$index_count] ne "" )
        {
                if( /$index_list[$index_count]/ )
                {
                        # it matches the index scan
                        return 1;
                }
                $index_count++;
        }
        return 0;
}

# makedbsafe
# 
# remove characters likely to upset the database routines
sub makedbsafe( % )
{
	my( %params )=@_;
	my( $mdsstring, $mdsallow );
	$mdsstring=$params{'string'};
	$mdsallow=$params{'allow'};
	$mdsdeny=$params{'deny'};

	# sanity check
	if( $mdsstring eq "" )
	{
		return( "" );
	}

	# mandatory changes
	$mdsstring =~ s/'/`/g;		# ' becomes back quote `
	$mdsstring =~ s/\<//g;		# remove HTML tags
	$mdsstring =~ s/\>//g;		#   "     "    "
	if( ! $mdsallow =~ /\// )
	{
		$mdsstring =~ s/\///g;	# remove slashes unless explicitly 
					# allowed
	}
	if( $mdsdeny =~ /\s/ )
	{
		$mdsstring =~ s/\s//g;   # if any space character is denied
					# remove any of them
	}

	return( $mdsstring );
}

#
# addheadercss
#
# add standard or javascript menu css to page
sub addheadercss( $ )
{
	my( $cssmode )=@_;
	
	# use these layouts in all menu types
	print "<style type=\"text/css\">\n";
	print "th {font-size: 11pt; font-family: Helvetica, Arial, ";
	print "san-serif; color: navy; font-weight: bold }\n";
	print "td {font-size: 9.5pt; font-family: Helvetica, Arial, ";
	print "san-serif; color: black }\n";
	print "b {font-size: 12pt; font-family: Helvetica, Arial, ";
	print "san-serif; color: black }\n";
	print "a {font-size: 9pt; font-family: Helvetica, Arial, ";
	print "san-serif; color: black; font-weight: bold }\n";
	print "a:link { text-decoration: none }\n";
	print "a:visited { text-decoration: none }\n";
	print "</style>\n";

	if( $cssmode eq "csshoriz" )
	{
		print "<link rel=\"stylesheet\" href=\"";
		print conf_get_attr("server","wacsurl")."styles/";
		print "csshorizontalmenu.css\" />\n";
		print "\n";
		print "<script type=\"text/javascript\" src=\"";
		print conf_get_attr("server","wacsurl");
		print "csshorizontalmenu.js\">\n";
		print "\n";
		print "/***********************************************\n";
		print " * CSS Horizontal List Menu- by JavaScript Kit ";
		print "(www.javascriptkit.com)\n";
		print " * Menu interface credits: http://www.dynamicdrive.com";
		print "/style/csslibrary/item/glossy-vertical-menu/\n";
		print " * This notice must stay intact for usage\n";
		print " * Visit JavaScript Kit at http://www.javascriptkit";
		print ".com for this script and 100s more\n";
		print " ***********************************************/\n";
		print "</script>\n";
	}
	else
	{
		print "<!--- unrecognised CSS request in addheadercss --->\n";
	}
}

#
# setgroupperms
#
sub setgroupperms( % )
{
	my( %params )=@_;
	my( $sgptarget, $sgpgroup, $sgpmode );

	$sgptarget = $params{'target'};
	$sgpgroup = $params{'group'};
	$sgpmode = $params{'mode'};

	# FIXME: check what the permissions currently are here
	system( "chmod ".$sgpmode." ".$sgptarget );

	# if we have no group specified, setting the mode was all we 
	# needed to do...
	if( $sgpgroup eq "" )
	{
		return 1;
	}

	# set the group permissions too.
	system( "chgrp ".$sgpgroup." ".$sgptarget );
	if( -d $sgptarget )
	{
		system( "chmod g+s ".$sgptarget );
	}
	return 1;
}

#
# treemkdir
#
sub treemkdir( % )
{
	my( %params )=@_;
	my( $tmdorigin, $tmdpath, $tmdnextdir );
	my( @tmdpathcomp, $tmdpathsofar );

	$tmdorigin = $params{'origin'};
	$tmdpath = $params{'path'};
	if( $debug > 3 )
	{
		print STDERR "treemkdir: making ".$tmdpath.
			" starting from ".$tmdorigin."\n";
	}

	$tmdpathsofar = $tmdorigin."/";
	@tmdpathcomp = split( /\//, $tmdpath );

	foreach $tmdnextdir ( @tmdpathcomp )
	{
		if( ! -d $tmdpathsofar."/".$tmdnextdir )
		{
			if( mkdir( $tmdpathsofar."/".$tmdnextdir ) == 1 )
			{
				setgroupperms(
					target=>$tmdpathsofar."/".$tmdnextdir,
					mode=>770,
					group=>conf_get_attr("security",
						"admingroup") );
			}
			else
			{
				print STDERR "treemkdir: failed to mkdir ".
					$tmdnextdir." at ".
					$tmdpathsofar."\n";
				return -1;
			}
		}
		$tmdpathsofar .= "/".$tmdnextdir;
	}
	return( 0 );
}

# show that we have parsed correctly
1;
__END__

=head1 NAME

Wacs - Web Adult Content Server - Core Services Module

=head1 NOTICE

THIS PROJECT DOES NOT CONTAIN ANY ADULT MATERIAL.  Rather it is a 
toolkit for presenting and managing your own collections by enabling
you to access those collections in a way similar to that provided by
subscription web sites.  Hopefully it contains many of the necessary
tools to create a good-looking site with best-of-breed facilities.

The idea of the project is to make a system for personal use, but Wacs
is quite capable of hosting an internet accessible site.  Before doing
so however, the author strongly recommends you consider the legal and
copyright infringement issues that could arise out of such an action. 
Those of you in the USA definitely need to consider the implications of
18 USC 2257 carefully.

=head1 SYNOPSIS

read_conf() - locate and read the XML based configuration file
conf_get_attr() - get the requested configuration attribute
auth_get_attr() - get the requested authentication attribute
check_auth() - check that this is an authorised access
auth_user() - return the registered username for this IP
auth_error() - give an authentication error message
add_auth() - add a new auth entry to access control list
gettoday() - return todays date in a format suitable for the database
timecomps() - covert a database format time into three parts
vendlink() - link back to the original vendors site for specified item
geticonlist() - gets the icon array for the specified object type
gettypecolour() - return background to use for this media type
divideup() - make directory name more readable
alsofeaturing() - find and list any other models featured in this set
checkexclude() - check for this file name being one to ignore/hide
checkindex() - guess whether this filename is an index file or not
makedbsafe() - remove characters likely to cause the database trouble
setgroupperms() - make sure the file ownerships are set right
treemkdir() - do a recursive mkdir of a tree of directories

=head1 DESCRIPTION

WACS is a collection of perl scripts and modules that creates a powerful
and flexible multimedia content management and presentation system tuned
to the specific requirements of adult content.  It aims to provide search,
indexing and presentation facilities very similar to those found on the
best of breed of quality adult web sites.  WACS may also be a very useful
starting point for those wishing to build their own adult web sites.  WACS
expects to run on a suitable server machine, and is intended to serve content
to other machines on the local home network such as desktop PCs, notebooks
and Media Centre PCs.  For those with a permanent broadband internet
connection, it could also potentially allow authenticated access to your
collection from remote locations such as work PCs or hotel internet
connections.  Useful if you want to look at your collection while on a 
business trip without leaving it on a corporate laptop, for instance.

WACS is very much still a work in progress and some programming and system
admin skills will be necessary to make use of it at this time.  The system
was developed on Linux for AMD64 (x86_64) with the Apache Web Server, Perl
5.8 and Oracle 10g as the database back end.  Some consideration has been
given to allowing use of an alternative database backend such as MySQL, but
the support may still be incomplete.

Almost all aspects of the WACS system are based upon an extensive database
back end which holds the metadata used by the search and presentation 
systems.  Much of the metadata is derived automatically from either
information gleaned from the source web site via the download mechanism or
from a slightly stylised naming convention for directories.  For example,
if the word "Garden" or "Tree" is found in the directory name, the "outdoors"
flag will be added to the metadata for that set.  Similarly, if the phrase
"Schoolgirl" is found in the directory name, that flag will be applied to
the set.  This can be manually overriden of course, but generally it works
well and is reasonably quick to use when importing new sets. 

=head1 SEE ALSO

wacs.cfg, wacs.acl, menu.cfg, http://wacsip.sourceforge.net/

=head1 LICENSE

Wacs is released under the Gnu Public License Version 2 at this time.
It is expected to migrate to Version 3 of the license when that is ratified.

=head1 AUTHORS

B King
=cut
