#
#    WACSID.PM - identification routine modules
#
#    (C) Copyright 2006,2007,2008,2009 B King
#    This file is part of WACS.
#
#    WACS is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 3 of the License, or
#    (at your option) any later version.
#
#    WACS is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with WACS.  If not, see <http://www.gnu.org/licenses/>.
#

package Wacs::WacsId;

require Exporter;

use HTML::Parser ();
use File::Basename;
use Data::Dumper;

@ISA		= qw(Exporter);
@EXPORT		= qw(ident_img ident_vid id_get_flag id_get_info
		     id_get_photog id_get_dnldno id_get_modelno id_get_modelid
		     id_get_modelname id_get_vendor id_get_dbhandle
		     id_get_key id_get_setkey id_get_setname id_get_status
		     id_get_notes id_get_setno reset_attr
		     dnld_img dnld_markdone dnld_checkadd dnld_update
		     vend_dnld id_mpage id_i_fetch id_gen_proto2struct
		     id_gen_tokenmatchstring id_gen_trymatch id_vid_trymatch
		     id_gen_getvarval chk_vid_type vid_getsize
		     find_cookies find_namestem extractphotog
		     chkid_existing );

# ident values
$id_catinfo="";
$id_catflag="";
$id_photog="";
$id_dnldno=0;
$id_modelno=0;
$id_modelid='';
$id_modelname='';
$id_vendor="";
$id_dbhandle='';
$id_key='';
$id_setkey='';
$id_setname='';
$id_status='';
$id_notes='';
$id_setno=0;

# global variables
$vfileloc='';		# vendor download file location
$cmdline='';		# wget command for getting files we don't have
$vfileurl='';		# location for video files to be downloaded from
$fv_catchnext=0;	# inside an interesting section (usually a table)
$fo_result='';		# the object we found in id_gen_findobj
$fo_type='';		# the type to look for in id_gen_findobj
$fo_multiple=0;		# are we expecting multiple results?
$fo_count=0;		# number of matching files
@fo_names=();		# array of the names found

# identification types 
my(%id_i_list)=(
	'1'=>"Standard Reverse Substitution from Vendor entry",
	'2'=>"ATKingdom Corporate aaa000BBB_000000 style",
	'3'=>"ATK Premium variant aaa000BBB_000000 style",
	'4'=>"Name and image number, TF/AW style",
	'5'=>"Name and image number, KPC style",
	'6'=>"Standard Parser, catch all downloading, KHA style"
	);
my(%id_v_list)=(
	'1'=>"Standard Reverse Substitution from Vendor entry",
	'2'=>"ATKingdom Corporate aaa000BBB_000000 style",
	'3'=>"ATK Premium variant aaa000BBB_000000 style",
	'4'=>"Name and Video Clip Number, TW/AW style",
	'5'=>"Name and Video Clip Number, KPC/KHA style"
	);
my(%dlnd_i_list)=(
	'1'=>"Eighteen Character Padded With Xs, SE Style",
	'2'=>"ATKingdom Corporate aaa000BBB_000000 style",
	'3'=>"Three part numeric, IFG Style",
	'4'=>"Name and image number, TF/AW Style",
	'5'=>"Name, hypen, set number, hyphen; KHA Style"
	);

# Set the debug level from the config files (MOSTLY THIS DOESN'T WORK!)
# if you really care, include this at the top of the entry point routine
$debug=Wacs::conf_get_attr("debug","module_wacsid");

# MAIN ROUTINES START HERE
# ========================
#
# ident_img
#
# tries to identify an image set's contents from it's name and other info
sub ident_img( % )
{
	my %params = @_;
	my( $idtype, $vendor, $fname, $dnotes, $dsetname, $dbphotog );
	my( $ddbhandle );
	my( $vendidsql, $vendidcsr, @vendidrec );

	$idtype = $params{'idtype'};
	$vendor = $params{'vendor'};
	$fname = $params{'fname'};
	$dnotes = $params{'dnotes'};
	$dsetname = $params{'dsetname'};
	$dbphotog = $params{'dbphotog'};
	$ddbhandle = $params{'dbhandle'};

	# reset attributes
	reset_attr( 'images' );
	if( $idtype < 1 && $vendor ne "" )
	{
		#                     0      1      2           3
		$vendidsql = "select vsite, vname, vshortname, vidtimg, ";
		#              4
		$vendidsql.= "vidtvid ";
		$vendidsql.= "from ".Wacs::conf_get_attr("tables","vendor")." ";
		$vendidsql.= "where vsite = ? ";
		$vendidcsr=$ddbhandle->prepare($vendidsql);
		$vendidcsr->execute( $vendor );
		if( @vendidrec = $vendidcsr->fetchrow_array )
		{
			$idtype = $vendidrec[3];
		}
	}

	# call appropriate routines
	if( $debug > 0 )
	{
		print STDERR "$idtype: ".$id_i_list{$idtype}."\n";
	}
}

# ident_vid
#
# tries to identify a video's contents from it's name.
sub ident_vid( % )
{
	my %params = @_;
	my( $idtype, $vendor, $fname, $dnotes, $dsetname, $dbphotog );
	my( $dsetflag, $ddbhandle );
	my( $vendidsql, $vendidcsr, @vendidrec );

	$idtype = $params{'idtype'};
	$vendor = $params{'vendor'};
	$fname = $params{'fname'};
	$dnotes = $params{'dnotes'};
	$dsetname = $params{'dsetname'};
	$dsetflag = $params{'dsetflag'};
	$dbphotog = $params{'dbphotog'};
	$ddbhandle = $params{'dbhandle'};

	# reset attributes
	reset_attr( 'video' );
	if( $idtype < 1 && $vendor ne "" )
	{
		#                     0      1      2           3
		$vendidsql = "select vsite, vname, vshortname, vidtimg, ";
		#              4
		$vendidsql.= "vidtvid ";
		$vendidsql.= "from ".Wacs::conf_get_attr("tables","vendor")." ";
		$vendidsql.= "where vsite = ? ";
		$vendidcsr=$ddbhandle->prepare($vendidsql);
		$vendidcsr->execute( $vendor );
		if( @vendidrec = $vendidcsr->fetchrow_array )
		{
			$idtype = $vendidrec[4];
		}
	}

	# call appropriate routines
	if( $debug > 0 )
	{
		print STDERR "$idtype: ".$id_v_list{$idtype}."\n";
	}
	if( $idtype == 1 )
	{
		id_v_simple( $vendor, $fname, $dnotes, $dsetflag );
	}
	if( $idtype == 2 )
	{
		id_v_atkg( $vendor, $fname, $dnotes, $dsetname, $dbphotog,
			  $dsetflag, $ddbhandle );
	}
	if( $idtype == 3 )
	{
		id_v_atkp( $vendor, $fname, $dnotes, $dsetname, $dbphotog,
			  $dsetflag, $ddbhandle );
	}
	if( $idtype == 4 )
	{
		# FIXME: pick out keywords for TeenFlood/AW videos
	}
	if( $idtype == 5 )
	{
		# FIXME: pick out keywords for Karup's HA videos
	}
}

#
# id_v_simple
#
# very simple identification (for videos)
sub id_v_simple( $$$$ )
{
	my( $svendor, $sfname, $sdnotes, $sdsetflag )=@_;

	# check to see if the download records flag specifies
	# any thing different from the defaults - if it does,
	# go with that
	if( $sdsetflag ne "" && $sdsetflag ne "S" )
	{
		$id_catinfo = $sdsetflag;
	}
	else
	{
		# use older techniques
		std_notes( $sdnotes );
		if( $svendor eq "SE" )
		{
			$id_catinfo .= "lesbian ";
			$id_catflag = "L";
		}
	}
}

#
# std_notes
#
# Standard Notes Passing
sub std_notes( $ )
{
	my( $notes )=@_;

	# standard note passing from the download dnotes field
	if( $notes eq "Catflag: B" )
	{
		$id_catflag = "B";
	}
	if( $notes eq "Catflag: F" )
	{
		$id_catinfo .= "fuck ";
		$id_catflag = "F";
	}
	if( $notes eq "Catflag: L" )
	{
		$id_catinfo .= "lesbian ";
		$id_catflag = "L";
	}
	if( $notes eq "Catflag: M" )
	{
		$id_catflag = "M";
	}
	if( $notes eq "Catflag: T" )
	{
		$id_catinfo .= "dildo";
		$id_catflag = "T";
	}
	if( $notes eq "Catflag: G" )
	{
		$id_catinfo .= "fuck ";
		$id_catflag = "G";
	}
	if( $notes eq "Catflag: S" )
	{
		$id_catflag = "S";
	}
	if( $notes eq "Catflag: C" )
	{
		$id_catflag = "C";
		$id_catinfo .= "clothed ";
	}
}

#
# id_v_atkg
#
# ATKingdom group companies
sub id_v_atkg( $$$$$$$ )
{
	my( $svendor, $sfname, $sdnotes, $sdsetname, $sdbphot, ,$sdsetflag, 
		$sdbhandle )=@_;

	# check to see if the download records flag specifies
	# any thing different from the defaults - if it does,
	# go with that
	if( $sdsetflag ne "" && $sdsetflag ne "S" )
	{
		$id_catflag = $sdsetflag;
	}
	else
	{
		std_notes( $sdnotes );
	}

	if( $debug > 0 )
	{
		print STDERR "id_v_atkg: catinfo=[$id_catinfo]\n";
	}

	# try to get the photographer name - check for expected format
	# NB: perl regex black magic here - bracket() the search expression
	# and it becomes a capture buffer named \1 within the search string
	# HOWEVER it also becomes $1 referenced outside the search context
	# $1 does not get reset until this scope ends, but then it is reset
	if( $sfname =~ /([a-z][a-z][a-z][0-9][0-9][0-9][A-Z][A-Z][A-Z])/ )
	{
		$id_photog= extractphotog( $1, $sdbphot, $sdbhandle );
		if( $debug > 1 )
		{
			print STDERR "photog $id_photog from $sfname\n";
		}
	}
}

# id_v_atkp
#
# special version for ATK Premium which is significantly different from
# the rest of the ATK group companies.
sub id_v_atkp( $$$$$$$ )
{
	my( $svendor, $sfname, $sdnotes, $sdsetname, $sdbphot, $sdsetflag,
		$sdbhandle )=@_;
	my( $updnotes );

	# if there are notes, read them; if not update them
	$updnotes = 0;
	if( $sdnotes eq "" )
	{
		$updnotes = 1;
	}
	else
	{
		if( $sdsetflag ne "" & $sdsetflag ne "S" )
		{
			$id_catflag = $sdsetflag;
		}
		else
		{
			std_notes( $sdnotes );
		}
	}

	# only ATKPremium seem to include type attributes in the video
	# file names
	if( $sfname =~ /[0-9]hc[a-z][0-9]/ )
	{
		$id_catflag = "F";
		$id_catinfo .= "fuck ";
		if( $updnotes == 1 )
		{
			$id_notes = "Catflag: F";
		}
		if( $debug > 1 )
		{
			print "id_v_atkp: type: Fuck\n";
		}
	}
	if( $sfname =~ /[0-9]gp[a-z][0-9]/ )
	{
		$id_catflag = "G";
		$id_catinfo .= "fuck ";
		if( $updnotes == 1 )
		{
			$id_notes = "Catflag: G";
		}
		if( $debug > 1 )
		{
			print "id_v_atkp: type: Fuck (Group Orgy)\n";
		}
	}
	if( $sfname =~ /[0-9]ls[a-z][0-9]/ )
	{
		$id_catflag = "L";
		$id_catinfo .= "lesbian ";
		if( $updnotes == 1 )
		{
			$id_notes = "Catflag: L";
		}
		if( $debug > 1 )
		{
			print "id_v_atkp: type: Lesbian\n";
		}
	}
	if( $sfname =~ /[0-9]ms[a-z][0-9]/ )
	{
		$id_catflag = "M";
		if( $updnotes == 1 )
		{
			$id_notes = "Catflag: M";
		}
		if( $debug > 1 )
		{
			print "$id_v_atkp: type: Masturbation\n";
		}
	}
	if( $sfname =~ /[0-9]ty[a-z][0-9]/ )
	{
		$id_catflag = "T";
		$id_catinfo .= "dildo ";
		if( $updnotes == 1 )
		{
			$id_notes = "Catflag: T";
		}
		if( $debug > 1 )
		{
			print "id_v_atkp: type: Toy\n";
		}
	}
	if( $debug > 0 )
	{
		print STDERR "id_v_atkp: catinfo=[$id_catinfo]\n";
	}

	# try to get the photographer name - check for expected format
	# NB: perl regex black magic here - bracket() the search expression
	# and it becomes a capture buffer named \1 within the search string
	# HOWEVER it also becomes $1 referenced outside the search context
	# $1 does not get reset until this scope ends, but then it is reset
	if( $sfname =~ /([a-z][a-z][a-z][0-9][0-9][0-9][A-Z][A-Z][A-Z])/ )
	{
		$id_photog= extractphotog( $1, $sdbphot, $sdbhandle );
		if( $debug > 1 )
		{
			print STDERR "photog $id_photog from $sfname\n";
		}
	}
}

#
# reset_attr
#
#
sub reset_attr( $ )
{
	my( $forwhat )=@_;

	# at the moment 
	$id_catflag="";
	$id_catinfo="";
	$id_photog="";
	$id_dnldno=0;
	$id_modelno=0;
	$id_modelid='';
	$id_modelname='';
	$id_vendor="";
	$id_dbhandle='';
	$id_key='';
	$id_setkey='';
	$id_setname='';
	$id_status='';
	$id_notes='';
	$id_setno=0;
}

# id_get_flag
#
# get the catflag
sub id_get_flag
{
	# return the previously acquired catflag
	return( $id_catflag );
}

# id_get_info
#
# get the catinfo string
sub id_get_info( $ )
{
	my( $existcat )=@_;

	# add in any existing values passed (model attributes, etc)
	$existcat .= " ".$id_catinfo;
	$id_catinfo = $existcat;
	# remember to call removedups or similar on what you get back from
	# this routine
	return( $id_catinfo );
}

# id_get_photog
#
# get the photographer id
sub id_get_photog
{
	# return the previously acquired photographer
	return( $id_photog );
}

# id_get_dnldno
#
# get the download number
sub id_get_dnldno
{
	# return the previously acquired download number
	return( $id_dnldno );
}

# id_get_modelno
#
# get the model number 
sub id_get_modelno
{
	# return the previously acquired model number
	return( $id_modelno );
}

# id_get_modelid
#
# get the model id (eri046, bailey_(6), etc)
sub id_get_modelid
{
	# return the previously acquired model id
	return( $id_modelid );
}

# id_get_modelname
#
# get the model's name
sub id_get_modelname
{
	# return the previously acquired model's name
	return( $id_modelname );
}

# id_get_vendor
#
# get the vendor name
sub id_get_vendor
{
	# return the previously acquited vendor id
	return( $id_vendor );
}

# id_get_dbhandle
#
# get the database handle
sub id_get_dbhandle
{
	# return the previously acquited database handle
	return( $id_dbhandle );
}

# id_get_key
#
# get the key (ie models key for this vendor)
sub id_get_key
{
	# return the previously acquited vendor id
	return( $id_key );
}

# id_get_setkey
#
# get the set key used most recently
sub id_get_setkey
{
	# return the previously acquited vendor id
	return( $id_setkey );
}

# id_get_setname
#
# get the name of the most recent set
sub id_get_setname
{
	# return the latest specified setname
	return( $id_setname );
}

# id_get_status
#
# get the status of the most recently studied record
sub id_get_status
{
	# return latest value
	return( $id_status );
}

# id_get_notes
#
# get the current value	of the notes field
sub id_get_notes
{
	# return latest value
	return( $id_notes );
}
 
# id_get_setno
#
# get the current value of the setno field
sub id_get_setno
{
	# return latest value
	return( $id_setno );
}

# dnld_img
#
# retrieve a download record based upon the namestem
sub dnld_img( % )
{
	my( %params )=@_;
	my( $dnldno, $didtype, $dvendor, $dfname, $dsetflag, $ddbhandle );
	my( $dsql, $dcsr, @drec );
	my( $length, $trimed, $modded, $modkey, $imsql, $imcsr );
	my( $addalt, $ziploc, $zipdir, $zipfile, @ziplist, $zipfound );
	my( $holdfile );

	$dnldno=0;
	$didtype = 0;
	$addalt = "";
	# it would be pointless if all of these were filled in!
	reset_attr( 'images' );
	$dnldno = $params{'downloadno'};
	$dvendor = $params{'vendor'};
	$didtype = $params{'idtype'};
	$dsetflag = $params{'setflag'};
	$dfname = $params{'dfname'};
	$ddbhandle = $params{'dbhandle'};

	$length	= length $dfname;

	# if the download number is given (probably found in .unpack and
	# passed to us), go straight to getting the details
	if( $dnldno > 0 )
	{
		if( $debug > 1 )
		{
			print STDERR "dnld_img: downloadno was given\n";
		}
		# NB: ignores any record with a download status of Error -
		# not the right model
                #                0           1         2        3      4
                $dsql = "select downloadno, dmodelno, dstatus, dtype, dsite, ";
                #          5         6         7       8     9
                $dsql .= "dsetname, dsetflag, dsetno, dkey, dsetkey ";
                $dsql .= "from ".Wacs::conf_get_attr("tables","download")." ";
                $dsql .= "where downloadno = '".$dnldno."' ";
		$dsql .= "and dstatus != 'E' ";
	}

	# if the vendor details were already filled in, try for the 
	# necessary info from the vendor database
	if( $dvendor ne "" )
	{
		if( $debug > 1 )
		{
			print STDERR "dnld_img: vendor was given\n";
		}
                #                     0      1      2           3
                $vendidsql = "select vsite, vname, vshortname, vidtimg, ";
                #              4
                $vendidsql.= "vidtvid ";
                $vendidsql.= "from ".Wacs::conf_get_attr("tables","vendor")." ";
		$vendidsql.= "where vsite = ? ";
                $vendidcsr=$ddbhandle->prepare($vendidsql);
                $vendidcsr->execute( $dvendor );
                if( @vendidrec = $vendidcsr->fetchrow_array )
                {
			if( $didtype == 0 )
			{
				$didtype = $vendidrec[3];
			}
                }
	}

	if( $didtype == 0 && $dfname ne "" && $dnldno == 0 )
	{
		# this is very similar to find_namestem but isn't
		# because it sets the didtype instead of returning
		# and because we may be considering an existing
		# output from find_namestem so the lengths are 
		# shorter.  Since we're going after a very finite
		# number of download records, we can afford to be
		# a little more lax.
		# ATK family sites
		if( $length >= 16 && $didtype == 0 )
		{
			# could be standard amkingdom format
			$trimed = substr( $dfname, 0, 16 );
			if( $trimed =~
			    /^[a-z][a-z][a-z][0-9][0-9][0-9][A-Z][A-Z][A-Z]/ )
			{
				$didtype = 2;
				print "MATCH: trimed is [$trimed]\n";
			}
		}
		# SE style
		if( $length >= 11 && $didtype == 0 )
		{
			# could be standard sapphic erotica, simonscans, etc
			# format
			$trimed = substr( $dfname, 0, 11 );
			if( $trimed =~ /[A-Za-z]*[a-z][0-9][0-9]/ )
			{
				$didtype = 1;
			}
		}
		# IFG style
	        if( $length >= 9 && $didtype == 0 )
		{
			# could be infocusgirls format
			$trimed = substr( $dfname, 0, 9 );
			$trimed =~ s/_/-/g;
			if( $trimed =~ 
				/^[0-9][0-9][0-9][0-9][0-9]-[0-9][0-9]-/ )
			{
				$didtype = 3;	# IFG style
			}
                }
		# karup's HA styles
		# 1) girlname-1
		if( $dfname =~ /^[a-z][a-z0-9]*-[0-9]*$/ )
		{
			# should check for models name
			$trimed = $dfname;
			$didtype = 6;	# KHA style
		}
		# 2) girlname-1-1500  (HI-RES)
		if( $dfname =~ /^[a-z][a-z0-9]*-[0-9]*-1500/ )
		{
			$trimed = $dfname;
			$didtype = 6;
		}
		# 3) girlname-girlname-1
		if( $dfname =~ /^[a-z][a-z0-9]*-[a-z][a-z0-9]*-[0-9]*$/ )
		{
			$trimed = $dfname;
			$didtype = 6;
		}
		# 4) girlname-1-1024  (LO-RES)
		if( $dfname =~ /^[a-z][a-z0-9]*-[0-9]*-1024/ )
		{
			$trimed = $dfname;
			$didtype = 6;
		}
		# show what didtype is now
		if( $debug > 1 )
		{
			print STDERR 
			"Found: ".$dlnd_i_list{$didtype}." - $trimed\n";
		}
	}

	# if we haven't determined the idtype, we can't do much more
	# unless of course we've been given the download number!
	if( $didtype == 0)
	{
		if( $dnldno > 0 )
		{
			print STDERR
				"dnld_img: no need to determine idtype\n";
			if( $debug > 1 )
			{
				print STDERR 
				"dnld_img: given downloadno was ".$dnldno."\n";
				print STDERR
				"dnld_img: sql is ".$dsql."\n";
			}
		}
		else
		{
			print STDERR
				"dnld_img: unable to determine idtype for ".
					$dfname."\n";
			return( undef );
		}
	}

	# general case - should work for idtypes 1
	if( $didtype == 1 && $dnldno == 0)
	{
		# NB: ignore download records marked as in error
		#                0           1         2        3      4
		$dsql = "select downloadno, dmodelno, dstatus, dtype, dsite, ";
		#          5         6         7       8     9
		$dsql .= "dsetname, dsetflag, dsetno, dkey, dsetkey ";
		$dsql .= "from ".Wacs::conf_get_attr("tables","download")." ";
		$dsql .= "where darchive = '".$dfname.".zip' ";
		$dsql .= "and dstatus != 'E' ";
	}
	if( $didtype == 2 && $dnldno == 0)
	{
		# for ATK family names we don't always know the photographer
		# part of the namestem when it's first listed in the download
		# record, so we put XXX instead - so we need to wildcard it
		# to cover both scenarios (ie a photographer reference or XXXs)
		$modded = $dfname;
		$modded =~
	s/([a-z][a-z][a-z][0-9][0-9][0-9])[A-Z][A-Z][A-Z]_([0-9]+)/\1%_\2%/;
		# this isn't all of the story either since we could have a
		# altkey based name in darchive
		$modkey = $dfname;
		$modkey =~ 
	s/([a-z][a-z][a-z][0-9][0-9][0-9])[A-Z][A-Z][A-Z]_([0-9]+)/\1/;
		$modstem = $dfname;
		$modstem =~
	s/([a-z][a-z][a-z][0-9][0-9][0-9])[A-Z][A-Z][A-Z]_([0-9]+)/_\2%/;
		#                 0      1     2
		$imsql = "select isite, ikey, ialtkey ";
		$imsql.= "from ".Wacs::conf_get_attr("tables","idmap")." ";
		$imsql.= "where ikey = '".$modkey."' ";
		$imcsr= $ddbhandle->prepare( $imsql );
		if( $debug > 1 )
		{
			print STDERR "IDchk SQL is: $imsql\n";
		}
		$imcsr->execute;
		while( @imrec = $imcsr->fetchrow_array )
		{
			# protect against an ATKP entry
			if( $imrec[2] ne "" )
			{
				$addalt .= " or darchive like '".$imrec[2]."%".
				  $modstem."%' ";
			}
			if( $debug > 1 )
			{
				print STDERR "IDmatch: ".$imrec[0]." - ".
					$imrec[1]." - ".$imrec[2]. "\n";
			}
		}

		#                0           1         2        3      4
		$dsql = "select downloadno, dmodelno, dstatus, dtype, dsite, ";
		#          5         6         7       8     9
		$dsql .= "dsetname, dsetflag, dsetno, dkey, dsetkey ";
		$dsql .= "from ".Wacs::conf_get_attr("tables","download")." ";
		$dsql .= "where darchive like '".$modded."' ";
		$dsql .= "and dstatus != 'E' ";
		if( $addalt ne "" )
		{
			if( $debug > 1 )
			{
				print STDERR "added: $addalt\n";
			}
			$dsql .= $addalt;
		}
		if( $debug > 1 )
		{
			print STDERR "SQL Query is: $dsql\n";
		}
	}
	if( $didtype == 3 && $dnldno == 0 )
	{
		$trimed = substr( $dfname, 0, 9 );
		$trimed =~ s/_/-/g;
		$zipfound = 0;
		$holdfile = '';
		if( $dvendor eq "" )
		{
			# FIXME: this should probably iterate through
			# vendors with this download type
			$dvendor = "IFG";
		}
		# work out where the zip files should be
		$ziploc = vend_dnld( $dvendor, $ddbhandle )."/zips";
		if( ! -d $ziploc )
		{
			print STDERR "No active zips to examine for $dvendor\n";
			return( undef );
		}
		# slurp the contents of that directory
		opendir( $zipdir, $ziploc );
		@ziplist = readdir( $zipdir );
		closedir( $zipdir );
		# run through them
		foreach $zipfile( sort @ziplist )
		{
			# check for exclusions
			if( Wacs::checkexclude( $zipfile )== 1 )
			{
				next;
			}
			if( ! -f $ziploc."/".$zipfile )
			{
				next;
			}
			# try to open it
			if( $debug > 0 )
			{
				print STDERR "dnld_img: type 3: testing ".
						$ziploc."/".$zipfile."\n";
			}
			open( ZIPINFO, "zipinfo ".$ziploc."/".$zipfile." |" ) ||
			die( "Unable to read file ".$ziploc."/".$zipfile."\n" );
			$modded = <ZIPINFO>; # read the header line
			$modded = <ZIPINFO>;   # first data line
			chomp( $modded );
			$modstem = substr( $modded, 53, 9 ); # Ick! FIXME
			$modstem =~ s/_/-/g;
			if( $debug > 2 )
			{
				print STDERR "File ".$zipfile." has stem ".
						$modstem."\n";
			}
			if( $modstem eq $trimed )
			{
				# found the zip file name we needed
				$holdfile = $zipfile;
				$zipfound++;
				last;
			}
		}
		if( $zipfound == 0 )
		{
			print STDERR "idtype 3: failed to find matching zip\n";
			return( undef );
		}
		#                0           1         2        3      4
		$dsql = "select downloadno, dmodelno, dstatus, dtype, dsite, ";
		#          5         6         7       8     9
		$dsql .= "dsetname, dsetflag, dsetno, dkey, dsetkey ";
                $dsql .= "from ".Wacs::conf_get_attr("tables","download")." ";
                $dsql .= "where darchive = '".$holdfile."' ";
		$dsql .= "and dstatus != 'E' ";
		if( $debug > 3 )
		{
			print STDERR "dnld_img: type 3: SQL query is ".
					$dsql."\n";
		}
	}
	if( $didtype == 4 && $dnldno == 0 )
	{
		print STDERR "Abby Winters/Teenflood style is not yet ";
		print STDERR "supported\n";
	}
	if( $didtype == 5 && $dnldno == 0 )
	{
		#                0           1         2        3      4
		$dsql = "select downloadno, dmodelno, dstatus, dtype, dsite, ";
		#          5         6         7       8     9
		$dsql .= "dsetname, dsetflag, dsetno, dkey, dsetkey ";
		$dsql .= "from ".Wacs::conf_get_attr("tables","download")." ";
		$dsql .= "where darchive = '".$dfname.".zip' ";
		$dsql .= "and dstatus != 'E' ";
	}
	if( $didtype == 6 && $dnldno == 0 )
	{
		#                0           1         2        3      4
		$dsql = "select downloadno, dmodelno, dstatus, dtype, dsite, ";
		#          5         6         7       8     9
		$dsql .= "dsetname, dsetflag, dsetno, dkey, dsetkey ";
		$dsql .= "from ".Wacs::conf_get_attr("tables","download")." ";
		$dsql .= "where darchive = '".$dfname.".zip' ";
		$dsql .= "and dstatus != 'E' ";
	}

	# check we do have a suitable query
	if( $dsql eq "" )
	{
		print STDERR "No suitable SQL query was generated.\n";
		return( undef );
	}
	# query should have been written now, so run it
	$dcsr = $ddbhandle->prepare( $dsql );
	$dcsr->execute;
	if( @drec = $dcsr->fetchrow_array )
	{
		if( $debug > 0 )
		{
			print "Found: dnldno ".$drec[0]." featuring ".
				$drec[1]."\n";
			print "     : status ".$drec[2]." type ".$drec[3].
				" flag ".$drec[6]." site ".$drec[4]."\n";
			print "     : setname ".$drec[5]."\n";
		}
		$id_dnldno = $drec[0];
		$id_modelno = $drec[1];
		$id_vendor = $drec[4];
		$id_setname = $drec[5];
		$id_catflag = $drec[6];
		$id_setno = $drec[7];   # mostly this will be null/0
		$id_key = $drec[8];
		$id_setkey = $drec[9];
	}
	else
	{
		if( $debug > 0 )
		{
			print STDERR "Nothing found for $dfname or $dnldno\n";
			print STDERR "SQL query was ".$dsql."\n";
			return( undef );
		}
	}
	return( $drec[0] );
}

# dnld_markdone
#
# mark the download record as done.
sub dnld_markdone( % )
{
	my( %params )=@_;
	my( $upddnl, $today );
	my( $dsetno, $downloadno, $dstatus, $dbhandle );
	
	$dsetno = $params{'setno'};
	$downloadno = $params{'downloadno'};
	$dstatus = $params{'status'};
	$dbhandle = $params{'dbhandle'};
	if( $dstatus eq "" )
	{
		$dstatus = "S";
	}

	$today = Wacs::gettoday;

	$upddnl = "update ".Wacs::conf_get_attr("tables","download");
        $upddnl .= " set dstatus = '".$dstatus."', dpulled = '".$today;
        $upddnl .= "', dsetno = '".$dsetno."', damended = '".$today."' ";
	$upddnl .= " where downloadno = '";
        $upddnl .= $downloadno."' ";

	if( $debug > 1 )
	{
		print STDERR "dnld_markdone: SQL Query: $upddnl\n";
	}
	$dbhandle->do( $upddnl );
        if( $DBI::errstr ne "" )
        {
                print STDERR "dnld_markdone: SQL Choaked\n";
                print STDERR "dnld_markdone: Query was: $upddnl\n";
                print STDERR "dnld_markdone: Error was: $DBI::errstr\n";
                exit -1;
        }
	return(0);	# success
}

# dnld_checkadd
#
# checks or adds a new download record 
sub dnld_checkadd( % )
{
	my( %params )=@_;
	my( $dsite, $dkey, $dsetkey, $dmodel, $dtype, $dsetname, $dnotes );
	my( $dsetflag, $dduration, $dimgcount, $durl, $darchive, $ddbhandle );
	my( $dstatus );
	my( $dcsql, $dccsr, @dcrec, $nextfree, $today, $crsql, $modchg );
	my( $downloadno, $dupdcmd );

	$dsite = $params{'site'};
	$dkey = $params{'key'};
	$dsetkey = $params{'setkey'};
	$dmodel = $params{'modelno'};
	$dtype = $params{'type'};
	$dsetname = $params{'setname'};
	$dsetflag = $params{'setflag'};
	$dnotes = $params{'notes'};
	$dduration = $params{'duration'};
	$dimgcount = $params{'imgcount'};
	$durl = $params{'url'};
	$darchive = $params{'archive'};
	$ddbhandle = $params{'dbhandle'};
	$dstatus = $params{'status'};

	# basic stuff and sanity checks	
	$today = Wacs::gettoday;
	if( $dstatus eq "" )
	{
		$dstatus = 'U';  # default dstatus is U if not specified
	}

	# check if it already exists
	# BUG FIX: (I Hope) - no longer checks only the same type for
	# already being the same key and set key - stops duplicate records
	# getting *AUTOMATICALLY* created for both image and video sets
	# - you can always create them manually in the few cases where it
	# happens (euronudes comes to mind).
	#                 0           1      2      3        4
	$dcsql = "select downloadno, dsite, dtype, dstatus, dsetname, ";
	#          5       6     7
	$dcsql.= "dnotes, durl, darchive ";
	$dcsql.= "from ".Wacs::conf_get_attr("tables","download")." ";
	$dcsql.= "where dmodelno = ? and dkey = ? and dsetkey = ? ";
	$dcsql.= "and dsite = ? ";
	$dccsr = $ddbhandle->prepare( $dcsql );
	$dccsr->execute( $dmodel, $dkey, $dsetkey, $dsite );
	if( $DBI::errstr ne "" )
	{
		print STDERR "SQL bombed: $DBI::errstr\n";
	}
	if( (@dcrec = $dccsr->fetchrow_array) )
	{
		$downloadno = $dcrec[0];	
		if( $debug > 0 )
		{
			print STDERR "Download Record Exists: $downloadno\n";
			print STDERR "Info:$dmodel, $dkey, $dsetkey, $dsite".
				", $dtype\n";
			print STDERR "Status:".$dcrec[3]." Name:".$dcrec[4].
				"\n";
		}
		# we might still want to go after the icon, size, title
		# etc so set these values just in case
		$id_dnldno = $downloadno;
		$id_key = $dkey;
		$id_setkey = $dsetkey;
		$id_status = $dcrec[3];
		if( $id_notes eq "" )
		{
			$id_notes = $dcrec[5];
		}
		if( $id_setname eq "" )
		{
			$id_setname = $dcrec[4];
		}
		# if the download record is complete, nothing more to do
		if( $dcrec[3] ne "X" )
		{
			# all done
			return( 1 );
		}
		# download record was incomplete
		if( $debug > 0 )
		{
			print STDERR "Download record marked as incomplete.\n";
		}
		# see if we have new information about it
		$dupdcmd = "update ".Wacs::conf_get_attr("tables","download");
		$dupdcmd.= " set ";
		if( $dcrec[6] eq "" && $durl ne "" )
		{
			$dupdcmd .= "durl = '".$durl."', ";
		}
		if( $dcrec[7] eq "" && $darchive ne "" )
		{
			$dupdcmd .= "darchive = '".$darchive."', ";
		}
		# OK, those were the vital ones
		if( $dupdcmd =~ /set $/ )
		{
			# nothing was added to these two fields, so
			# no substantive change
			return( 1 );
		}
		# we do appear to have a substantive change in the amount
		# of information available
		if( $dcrec[4] ne $dsetname )
		{
			$dupdcmd.= "dsetname = '".$dsetname."', ";
		}
		$dupdcmd.= "dstatus = 'U', damended = '".today."' ";
		$dupdcmd.= "where downloadno = '".$dcrec[0]."' ";
		if( $debug > 0 )
		{
			print STDERR "incomplete record becomes viable.\n";
			print STDERR "update cmd is: ".$dupdcmd."\n";
		}
		$dbhandle->do( $dupdcmd );
		if( $DBI::errstr ne "" )
		{
			print STDERR "dnld_checkadd: SQL update croaked\n";
			print STDERR "Error was: $DBI::errstr\n";
			print STDERR "Query was: $dupdcmd\n";
			return( -1 );
		}
		return( 1 );
	}

	# create it - so allocate new key
	$nextfree = WacsStd::alloc_nextkey( 
			Wacs::conf_get_attr("tables","download"),
			   "downloadno", $ddbhandle );
	$crsql = "insert into ".Wacs::conf_get_attr("tables","download");
	$crsql.= " ( downloadno, dmodelno, dstatus, dtype, dsite, dkey, ";
	$crsql.= "dsetkey, dsetname, dnotes, durl, darchive, dsetflag, ";
	$crsql.= "dadded ) ";
	#                  downloadno       dmodelno       dstatus  
	$crsql.= "values( '".$nextfree."','".$dmodel."','".$dstatus;
	#             dtype
	$crsql.= "','".$dtype."','";
	#        dsite        dkey        dsetkey       dsetname
	$crsql.= $dsite."','".$dkey."','".$dsetkey."','".$dsetname."','";
	#        dnotes       durl        darchive         dsetflag
	$crsql.= $dnotes."','".$durl."','".$darchive."','".$dsetflag."','";
	#        dadded
	$crsql.= $today."' ) ";
	
	# add some output when we create a new one
	if( $debug > 0 )
	{
		print STDERR "**NEW ** Download Record Created: $nextfree\n";
		print STDERR "Info:$dmodel, $dkey, $dsetkey, $dsite, $dtype\n";
		print STDERR "Status:U Name:".$dsetname."\n";
	}

	# do the creation bit and store number used in case any one needs
	# it
	if( $debug > 1 )
	{
		print STDERR "Insert SQL is: ".$crsql."\n";
	}
	$ddbhandle->do( $crsql );
	if( $DBI::errstr ne "" )
	{
		print STDERR "dnld_checkadd: SQL Croaked\n";
		print STDERR "Error was: $DBI::errstr\n";
		print STDERR "Query was: $crsql\n";
		return -1;
	}
	$id_dnldno = $nextfree;
	$id_key = $dkey;
	$id_setkey = $dsetkey;
	$id_status = 'U';  # we've just created it so it must be 'U'

	# Since we stored a new download record for this model, the 
	# model's idmap for this site should be updated to show that
	# something changed (but do not update any idmap entries for
	# this site marked as obsolete)
	$modchg = "update ".Wacs::conf_get_attr("tables","idmap")." ";
	$modchg.= "set ichanged = '";
	$modchg.= $today."', iactive = 'Y' where isite = '";
	$modchg.= $dsite."' and imodelno = '";
	$modchg.= $dmodel."' and iactive != 'O' ";
	if( $debug > 1 )
	{
		print STDERR "Changed SQL is: ".$modchg."\n";
	}
	$ddbhandle->do( $modchg );
	if( $DBI::errstr ne "" )
	{
		print STDERR "Model Update Flagging Failed\n";
		print STDERR "Error was: ".$DBI::errstr."\n";
		print STDERR "SQL was: ".$modchg."\n";
	}
	# all done
	return 0;
}

# dnld_update
#
# update the specified download record with new information
# (This is an easy way to cope with keywords that appear after the 
#  link to the target object.)
sub dnld_update( % )
{
	my( %params )=@_;
	my( $dnldno, $field, $value, $dhandle );
	my( $updcmd );

	$dnldno = $params{'dnldno'};
	$field = $params{'field'};
	$value = $params{'value'};
	$dhandle = $params{'dbhandle'};

	# sanity checks first
	if( $field eq "" )
	{
		return;
	}
	if( $value eq "" )
	{
		return;
	}
	if( $dnldno eq "" )
	{
		return;
	}

	# start building the query
	$updcmd = "update ".Wacs::conf_get_attr("tables","download")." ";
	if( $field eq "setname" )
	{
		$updcmd.= "set dsetname = '".$value."' ";
	}
	if( $field eq "notes" )
	{
		$updcmd.= "set dnotes = '".$value."' ";
	}
	if( $field eq "setflag" )
	{
		$updcmd.= "set dsetflag = '".$value."' ";
	}
	$updcmd.= "where downloadno = '".$dnldno."' ";

	# run the query and check for errors
	$dhandle->do( $updcmd );
	if( $DBI::errstr ne "" )
	{
		print STDERR "SQL update choaked\n";
		print STDERR "Error was: $DBI::errstr\n";
		print STDERR "SQL command was: $updcmd\n";
	}
}

# vend_dnld
#
# return the download location for this vendor
sub vend_dnld( $$ )
{
	my( $dvendor, $ddbhandle )=@_;
	my( $dsql, $dcsr, @drec );

	$dsql = "select vshortname ";
	$dsql .= "from ".Wacs::conf_get_attr("tables","vendor")." ";
	$dsql .= "where vsite = ? ";
	$dcsr=$ddbhandle->prepare( $dsql );
	$dcsr->execute( $dvendor );
	
	if( ! (@drec = $dcsr->fetchrow_array) )
	{
		print STDERR "vend_dnld: No vendor $dvendor\n";
		return undef;
	}
	if( $drec[0] eq "" )
	{
		print STDERR "vend_dnld: No shortname specified for $dvendor\n";
		return undef;
	}
	return( Wacs::conf_get_attr("fsloc","download")."/".$drec[0] );
}
	
# find_cookies
#
# find the current web browser cookies file
sub find_cookies( $$ )
{
	my( $browser, $cookiefile )=@_;
	my( $homedir, $dirh, $checkit, $mtime, $latest );
	my( @dirlist, %choices, $found );
	my( $dev,$ino,$mode,$nlink,$uid,$gid,$rdev,$size );
	my( $atime,$mtime,$ctime,$blksize,$blocks );

	%choices = ();
	$found = 0;
	$homedir = $ENV{"HOME"};
	# put cases for other browsers here
	if( $browser eq "firefox" )
	{
		$homedir .= "/.mozilla/firefox/";
	}
	else
	{
		print STDERR "Browser ".$browser." not supported.\n";
	}
	# slurp the directory contents
	if( $debug > 3 )
	{
		print STDERR "Browser file area is ".$homedir."\n";
	}
	opendir( $dirh, $homedir );
	@dirlist = readdir( $dirh );
	closedir( $dirh );

	# scan through them 
	foreach $checkit ( sort @dirlist )
	{
		if( $debug > 3  )
		{
			print STDERR "considering $checkit...\n";
		}
		if( -f "$homedir/$checkit/$cookiefile" )
		{
			# store the date.
			if( $debug > 3 )
			{
				print STDERR
					"There's a cookie file in $checkit\n";
			}
			($dev,$ino,$mode,$nlink,$uid,$gid,$rdev,$size,
			      $atime,$mtime,$ctime,$blksize,$blocks)
                          = stat( "$homedir/$checkit/$cookiefile" );
			$choices{$checkit}=$mtime;
			$found++;
		}
	}
	
	# no cookies file found
	if( $found == 0 )
	{
		if( $debug > 0 )
		{
			print STDERR "cookies file $cookiefile not found.\n";
		}
		return undef;
	}
	# just one cookies file found
	if( $found == 1 )
	{
		@dirlist = keys %choices;
		if( $debug > 1 )
		{
			print STDERR "returning single result $dirlist[0]\n";
		}
		# only one location found
		return( "$homedir/$dirlist[0]/$cookiefile" );
	}
	# several found, look for the newest
	$mtime = 0;
	$latest ='';
	foreach $checkit ( keys %choices )
	{
		if( $mtime < $choices{$checkit} )
		{
			# this one is newer
			$mtime = $choices{$checkit};
			$latest = $checkit;
		}
	}
	if( $debug > 1 )
	{
		print STDERR
		   "newest was $latest at ".scalar localtime( $mtime )."\n";
	}
	return( "$homedir/$latest/$cookiefile" );
}

# vid_getsize
#
# get the size of the video file
sub vid_getsize( $ )
{
        my( $vfpathname ) =@_;
        my( $dev,$ino,$mode,$nlink,$uid,$gid,$rdev,$size);
        my( $atime,$mtime,$ctime,$blksize,$blocks);

	if( ! -f $vfpathname )
	{
		print STDERR 
		   "vid_getsize: WARNING: file $vfpathname not found.\n";
		return( 0 );
	}
        ($dev,$ino,$mode,$nlink,$uid,$gid,$rdev,$size,
         $atime,$mtime,$ctime,$blksize,$blocks)
         = stat($vfpathname);
        return( $size );
}

#
# extractphotog
#
# Try to extract the photographer name from wellknown name stem encodings
sub extractphotog( $$$ )
{
        my( $srcstem, $dbtable, $dbhandle )=@_;
        my( $phoref, $phoOK, $phosql, $phocsr, @phorec );

        $phoOK = 0;
        # assumes AMK/ATKP format namestem but at least checks for all
        # capitals....
        $phoref = substr( $srcstem, 6, 3 );
        if( $phoref =~ /[A-Z][A-Z][A-Z]/ )
        {
		if( $debug > 2 )
		{
			print STDERR "extractphotog: $phoref found\n";
		}
                # passes this test
                $phoOK = 1;
        }
        if( $phoOK == 0 )
        {
                # doesn't look like an AMK style photographer ref
                return( "" );
        }

        $phosql = "select pref, pname, prating from ";
        $phosql .= $dbtable." where pref = ? ";
        $phocsr = $dbhandle->prepare( $phosql );
        $phocsr->execute( $phoref );
        @phorec = $phocsr->fetchrow_array;
        if( $phorec[0] eq $phoref )
        {
                # print "Photographer: $phorec[1] ($phorec[0]) $phorec[2]/5\n";
                return( $phoref );
        }
        print "Unknown Photographer: $phoref (maybe)\n";
        return( "" );
}

#
# Find_NameStem
#
# look for well known image naming conventions within the adult site
# community
sub find_namestem( $ )
{
        my( $wherearewe )=@_;
        my( $length, $sample, $trimed, $matched );
        my( $fndirh, $fnfile );

        $matched=0;
        $sample="";
        opendir( $fndirh, $wherearewe );
        while( $fnfile = readdir( $fndirh ))
        {
		# note explicit call to Wacs module function
                if( !Wacs::checkexclude( $fnfile ) )
                {
                        # not on the exclusion list
                        if( !Wacs::checkindex( $fnfile ) )
                        {
                                $sample = $fnfile;
                                break;
                        }
                }
        }
        if( $sample eq "" )
        {
                print "Sample name was zero-length - aborting\n";
                print "file was $fnfile, location was $wherearewe.\n";
                exit -1;
        }
        $length = length $sample;
        # print "Filename found: $sample  Length: $length";
        if( $length == 18 )
        {
                # could be standard sapphic erotica, simonscans, etc
                # format
                $trimed = substr( $sample, 0, 11 );
                # print "Trimed: $trimed\n";
                if( $trimed =~ /[A-Za-z]*[0-9][0-9]/ )
                {
			if( $debug > 2 )
			{
				print STDERR "Found a match (SE style) - ".
					$trimed."\n";
			}
                        return $trimed;
                }
        }
        if( $length > 20 )
        {
                # could be standard amkingdom format
                $trimed = substr( $sample, 0, 16 );
                if( $trimed =~ /^[a-z][a-z][a-z][0-9][0-9][0-9][A-Z][A-Z]/ )
                {
			if( $debug > 2 )
			{
				print STDERR "Found a match (amkingdom style)".
					" - ".$trimed."\n";
			}
                        return $trimed;
                }
        }
        if( $length = 16 )
        {
                # could be infocusgirls format
                $trimed = substr( $sample, 0, 9 );
                $trimed =~ s/_/-/g;
                if( $trimed =~ /^[0-9][0-9][0-9][0-9][0-9]-[0-9][0-9]-/ )
                {
			if( $debug > 2 )
			{
				print STDERR
					"Found a match (infocusgirls style) - ".
					$trimed."\n";
			}
                        return $trimed;
                }
        }
	# variants of Karup's HA
	# 1) girlname-1-001.jpg
	if( $sample =~ /^[a-z][a-z0-9]*-[0-9]*-[0-9][0-9][0-9]\.jpg$/ )
	{
		$trimed = $sample;
		$trimed =~ s/-[0-9][0-9][0-9]\.jpg$//;
		if( $trimed =~ /^[a-z][a-z0-9]*-[0-9]*$/ )
		{
			if( $debug > 2 )
			{
				print STDERR 
					"Found a match (Karup's HA style 1) - ".
					$trimed."\n";
			}
			return $trimed;
		}
	}
	# 2) girlname-1-1500-001.jpg   (HI-RES)
	if( $sample =~ /^[a-z][a-z0-9]*-[0-9]*-1500-[0-9][0-9][0-9]\.jpg$/ )
	{
		$trimed = $sample;
		$trimed =~ s/-[0-9][0-9][0-9]\.jpg$//;
		if( $trimed =~ /^[a-z][a-z0-9]*-[0-9]*-1500$/ )
		{
			if( $debug > 2 )
			{
				print STDERR 
					"Found a match (Karup's HA style 2) - ".
					$trimed."\n";
			}
			return $trimed;
		}
	}
	# 3) girlname-girlname-1-001.jpg  (Lesbian or Orgy)
	if( $sample =~ 
	   /^[a-z][a-z0-9]*-[a-z][a-z0-9]*-[0-9bg]*-[0-9][0-9][0-9]\.jpg$/ )
	{
		$trimed = $sample;
		$trimed =~ s/-[0-9][0-9][0-9]\.jpg$//;
		if( $trimed =~ /^[a-z][a-z0-9]*-[a-z][a-z0-9]*-[0-9bg]*$/ )
		{
			if( $debug > 2 )
			{
				print STDERR
					"Found a match (Karup's HA style 3) - ".
					$trimed."\n";
			}
			return $trimed;
		}
	}
        return undef;
}

# id_mpage
#
# process a modelpage looking for links to suitable sets
sub id_mpage( % )
{
	my( %params )=@_;
	my( $iidtype, $vidtype, $vendor, $href, $onmouse, $onclick );
	my( $modelno, $modelname, $modelkey, $sitemid, $ddbhandle );
	my( $iproto, $vproto, $setname, $snotes, $setno, $vpageurl );
	my( $target, $dumy, $clean, $setname, $padsid, $sameurl );

	$iidtype = $params{'iidtype'};
	$vidtype = $params{'vidtype'};
	$vendor = $params{'vendor'};
	$modelno = $params{'modelno'};
	$modelname = $params{'name'};
	$modelkey = $params{'modelkey'};
	$sitemid = $params{'sitemid'};
	$href = $params{'href'};
	$onmouse = $params{'onmouse'};
	$onclick = $params{'onclick'};
	$iproto = $params{'iproto'};
	$vproto = $params{'vproto'};
	$setname = $params{'setname'};
	$snotes = $params{'notes'};
	$setno = $params{'setno'};
	$sameurl = $params{'sameurl'};
	$ddbhandle = $params{'dbhandle'};

	# ensure debug level is set - this is an entry point to the module
	# from chkmodel
	$debug=Wacs::conf_get_attr("debug","module_wacsid");
	# continue...
	if( $iidtype < 1 )
	{
		# at least an image type must be identfied
		if( $debug > 0 )
		{
			print STDERR "id_mpage: ignoring invalid img id type\n";
		}
		return undef;
	}
	if( $debug > 3 )
	{
		print STDERR "id_mpage: img:".$iidtype."-".
				$id_i_list{$iidtype}."\n";
		print STDERR "id_mpage: vid:".$vidtype."-".
				$id_v_list{$vidtype}."\n";
	}

	# this routine is an external entry point into the WacsId module
	# so at this point if we want to set global variables we can
	reset_attr( 'all' );
	$id_vendor = $vendor;
	$id_modelno = $modelno;
	$id_modelid = $modelkey;
	$id_modelname = $modelname;
	$id_key = $sitemid;
	$id_dbhandle = $ddbhandle;
	$id_setname = $setname;
	$id_notes = $snotes;
	$id_setno = $setno;

	#  IMAGE TYPES:
	#  ============
	# sort out what to do per type
	# first - generic 
	if( $iidtype == 1 )
	{
		if( $debug > 3 )
		{
			print STDERR
			"id_mpage: generic image: call id_gen_trymatch with ".
				$href."\n";
			print STDERR
			"id_mpage: iproto: ".Dumper( $iproto )."\n";
		}
		return( id_gen_trymatch( $iproto, $vproto, $href ));
	}
		
	# this is for AMK and ATE
	if( $iidtype == 2 )
	{
		if( $href =~ /modelhome/ )
		{
			# link on models name - ignore
			return undef;
		}
		if( $href =~ /pagetype=series/ )
		{
			$mid = $sitemid;
			( $dumy, $sid ) = split( /set_id=/, $href, 2 );
			$padsid = sprintf("%-6.6d", $sid );
			if( $debug > 1 )
			{
				print STDERR 
				"IMAGES: $modelname|$sid|$href|$modelno\n";
			}
			$setname=sprintf( "%s (%s) set %-6.6d", $modelname,
					  $modelkey, $padsid );
			dnld_checkadd( 'site'=>$vendor,
					'type'=>'I',
					'key'=>$mid,
					'setkey'=>$sid,
					'setname'=>$setname,
					'imgcount'=>$imagecount,
					'modelno'=>$modelno,
					'url'=>$href,
					'archive'=>$modelkey."XXX_".
							$padsid.".zip",
					'dbhandle'=>$ddbhandle );
			return( $sid );
		}
	}

	# this is for ATKP only
	if( $iidtype == 3 )
	{
		if( $href =~ /modelhome/ )
		{
			# link on models name - ignore
			return undef;
		}
		( $stem, $target ) = split( /modelupdate\&/, $href, 2 );
		( $sid, $mid ) = split( /\&/, $target, 2 );
		$sid =~ s/thisupdate=//g;
		$mid =~ s/thismodel=//g;
		if( $mid ne "" && $mid ne $sitemid )
		{
			# if it's about someone else, we're not interested
			print "About someone else: $mid\n";
			return undef;
		}
		$lastmodelid = $mid;
		if( $setname eq "" )
		{
			$setname=sprintf( "%s (%s) set %-6.6d", $modelname,
                                          $modelkey, $sid );
		}

		if( $debug > 3 )
		{
			print STDERR
				"id_mpage: type ".$iidtype." image: model ".
					$mid." set ".$sid."\n";
		}
		if( $onmouse ne "" && $debug > 2 )
		{
			print STDERR "OnMouseOver is $onmouse\n";
		}
		# check in the URL
		if( $href =~ /photos/ )
		{
			if( $debug > 1 )
			{
				print STDERR 
					"IMAGES: $mid|$sid|$href|$modelno\n";
			}
			dnld_checkadd( 'site'=>$vendor,
					'type'=>'I',
					'key'=>$mid,
					'setkey'=>$sid,
					'setname'=>$setname,
					'imgcount'=>$imagecount,
					'modelno'=>$modelno,
					'url'=>$href,
					'archive'=>$mid."XXX_".$sid.".zip",
					'dbhandle'=>$ddbhandle );
			return( $sid );
		}
		# check in the onmouse
		if( $onmouse =~ /photos/ )
		{
			if( $debug > 1 )
			{
				print STDERR "IMAGES: $mid|$sid|$href|$modelno\n";
			}
			dnld_checkadd( 'site'=>$vendor,
					'type'=>'I',
					'key'=>$mid,
					'setkey'=>$sid,
					'setname'=>$setname,
					'imgcount'=>$imagecount,
					'modelno'=>$modelno,
					'url'=>$href,
					'archive'=>$mid."XXX_".$sid.".zip",
					'dbhandle'=>$ddbhandle );
			return( $sid );
		}
	}

	# Image ID type 4 is currently not supported

	# this for the very simplistic Karup's PC 
	if( $iidtype == 5 )
	{
		# check in the URL
		if( $href =~ /album/ )
		{
			# top and tail the url
			( $dumy, $target )=split( /album/, $href, 2 );
			( $clean, $dumy )=split( /\//, $target, 2 );
			$mid = $modelkey;
			$sid = $clean;
			# check for clues to pass onwards
			if( $href =~ /girltoys/ )
			{
				$id_catflag = "T";
				$notes = "Catflag: T";
			}
			elsif( $href =~ /lesbian/ )
			{
				$id_catflag = "L";
				$notes = "Catflag: L";
			}
			elsif( $href =~ /guygirl/ )
			{
				$id_catflag = "F";
				$notes = "Catflag: F";
			}
			# set up the modelname
			$setname = $modelname." Set ".$sid;
			( $target, $stem, $clean )= fileparse( $href );
			if( $href =~ /lesbian/ )
			{
				# make suitable for use as key
				$clean = $target;
				$clean =~ s/1\.htm$//;
				$clean =~ s/\&/_and_/g;
				$sid .= "-".$clean;
				# make suitable for display
				$clean =~ s/_/ /g;
				$setname = $clean." Set ".$sid;
			}
			$clean = $href;
			# change 1.htm to 1.zip
			$clean =~ s/1\.htm/1.zip/;
			$target =~ s/1\.htm$/1.zip/;
			if( $debug > 1 )
			{
				print STDERR 
					"IMAGES: $mid|$sid|$clean|$modelno\n";
			}
			dnld_checkadd( 'site'=>$vendor,
					'type'=>'I',
					'key'=>$mid,
					'setkey'=>$sid,
					'setname'=>$setname,
					'setflag'=>$id_catflag,
					'imgcount'=>$imagecount,
					'modelno'=>$modelno,
					'url'=>$clean,
					'archive'=>$target,
					'dbhandle'=>$ddbhandle );
			return( $sid );
		}
	}
	# this is for Karup's HA - it's seperate from the default since
	# they have database-based setpages but filesystem tree based
	# retrieval - a sort of half way house.
	# OK, first try for images. (or both if set URLs are the same 
	# for both settypes)
	if( $iidtype == 6 )
	{
		if( $debug > 3 )
		{
			print STDERR
			"id_mpage: karupsha image: call id_gen_trymatch with ".
				$href."\n";
			print STDERR
			"id_mpage: iproto: ".Dumper( $iproto )."\n";
			print STDERR
			"id_mpage: vproto: ".Dumper( $vproto )."\n";
		}
		return( id_gen_trymatch( $iproto, $vproto, $href ));
	}

	#  VIDEO TYPES:
	#  ============

	# generic type matching for videos
	if( $vidtype == 1 )
	{
		# issue warning if appropriate
		if( $sameurl == 1 )
		{
			print STDERR "WARNING: video page URL is the same ".
				"as the image page URL.\n";
		}
		if( $debug > 3 )
		{
			print STDERR
			"id_mpage: generic video: call id_gen_trymatch with ".
				$href."\n";
			print STDERR
			"id_mpage: vproto: ".Dumper( $vproto )."\n";
		}
		return( id_gen_trymatch( $iproto, $vproto, $href ));
	}

	# this is for AMK and ATE videos
	if( $vidtype == 2 )
	{
		if( $href =~ /movies.php/ )
                {
			# treat findvids_atkg as if it were dnld_checkadd
			$mid = $sitemid;
			findvids_atkg( 'site'=>$vendor,
				       'type'=>'V',
				       'key'=>$mid,
				       'setkey'=>$sid,
				       'setname'=>$setname,
				       'duration'=>$duration,
                                       'modelkey'=>$modelkey,
				       'modelno'=>$modelno,
				       'name'=>$modelname,
				       'url'=>$href,
				       'dbhandle'=>$ddbhandle );
			return( $sid );
                }
		# new style ATK Galleria video index page
		if( $href =~ /vnavdata/ )
                {
			# treat findvids_atkg as if it were dnld_checkadd
			$mid = $sitemid;
			findvids_atkg( 'site'=>$vendor,
				       'type'=>'V',
				       'key'=>$mid,
				       'setkey'=>$sid,
				       'setname'=>$setname,
				       'duration'=>$duration,
                                       'modelkey'=>$modelkey,
				       'modelno'=>$modelno,
				       'name'=>$modelname,
				       'url'=>$href,
				       'dbhandle'=>$ddbhandle );
			return( $sid );
		}
	}
	# ATKP video determining
	if( $vidtype == 3 )
	{
		if( $href =~ /modelhome/ )
		{
			# link on models name - ignore
			return undef;
		}
		( $stem, $target ) = split( /modelupdate\&/, $href, 2 );
		( $sid, $mid ) = split( /\&/, $target, 2 );
		$sid =~ s/thisupdate=//g;
		$mid =~ s/thismodel=//g;
		if( $mid ne "" && $mid ne $sitemid )
		{
			# if it's about someone else, we're not interested
			print STDERR "About someone else: $mid\n";
			return undef;
		}
		$lastmodelid = $mid;
		if( $setname eq "" )
		{
			$setname=sprintf( "%s (%s) movie %-6.6d", $modelname,
                                          $modelkey, $sid );
		}
		$vpageurl = Wacs::vendlink( 'vendor'=>$vendor,
				      'page'=>'vidpage',
				      'key'=>$mid,
				      'setkey'=>$sid,
				      'dbhandle'=>$ddbhandle );

		if( $debug > 3 )
		{
			print STDERR
				"id_mpage: type ".$iidtype." video: model ".
					$mid." set ".$sid."\n";
		}
		if( $onmouse ne "" && $debug > 2 )
		{
			print STDERR "OnMouseOver is $onmouse\n";
		}
		if( $href =~ /viewmovie/ )
		{
			if( $debug > 0 )
			{
				print STDERR
					"MOVIE: $mid|$sid|$vpageurl|$modelno\n";
			}
			findvids_atkg( 'site'=>$vendor,
					'type'=>'V',
					'key'=>$mid,
					'setkey'=>$sid,
					'setname'=>$setname,
					'duration'=>$duration,
					'modelno'=>$modelno,
					'modelkey'=>$modelkey,
					'url'=>$vpageurl,
					'notes'=>$snotes,
					'archive'=>$mid."XXX_".$sid.".mpg",
					'dbhandle'=>$ddbhandle );
			return( $sid );
		}
		if( $onmouse =~ /viewmovie/ )
		{
			if( $debug > 0 )
			{
				print STDERR "MOVIE: $mid|$sid|$href|$modelno\n";
			}
			findvids_atkg( 'site'=>$vendor,
					'type'=>'V',
					'key'=>$mid,
					'setkey'=>$sid,
					'setname'=>$setname,
					'duration'=>$duration,
					'modelno'=>$modelno,
					'modelkey'=>$modelkey,
					'url'=>$vpageurl,
					'notes'=>$snotes,
					'archive'=>$mid."XXX_".$sid.".mpg",
					'dbhandle'=>$ddbhandle );
			return( $sid );
		}
	}
	# Video ID type 4 is currently not supported.

	# now for Karup's HA videos
	if( $vidtype == 5 )
	{
		# FIXME: this is rather sweeping
		if( $sameurl == 1 )
		{
			print STDERR "WARNING: video page URL is the same ".
				"as the image page URL.\n";
			# at present this is a killer for KPC
			if( $iidtype == 6 )
			{
				print STDERR "**** KPC doesn't work ****\n";
				return( undef );
			}
		}

		print STDERR "KarupsHA Videos: ".$href."\n";
		if( $debug > 3 )
		{
			print STDERR
			"id_mpage: karupsha video: call id_vid_trymatch with ".
				$href."\n";
			print STDERR
			"id_mpage: vproto: ".Dumper( $vproto )."\n";
		}
		return( id_vid_trymatch( $vproto, $href ));
	}

	# all done
	return undef;
}

# id_i_fetch
#
# does the necessary magic to find images from the parser a-tag callback
# for various sites; needs to be passed the idtype previously determined
sub id_i_fetch( % )
{
	my( %params )=@_;
	my( $idtype, $vendor, $href, $onmouse, $onclick, $ddbhandle );
	my( $vurlstem, $target, $dumy, $clean );

	$idtype = $params{'idtype'};
	$vendor = $params{'vendor'};
	$vurlstem = $params{'vurlstem'};
	$href = $params{'href'};
	$onmouse = $params{'onmouse'};
	$onclick = $params{'onclick'};
	$ddbhandle = $params{'dbhandle'};

	if( $idtype < 1 )
	{
		print STDERR "id_i_fetch: no id type specified\n";
		return undef;
	}
	if( $debug > 0 )
	{
		print STDERR "id_i_fetch:".$idtype."-".$id_i_list{$idtype}."\n";
	}
	# SE style
	if( $idtype == 1 )
	{
		# if we're already in a zip page and can find real zips,
		# just go for it.
		if( $href =~ /.zip/ )
		{
			return( $href );
		}
		return undef;
	}
	# ATK Corporate
	if( $idtype == 2 )
	{
		# case for AMK
		if( $onclick =~ /galleria_zips/ )
		{
			( $dumy, $target ) = split( /\("/, $onclick, 2 );
			( $clean, $dumy ) = split( /",/, $target, 2 );
			( $target, $stem, $mid )= fileparse( $clean );
			if( $debug > 1 )
			{
				print "OnClick is: $clean ";
				print "[target=$target,stem=$stem]\n";
			}
			return( $clean );
		}
		# case for ATE
		if( $onclick = /exotics_zips/ )
		{
			( $dumy, $target ) = split( /\("/, $onclick, 2 );
			( $clean, $dumy ) = split( /",/, $target, 2 );
			( $target, $stem, $mid )= fileparse( $clean );
			if( $debug > 1 )
			{
				print "OnClick is: $clean ";
				print "[target=$target,stem=$stem]\n";
			}
			return( $clean );
		}
		# CATCH ALL: if the above fall through
		# if we're already in a zip page and can find real zips,
		# just go for it.
		if( $href =~ /.zip/ )
		{
			return( $href );
		}
	}
	# ATK Premium special
	if( $idtype == 3 )
	{
		# very simple, return the first zip file name we encounter
		if( $href =~ /.zip/ )
		{
			return( $href );
		}
		else
		{
			$href =~ s/^\.\.\///;
			if( $debug > 0 )
			{
				print STDERR
				"id_i_fetch: ATKP iframe: ".$vurlstem.
				"members/styles/standard/".$href."\n";
			}
			return( $vurlstem."members/styles/standard/".$href );
		}
	}
	# TF/AW 
	if( $idtype == 4 )
	{
		print STDERR "TF/AW style not yet supported\n";
		return undef;
	}
	# catch all
	return undef;
}

# id_gen_proto2struct 
#
# scan through the prototype link url from the vendor database and work
# out what the tokens it contains are and return them as a structure for
# subsequent use.  From GKK.
sub id_gen_proto2struct( $ )
{
	my( $proto )=@_;
	my %tokens = { }; # hashref, about to be populated

	# NB: this is an entry point from chkmodel's parsepage routine
	#     so we need to check if high level debugging is requested
	$debug=Wacs::conf_get_attr("debug","module_wacsid");
	if( $debug > 2 )
	{
		print STDERR "id_gen_proto2struct: called with ".$proto.".\n";
	}

	# Analyze prototype for subsequent matching and store in a hash (struct)
	# return a reference to that hash
	while ( $proto =~ m/#([A-Z]+)#/g)
	{
		my $token = $1;
		my $matchstring = id_gen_tokenmatchstring( $proto, $token );
		$tokens{$token} = $matchstring;
		if( $debug > 2 )
		{
			print STDERR
			"id_gen_proto2struct: $matchstring points to $token\n";
		}
	}
	# set should have the list now
	if( $debug > 1 )
	{
		print STDERR "id_gen_proto2struct: ".Dumper( %tokens )."\n";
	}
	return { prototext=>$proto, tokens=>\%tokens };
}

# id_gen_tokenmatchstring
#
# Please pass $token name without the surrounding "#" characters
# From GKK.
sub id_gen_tokenmatchstring( $$ )
{
	my ( $prototext, $token ) = @_;
	my $matchstring;
	# the variable match string in a pattern is:
	#   a "URL-element-separator-char" ( ? / ; & )
	#   followed by one or more non-"URL-element-separator-char"s
	#   followed by a hash, the variable name, and another hash
	# return the part up to, but not including, the first hash
	if ($prototext =~ m{[\?/;&]([^\?/;&]+)#$token#})
	{
		$matchstring = $1;
	}
	return $matchstring;
}

# id_gen_trymatch
#
# the iproto_struct and vproto_struct have been generated by proto2struct()
# alternatively, you can pass $iproto and $vproto (the prototype text)
# and call proto2struct() to analyze the prototypes first thing after you
# read your parameters
# From GKK
sub id_gen_trymatch( $$$ )
{
	my ( $iproto_tokens, $vproto_tokens, $url ) = @_;

	my $mid = '';
	my $sid = '';
	my $vext = '';
	my $vurl = '';
	my $token;
	my %tokens;
	my $failed = 0;
	my $vals = { };

	foreach $token (keys %{$iproto_tokens})
	{
		if( $token =~ /^HASH/ )
		{
			next;
		}
		if( $debug > 2 )
		{
			print STDERR "considering token $token...\n";
		}
		my $tokenmatch = $iproto_tokens->{$token};
		my $val = id_gen_getvarval($url, $tokenmatch);
		if (defined($val))
		{
			$vals->{$token} = $val;
			if( $debug > 3 )
			{
				print STDERR "value for $token is $val\n";
			}
		}
		else
		{
			$failed = 1;
			last;
		}
	}

	unless ($failed)
	{
		# we got a match for image url
		$mid = id_get_key;
		if( $vals->{'KEY'} ne "" )
		{
			if( $vals->{'KEY'} ne $mid )
			{
				if( $debug > 1 )
				{
					print STDERR
					"Skipping because key found ".
					"doesn't match model $mid\n";
				}
				return undef;
			}
			$mid = $vals->{'KEY'};
		}
		if( $vals->{'ALT'} ne "" )
		{
			$mid = $vals->{'ALT'};
		}
		# this bit MUST match
		$sid = $vals->{'SETKEY'};
		if( $sid eq "" )
		{
			# empty - ignore this
			return undef;
		}

		# do something (register?) with $url and the token=>value pairs
		#in %$vals
		if( $url =~ /movie/ || $url =~ /video/ )
		{
			if( $debug > 1  )
			{
				print STDERR 
				"skipping $url at image stage.\n";
			}
		}
		else
		{
			id_gen_findobj( 'site'=>id_get_vendor,
				       'type'=>'I',
				       'key'=>$mid,
				       'setkey'=>$sid,
				       'setname'=>id_get_modelname." Set ".$sid,
				       'modelno'=>id_get_modelno,
				       'url'=>$url,
				       'dbhandle'=>id_get_dbhandle );
			# return( $sid );
		}
	}
	if( $debug > 3 )
	{
		print STDERR "Continuing to video parsing...\n";
		print STDERR "vproto_tokens: ".Dumper( $vproto_tokens)."\n";
	}

	# reset for next try
	$failed = 0;
	$vals = { };

	# try vproto
	foreach $token (keys %{$vproto_tokens})
	{
		if( $token =~ /^HASH/ )
		{
			next;
		}
		if( $debug > 2 )
		{
			print STDERR "considering for video token $token...\n";
		}
		my $tokenmatch = $vproto_tokens->{$token};
		my $val = id_gen_getvarval($url, $tokenmatch);
		if (defined($val))
		{
			$vals->{$token} = $val;
			if( $debug > 3 )
			{
				print STDERR "value for $token is $val\n";
			}
		}
		else
		{
			$failed = 1;
			last;
		}
	}

	unless ($failed)
	{
		# we got a match for video url
		$mid = id_get_key;
		if( $vals->{'KEY'} ne "" )
		{
			if( $vals->{'KEY'} ne $mid )
			{
				if( $debug > 0 )
				{
					print STDERR
					"Skipping because key found ".
					"doesn't match model $mid\n";
				}
				return undef;
			}
			$mid = $vals->{'KEY'};
		}
		if( $vals->{'ALT'} ne "" )
		{
			$mid = $vals->{'ALT'};
		}
		# this bit MUST match
		$sid = $vals->{'SETKEY'};
		if( $sid eq "" )
		{
			# empty - ignore this
			return undef;
		}

		# do something (register?) with $url and the token=>value pairs
		#in %$vals
		id_gen_findobj( 'site'=>id_get_vendor,
			       'type'=>'V',
			       'key'=>$mid,
			       'setkey'=>$sid,
			       'setname'=>id_get_modelname." Movie ".$sid,
			       'modelno'=>id_get_modelno,
			       'url'=>$url,
			       'dbhandle'=>id_get_dbhandle );
	}
	return( $sid );
}

# id_vid_trymatch
# 
# this is a special version of id_gen_trymatch which only attempts a
# video match - initially created because Karup's HA use the same
# setpage for both images and video.  WHAT A REALLY DUMB IDEA - errr 
# I mean who'd be that stupid - oh yes, *US* - ooops - OK, I'll shut
# up now!
sub id_vid_trymatch( $$ )
{
	my ( $vproto_tokens, $url ) = @_;

	my $mid = '';
	my $sid = '';
	my $vext = '';
	my $vurl = '';
	my $token;
	my %tokens;
	my $failed = 0;
	my $vals = { };

	# work through vproto
	foreach $token (keys %{$vproto_tokens})
	{
		if( $token =~ /^HASH/ )
		{
			next;
		}
		if( $debug > 2 )
		{
			print STDERR "considering for video token $token...\n";
		}
		my $tokenmatch = $vproto_tokens->{$token};
		my $val = id_gen_getvarval($url, $tokenmatch);
		if (defined($val))
		{
			$vals->{$token} = $val;
			if( $debug > 3 )
			{
				print STDERR "value for $token is $val\n";
			}
		}
		else
		{
			$failed = 1;
			last;
		}
	}

	unless ($failed)
	{
		# we got a match for video url
		$mid = id_get_key;
		if( $vals->{'KEY'} ne "" )
		{
			if( $vals->{'KEY'} ne $mid )
			{
				if( $debug > 0 )
				{
					print STDERR
					"Skipping because key found ".
					"doesn't match model $mid\n";
				}
				return undef;
			}
			$mid = $vals->{'KEY'};
		}
		if( $vals->{'ALT'} ne "" )
		{
			$mid = $vals->{'ALT'};
		}
		# this bit MUST match
		$sid = $vals->{'SETKEY'};
		if( $sid eq "" )
		{
			# empty - ignore this
			return undef;
		}

		# do something (register?) with $url and the token=>value pairs
		#in %$vals
		id_gen_findobj( 'site'=>id_get_vendor,
			       'type'=>'V',
			       'key'=>$mid,
			       'setkey'=>$sid,
			       'setname'=>id_get_modelname." Movie ".$sid,
			       'modelno'=>id_get_modelno,
			       'url'=>$url,
			       'dbhandle'=>id_get_dbhandle );
		return( $sid );
	}
}

# id_gen_findobj
#
# find what the target object file is actually called
sub id_gen_findobj( % )
{
	my( %params )=@_;
	my( $fo_site, $fo_key, $fo_setkey, $fo_setname );
	my( $fo_modelno, $fo_url, $fo_dbhandle );
	my( $fo_archive, $vfileloc, $typeloc, $modelobjs );
	my( $vsql, $vcsr, $vrec );
	my( $resname, $resurl, $resarc, $dumy, $thisfile );

	$fo_site = $params{'site'};
	$fo_type = $params{'type'};
	$fo_key = $params{'key'};
	$fo_setkey = $params{'setkey'};
	$fo_setname = $params{'setname'};
	$fo_modelno = $params{'modelno'};
	$fo_url = $params{'url'};
	$fo_dbhandle = $params{'dbhandle'};

	# find out where cache files for this vendor live
	$vfileloc = vend_dnld( $fo_site, $fo_dbhandle );
	if( $fo_type eq "I" )
	{
		$typeloc = $vfileloc."/setpage-cache";
		$modelobjs = "page-".$fo_site."-".$fo_modelno."-".$fo_setkey;
	}
	elsif( $fo_type eq "V" )
	{
		$typeloc = $vfileloc."/vidpage-cache";
		$modelobjs = "vid-".$fo_site."-".$fo_modelno."-".$fo_setkey;
	}
	else
	{
		print STDERR "id_gen_findobj: unknown type $fo_type\n";
		return;
	}
	if( ! -d $typeloc )
	{
		system( "mkdir -p $typeloc" );
	}
	
	# the global variables should already be right from id_mpage
	# NB: the next 90 odd lines are similar, but not the same as
	# findvids_atkg.

        # query the vendor database for username and password, subscription
        # state and whether the models video page is retrievable
        #                0          1          2            3
        $vsql = "select vusername, vpassword, vsubscribed, vviduse, ";
        #          4        5        6        7        8        9
        $vsql .= "vimguse, vidtimg, vidtvid, vsrvvid, vsrvimg, vweburl, ";
	#          10        11
	$vsql .= "vmultimg, vmultvid ";
        $vsql .= "from ".Wacs::conf_get_attr("tables","vendor")." ";
        $vsql .= "where vsite = ? ";
        $vcsr=$fo_dbhandle->prepare( $vsql );
        $vcsr->execute( $fo_site );
	if( ! (@vrec = $vcsr->fetchrow_array) )
	{
		print STDERR 
			"id_gen_findobj: fetch of vendor $fo_site failed\n";
		if( $DBI::errstr ne "" )
		{
			print STDERR "SQL Choaked\n";
			print STDERR "Error was: $DBI::errstr\n";
			print STDERR "SQL was: $vsql\n";
		}
		return;
	}

        # cook up command line - note we're relying on the passed URL
	# rather than looking it up in the vendor database
	# NB: the result is stored in global var $wgetcmd  for future use
	$cookies=find_cookies( "firefox","cookies.txt" );
	$browser='Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.8.0.8) Gecko/20061108 Fedora/1.5.0.8-1.fc5 Firefox/1.5.0.8';
        $wgetcmd ="wget ";
        if( $cookies ne "" )
        {
                $wgetcmd .= "--load-cookies ".$cookies." ";
        }
        if( $vrec[0] ne "" )
        {
                $wgetcmd .= "--http-user=".$vrec[0]." ";
        }
        if( $vrec[1] ne "" )
        {
                $wgetcmd .= "--http-passwd=".$vrec[1]." ";
        }
        $wgetcmd  .= "-U \"". $browser."\" ";

	# sort out server prefix (in case it's needed)
	$vfileurl = $vrec[9];
	if( $fo_type eq "I" )
	{
		if( $vrec[8] ne "" )
		{
			# is a custom image file server specified -
			# if yes, store
			$vfileurl = $vrec[8];
		}
	}
	if( $fo_type eq "V" )
	{
		if( $vrec[7] ne "" )
		{
			# is a custom video file server specified -
			# if yes, store
			$vfileurl = $vrec[7];
		}
	}

	# note single quote protection for url 'cos of embedded &
	if( $fo_url =~ /http/ )
	{
		$cmdline = $wgetcmd."'".$fo_url."' -O ".$modelobjs;
	}
	else
	{
		$cmdline = $wgetcmd."'".$vfileurl.$fo_url."' -O ".$modelobjs;
	}
	chdir( $typeloc );
	# check for by-type page useability rules
	if( $fo_type eq "V" && $vrec[3] eq "Y" )
        {
		# video index page usable
		if( $debug > 2 )
		{
			print STDERR "Getting Model ".$fo_type.
						"-type Page: $cmdline\n";
		}
		# NB: don't try and get this file again if we have it
		if( ! -f $modelobjs )
		{
			system( $cmdline );
		}
	}
	elsif( $fo_type eq "I" && $vrec[4] eq "Y" )
	{
		# image index page usable
		if( $debug > 2 )
		{
			print STDERR "Getting Model ".$fo_type.
						"-type Page: $cmdline\n";
		}
		# NB: don't try and get this file again if we have it
		if( ! -f $modelobjs )
		{
			system( $cmdline );
		}
	}
	else
	{
		print STDERR 
			"Model's ".$fo_type."-type page marked as unusable\n";
		print STDERR "please Save As $modelobjs in $typeloc/.\n";
		# add a download record for this marked as incomplete so we
		# can at least identify that it exists
		dnld_checkadd( 'site'=>$fo_site,
				'status'=>'X',
				'type'=>$fo_type,
				'key'=>$fo_key,
				'setkey'=>$fo_setkey,
				'setname'=>$fo_setname,
				'modelno'=>$fo_modelno,
				'url'=>'',
				'archive'=>'',
				'dbhandle'=>$fo_dbhandle );
	}

	# check the file does appear
	if( ! -f "$typeloc/$modelobjs" )
	{
		print STDERR "id_gen_findobj: download of $modelobjs failed.\n";
		return;
	}

	# NB: fo_type is a global for the benefit of fo_start which needs
	# to know this
	$fo_result = '';   # this is a global modified by fo_start
	$fo_multiple = 0;
	$fo_count = 0;
	@fo_names=();

	# the parser should pay attention to the vmultimg and vmultvid
	# records above (vrec[10] and vrec[11] respectively).
	if( $fo_type eq "I" )
	{
		if( $vrec[10] eq "Y" )
		{
			$fo_multiple=1;
		}
	}
	elsif( $fo_type eq "V" )
	{
		if( $vrec[11] eq "Y" )
		{
			$fo_multiple=1;
		}
	}

	# create and invoke the HTML parser
	$parser = HTML::Parser->new( api_version => 3,
				start_h => [\&fo_start, "tagname, attr"],
				text_h => [\&fo_text, "dtext"],
				);
	if( $debug > 2 )
	{
		print STDERR "id_gen_findobj: invoking parser on $modelobjs.\n";
	}
	$parser->parse_file( "$typeloc/$modelobjs" );
	if( $debug > 2 )
	{
		print STDERR "id_gen_findobj: parsing done on $modelobjs.\n";
	}

	# cleanup and be ready for the next call
	chdir( "$vfileloc" );

	if( $fo_result ne "" )
	{
		if( $debug > 3 )
		{
			print STDERR "id_gen_findobj: parse found:".
				$fo_result."\n";
		}
		if( $fo_multiple == 0 )
		{
			($resarc, $resurl, $dumy)= fileparse( $fo_result );
			if( $debug > 4 )
			{
				print STDERR "id_gen_findobj: single zip:".
					$resarc." at ".$resurl.
					" to be added.\n";
			}
			dnld_checkadd( 'site'=>$fo_site,
					'type'=>$fo_type,
					'key'=>$fo_key,
					'setkey'=>$fo_setkey,
					'setname'=>$fo_setname.": ".$resarc,
					'modelno'=>$fo_modelno,
					'url'=>$fo_result,
					'archive'=>$resarc,
					'dbhandle'=>$fo_dbhandle );		
		}
		else
		{
			# work through the stored names
			while( $fo_count )
			{
				print STDERR "fo_names[".($fo_count-1)."]: ".
						$fo_names[($fo_count-1)]."\n";
				($resarc, $resurl, $dumy)= fileparse( 
					$fo_names[($fo_count-1)]);
				if( $fo_count > 1 )
				{
					$thisfile = $fo_setkey."-".
							($fo_count-1);
				}
				else
				{
					$thisfile = $fo_setkey;
				}
				if( $debug > 4 )
				{
					print STDERR 
					  "id_gen_findobj: multiple zips:".
					  $resarc." at ".$resurl.
					  " to be added.\n";
				}
				dnld_checkadd( 'site'=>$fo_site,
                                        'type'=>$fo_type,
                                        'key'=>$fo_key,
                                        'setkey'=>$thisfile,
                                        'setname'=>$fo_setname.": ".$resarc,
                                        'modelno'=>$fo_modelno,
                                        'url'=>$fo_names[($fo_count-1)],
                                        'archive'=>$resarc,
                                        'dbhandle'=>$fo_dbhandle );
				$fo_count--;
			}
		}
	}
}

# id_gen_getvarval
#
# match routine used above - also from GKK
sub id_gen_getvarval( $$ )
{
	my ( $url, $varmatch ) = @_;
	my $val;
	if ( $url =~ m{[\?/;&]$varmatch([^\?/;&]+)[\?/;&]?} )
	{
		$val = $1;
	}
	return $val;
}

# findvids_atkg
#
# find videos from the model's video page for ATK Group companies
sub findvids_atkg( % )
{
        my( %params )=@_;
        my( $dsite, $dkey, $dsetkey, $dmodel, $dtype, $dsetname, $dnotes );
        my( $dduration, $dimgcount, $durl, $darchive, $ddbhandle, $mname );
	my( $modelvids, $dmodelid, $cmdline, $browser, $cookies, $allowuse );
	my( $vsql, $vcsr, @vrec, $parser );

	$dsite = $params{'site'};
	$dkey = $params{'key'};
	$dsetkey = $params{'setkey'};
	$dmodel = $params{'modelno'};
	$dmodelid = $params{'modelkey'};
	$mname = $params{'name'};
	$dtype = $params{'type'};
	$dsetname = $params{'setname'};
	$dnotes = $params{'notes'};
	$dduration = $params{'duration'};
	$dimgcount = $params{'imgcount'};
	$durl = $params{'url'};
	$darchive = $params{'archive'};
	$ddbhandle = $params{'dbhandle'};

	# OK, where do we put our work files
	$vfileloc = vend_dnld( $dsite, $ddbhandle );
	if( ! -d "$vfileloc/vidpage-cache" )
	{
		system( "mkdir -p $vfileloc/vidpage-cache" );
	}
	chdir( "$vfileloc/vidpage-cache" );

	# work out the name of this file
	if( $dsite eq "ATKP" )
	{
		# per video pages in ATKP
		$modelvids="vid-$dsite-$dmodel-$dsetkey";
	}
	else
	{
		# other sites have a per-model video page
		$modelvids="vid-$dsite-$dmodel";
	}

	# reset the global attributes and then set a couple we need in
	# the parser
	reset_attr( 'video' );
	$id_modelno = $dmodel;
	$id_modelid = $dmodelid;
	$id_modelname = $mname;
	$id_vendor = $dsite;
	$id_dbhandle = $ddbhandle;
	$id_key = $dkey;
	$id_setkey = $dsetkey;
	$id_setname = $dsetname;
	$id_notes = $dnotes;


        # query the vendor database for username and password, subscription
        # state and whether the models video page is retrievable
        #                0          1          2            3
        $vsql = "select vusername, vpassword, vsubscribed, vmviduse, ";
        #          4        5        6        7        8
        $vsql .= "vidtimg, vidtvid, vsrvvid, vweburl, vviduse ";
        $vsql .= "from ".Wacs::conf_get_attr("tables","vendor")." ";
        $vsql .= "where vsite = ? ";
        $vcsr=$ddbhandle->prepare( $vsql );
        $vcsr->execute( $dsite );
	if( ! (@vrec = $vcsr->fetchrow_array) )
	{
		print STDERR "findvids_atkg: fetch of vendor $dsite failed\n";
		if( $DBI::errstr ne "" )
		{
			print STDERR "SQL Choaked\n";
			print STDERR "Error was: $DBI::errstr\n";
			print STDERR "SQL was: $vsql\n";
		}
		return;
	}

        # cook up command line - note we're relying on the passed URL
	# rather than looking it up in the vendor database
	# NB: the result is stored in global var $wgetcmd  for future use
	$cookies=find_cookies( "firefox","cookies.txt" );
	$browser='Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.8.0.8) Gecko/20061108 Fedora/1.5.0.8-1.fc5 Firefox/1.5.0.8';
        $wgetcmd ="wget ";
        if( $cookies ne "" )
        {
                $wgetcmd .= "--load-cookies ".$cookies." ";
        }
        if( $vrec[0] ne "" )
        {
                $wgetcmd .= "--http-user=".$vrec[0]." ";
        }
        if( $vrec[1] ne "" )
        {
                $wgetcmd .= "--http-passwd=".$vrec[1]." ";
        }
        $wgetcmd  .= "-U \"". $browser."\" ";
	if( $vrec[6] ne "" )
	{
		# is a custom video file server specified - if yes, store
		$vfileurl = $vrec[6];
	}
	else
	{
		# standard location it is then
		$vfileurl = $vrec[7];
	}

	# note single quote protection for url 'cos of embedded &
	if( $durl =~ /^\.\.\// )
	{
		# strip off a ../ start if there is one
		$durl =~ s/^\.\.\///;
	}
	if( $durl =~ /http/ )
	{
		$cmdline = $wgetcmd."'".$durl."' -O ".$modelvids;
	}
	else
	{
		$cmdline = $wgetcmd."'".$vfileurl.$durl."' -O ".$modelvids;
	}
	# are we allowed to pull this page for ourselves?
	if( $dsite eq "ATKP" )
	{
		$allowuse = $vrec[8];
	}
	else
	{
		$allowuse = $vrec[3];
	}
	# then do it....
	if( $allowuse eq "Y" )
        {
		if( $debug > 2 )
		{
			print STDERR "Getting Model Video Page: $cmdline\n";
		}
		system( $cmdline );
	}
	else
	{
		print STDERR "Model's Video index page marked as unusable\n";
		print STDERR "please Save As $modelvids in $vfileloc/";
		print STDERR "vidpage-cache.\n";
	}

	# check the file does appear
	if( ! -f "$vfileloc/vidpage-cache/$modelvids" )
	{
		print STDERR "findvids_atkg: download of $modelvids failed.\n";
		return;
	}

	# create and invoke the HTML parser
	$parser = HTML::Parser->new( api_version => 3,
				start_h => [\&fv_start, "tagname, attr"],
				text_h => [\&fv_text, "dtext"],
				);
	if( $debug > 1 )
	{
		print STDERR
			"findvids_atkg: running parser on ".$modelvids."\n";
	}
	$parser->parse_file( "$vfileloc/vidpage-cache/$modelvids" );
	if( $debug > 1 )
	{
		print STDERR
			"findvids_atkg: completed parsing ".$modelvids."\n";
	}

	# cleanup and be ready for the next call
	chdir( "$vfileloc" );
	$modelvids='';
}

# fv_start
#
# find vids start new tag callback routine
sub fv_start( $$ )
{
	my( $tagname, $attr ) = @_;
	my( $target, $gallery, $dumy, $iconurl, $vicondir, $setname );

	if( $tagname eq "td" )
	{
		$fv_catchnext = 1;
	}
	if( $tagname eq "/td" )
	{
		$fv_catchnext = 0;
	}
	if( $tagname eq "a" )
	{
		$url = $attr->{href};
		if( $url =~ /mailto/ )
		{
			return;
		}
		if( $debug > 3 )
		{
			print STDERR "fv_start: URL is: $url\n";
		}

		# ATKP has a video page per movie but we don't need to 
		# worry about that, this should still work.
		if( id_get_vendor eq "ATKP" )
		{
			if( chk_vid_type( $url ) )
			{
				($target, $gallery, $dumy) = fileparse( $url );
				print STDERR "fv_start: found movie ".$target.
					" in ".$gallery."\n";
				id_v_atkp( id_get_vendor, $target, 
					   id_get_notes, id_get_setname,
					   Wacs::conf_get_attr("tables",
						"photographer"),
					   id_get_flag, id_get_dbhandle );
				dnld_checkadd( 'site'=>id_get_vendor,
                                                'type'=>'V',
                                                'key'=>id_get_key,
                                                'setkey'=>id_get_setkey,
                                                'setname'=>id_get_setname,
                                                'modelno'=>id_get_modelno,
						'notes'=>id_get_notes,
                                                'url'=>$url,
                                                'archive'=>$target,
						'dbhandle'=>id_get_dbhandle );
			}
		}
		# we have a usable index right here
		if( chk_vid_type( $url ) )
		{
			($target, $gallery, $dumy) = fileparse( $url );
			print STDERR "fv_start: found movie ".$target.
				" in ".$gallery."\n";
			dnld_checkadd( 'site'=>id_get_vendor,
					'type'=>'V',
					'key'=>id_get_key,
					'setkey'=>$target,
					'setname'=>$target,
					'modelno'=>id_get_modelno,
					'url'=>$url,
					'archive'=>$target,
					'dbhandle'=>id_get_dbhandle );
		}
	}
	if( $tagname eq "img" )
	{
		if( $debug > 3 )
		{
			print STDERR "fv_start: img tag found: ".$attr->{src}.
					"\n";
		}
		# now for the video thumb icon check the directory exists
		if( id_get_modelname ne "" )
		{
			$vicondir = $vfileloc."/icons/".id_get_modelname."_".
						id_get_modelid;
			$vicondir =~ s/\s//g;
		}
		else
		{
			if( $debug > 0 )
			{
				print STDERR "fv_start: using ".
					"fallback icon location.\n";
			}
			$vicondir = $vfileloc."/vidicons";
		}
		# is the directory there
		if( ! -d $vicondir )
		{
				system("mkdir -p $vicondir");
		}
		chdir( "$vicondir" );
		if( $debug > 1 )
		{
			print STDERR "fv_start: moving to ".$vicondir."\n";
		}

		# check if the icon is already there.
		$iconurl = $attr->{src};
		($target, $gallery, $dumy) = fileparse( $iconurl );
		if( -f "$target" )
		{
			if( $debug > 4 )
			{
				print STDERR "fv_start: ".$vicondir."/".$target.
						" already exists\n";
			}
			# yes, it is....
			return;
		}
		# check for including the http bit (and site name)
		if( $iconurl =~ /http/ )
		{
			system( $wgetcmd."'".$iconurl."'" );
		}
		else
		{
			system( $wgetcmd."'".$vfileurl."/".$iconurl."'" );
		}
		# end of img tag
	}
}

# fv_text
#
# find vids text callback routine
sub fv_text( $ )
{
	my( $text ) = @_;

	return unless $fv_catchnext eq 1;
	chomp( $text );
	$text =~ s/\s+$//g;
	$text =~ s/^\s+//g;
	if( $text eq "?" )
	{
		$text = '';
	}
	if( lc( $text ) =~ /toys/ )
	{
		dnld_update( 'dnldno'=>id_get_dnldno,
			     'field'=>'setflag',
			     'value'=>'T',
			     'dbhandle'=>id_get_dbhandle );
		$fv_catchnext=0;
	}
	if( lc( $text ) =~ /masturbation/ )
	{
		dnld_update( 'dnldno'=>id_get_dnldno,
			     'field'=>'setflag',
			     'value'=>'M',
			     'dbhandle'=>id_get_dbhandle );
		$fv_catchnext=0;
	}
	if( lc( $text ) =~ /action/ )
	{
		dnld_update( 'dnldno'=>id_get_dnldno,
			     'field'=>'setflag',
			     'value'=>'F',
			     'dbhandle'=>id_get_dbhandle );
		$fv_catchnext=0;
	}
	if( lc( $text ) =~ /lesbian/ )
	{
		dnld_update( 'dnldno'=>id_get_dnldno,
			     'field'=>'setflag',
			     'value'=>'L',
			     'dbhandle'=>id_get_dbhandle );
		$fv_catchnext=0;
	}
}

# fo_start
#
# parser callback function for id_gen_findobj
sub fo_start( $$ )
{
	my( $tagname, $attr ) = @_;
	my( $url );

	if( $tagname eq "a" )
	{
		# to improve with matching (ie in case they're .ZIP,
		# .WMV, etc) we lower case what we test against
		$url = lc( $attr->{href} );
		if( $url =~ /mailto/ )
		{
			return;
		}
		if( $fo_type eq "I" )
		{
			# FIXME: this does not work for multiples
			if( $url =~ /.zip/ )
			{
				# capture the first one  (FIXME: multiples)
				if( $fo_result eq "" )
				{
					if( $debug > 4 )
					{
						print STDERR
						 "found zip file in ".
						 $attr->{href}.
						 " -- returning.\n";
					}
					$fo_result = $attr->{href};
					return;
				}
			}
		}
		if( $fo_type eq "V" )
		{
			if( $fo_multiple == 0 )
			{
				# only looking for one entry
				if( $fo_result eq "" )
				{
					if( chk_vid_type( $url ) )
					{
						$fo_result = $attr->{href};
						return;
					}
				}
			}
			else
			{
				if( debug > 3 )
				{
					print STDERR "fo_start: ".
						"video multiple mode: ".
						"adding ".$url."\n";
				}
				if( $fo_result eq "" )
				{
					if( chk_vid_type( $url ) )
					{
						$fo_result = $attr->{href};
						$fo_names[$fo_count]=
							$attr->{href};
						$fo_count++;
					}
				}
				elsif( chk_vid_type( $url ) )
				{
					$fo_names[$fo_count]=
						$attr->{href};
					$fo_count++;
				}
			}
		}
	}
}

# fo_text
#
# parser callback function for id_gen_findobj
sub fo_text( $ )
{
	my( $text ) = @_;
}

# chk_vid_type
#
# check to see if this url is a video file type
sub chk_vid_type( $ )
{
	my( $testurl )=@_;

	# test for MPEG-1 and MPEG-2 movie types
	if( $testurl =~ /\.mpg/ )
	{
		return 1;
	}
	if( $testurl =~ /\.MPG/ )
	{
		return 1;
	}
	if( $testurl =~ /\.mpeg/ )
	{
		return 1;
	}
	# test for MPEG-4 aka DiVX ;-) movie types
	if( $testurl =~ /\.mp4/ )
	{
		return 1;
	}
	# test for Windows Media movie types
	if( $testurl =~ /\.wmv/ )
	{
		return 1;
	}
	if( $testurl =~ /\.WMV/ )
	{
		return 1;
	}
	# test for QuickTime movie types
	if( $testurl =~ /\.mov/ )
	{
		return 1;
	}
	if( $testurl =~ /\.MOV/ )
	{
		return 1;
	}
	# test for older AVI files
	if( $testurl =~ /\.avi/ )
	{
		return 1;
	}
	if( $testurl =~ /\.AVI/ )
	{
		return 1;
	}

	# no matches found
	return 0;  # false
}

sub chkid_existing( % )
{
	my( %params )=@_;
	my( $chksite, $chkkey, $chkalt, $chkname, $chkuseold, $chkdbhandle );
	my( $chksql, $chkcsr, @chkrec, $keyless );

	$chksite = $params{'site'};
	$chkkey = $params{'key'};
	$chkalt = $params{'altkey'};
	$chkname = $params{'name'};
	$chkid = $params{'useold'};
	$chkdbhandle = $params{'dbhandle'};
	$keyless = 0;

	# when it finds a matching idmap, it should return our model
	# number - zero means not found.
	$chksql = "select imodelno, iname, isite, ikey, ialtkey, iactive ";
	$chksql.= "from ".Wacs::conf_get_attr("tables","idmap")." ";
	$chksql.= "where isite = ? ";
	if( $chkkey ne "" )
	{
		$chksql.= "and ikey = ? ";
	}
	else
	{
		$chksql.= "and iname = ? ";
		$keyless = 1;
	}
	if( $chkalt ne "" )
	{
		$chksql.= "and ialtkey = ? ";
	}
	# only consider very active and dormant active IDmaps, not
	# incomplete or obsolete ones.
	if( $chkuseold ne "Y" )
	{
		$chksql.= "and iactive in ('Y','D') ";
	}
	$chkcsr = $chkdbhandle->prepare( $chksql );
	if( $chkalt eq "" )
	{
		if( $keyless == 1 )
		{
			$chkcsr->execute( $chksite, $chkname );
		}
		else
		{
			$chkcsr->execute( $chksite, $chkkey );
		}
	}
	else
	{
		$chkcsr->execute( $chksite, $chkkey, $chkalt );
	}
	if( $DBI::errstr ne "" )
	{
		print STDERR "SQL Choaked\n";
		print STDERR "Error was: ".$DBI::errstr."\n";
		print STDERR "Query was: ".$chksql."\n";
		exit -1;
	}
	@chkrec = $chkcsr->fetchrow_array;
	if( $chkrec[0] > 0 )
	{
		# found her
		if( $debug > 0 )
		{
			print STDERR "chkid_existing: model found as ".
				$chkrec[1]." (".$chkrec[0].")\n";
		}
		return( $chkrec[0] );
	}

	return( 0 );
}

# show that we have parsed correctly
1;
__END__

=head1 NAME

WacsId - Web Adult Content Server - Identifier Module

=head1 SYNOPSIS

ident_img() - Identify characteristics of an image set from download info
ident_vid() - Identify characteristics of a video clip from download info
id_get_flag() - get previously determined flag (run ident_* first)
id_get_info() - get previously determined catinfo (run ident_* first)
id_get_photog() - get previously determined photographer (run ident_* first)
dnld_markdone() - mark the download as having been done
dnld_checkadd() - check and add download record if it does not exist
dnld_update() - modify a download record
find_cookies() - find the cookies file associated with named web browser
vid_getsize() - get the size of a video clip
extractphotog() - find the photographer name (only works on ATK at present)
find_namestem() - find the common stem of a file name
id_i_fetch() - vendor specific fetch techniques
chkid_existing() - check to see if we already have the specified idmap

=head1 SEE ALSO

Wacs

=cut
