#
#    WacsStd.PM - standard building blocks for Wacs Web Apps
#
#    (C) Copyright 2007,2008,2009,2010 B King
#    This file is part of WACS.
#
#    WACS is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 3 of the License, or
#    (at your option) any later version.
#
#    WACS is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with WACS.  If not, see <http://www.gnu.org/licenses/>.
#

package Wacs::WacsStd;

require Exporter;
use Wacs::WacsUI;
use XML::Simple;
use Data::Dumper;

@ISA 		= qw(Exporter);
@EXPORT 	= qw(masthead modelheads findmodel findrecentsets 
		     findrecentmodels modelheadshot getgallery callframe
		     kwscore_reset kwscore_get kwscore_process
		     removedups removeconflicts addassoc alloc_nextkey);

# global variables
@kwscore_loc=();        # keyword scores for locations
@kwscore_cat=();        # keyword scores for catflag
@kwscore_det=();	# keyword scores for detailed location
@kwscore_attr=();	# keyword scores for attributes
@kwscore_other=();	# keyword scores for other
$kwsshown=0;		# have we shown the results table yet?

# set the debug level
$debug=Wacs::conf_get_attr("debug","module_wacsstd");

# 
# masthead
#
# draw a page mast head
sub masthead( % )
{
	my( %params )=@_;
	my( $mhsetno, $mhstype, $mhsarea, $mhscategory, $mhsdirectory );
	my( $mhstitle, $mhsofftitle, $mhsimages, $mhsformat, $mhsphotog );
	my( $mhsindexes, $mhsaspect, $mhsdurhrs, $mhsdurmin, $mhsdursec );
	my( $mhsfoundry, $mhmodelno, $mhdownloadno, $mhscatflag, $mhwidth );
	my( $mhaddlinks, $mhuseicon );
	my( $prettytitle, $bgcolour, $mh_fore, $mh_back, $mh_high );
	my( $dbhandle, %typenames );

	# get the parameters given
	$mhsetno=$params{'setno'};
	$mhstype=$params{'stype'};
	$mhscatinfo=$params{'scatinfo'};
	$mhscatflag=$params{'scatflag'};
	$mhstitle=$params{'stitle'};
	$mhsofftitle=$params{'sofftitle'};
	$mhsarea=$params{'sarea'};
	$mhscategory=$params{'scategory'};
	$mhsdirectory=$params{'sdirectory'};
	$mhsimages=$params{'simages'};
	$mhsindexes=$params{'sindexes'};
	$mhsaspect=$params{'saspect'};
	$mhsformat=$params{'sformat'};
	$mhsdurhrs=$params{'sdurhrs'};
	$mhsdurmin=$params{'sdurmin'};
	$mhsdursec=$params{'sdursec'};
	$mhsphotog=$params{'sphotog'};
	$mhsfoundry=$params{'sfoundry'};
	$mhmodelno=$params{'modelno'};
	$mhdownloadno=$params{'downloadno'};
	$mhaddlinks=$params{'addlinks'};
	$mhuseicon=$params{'useicon'};
	$mhwidth=$params{'width'};
	$dbhandle=$params{'dbhandle'};

	# work out the page background colour
	if( $mhstype eq "I" )
	{
		$bgcolour = Wacs::conf_get_attr("colours","setbackground");
	}
	elsif( $mhstype eq "V" )
	{
		$bgcolour = Wacs::conf_get_attr("colours","vidbackground");
	}
	elsif( $mhstype eq "D" )
	{
		$bgcolour = Wacs::conf_get_attr("colours","dvdbackground");
	}
	else
	{
		$bgcolour = Wacs::conf_get_attr("colours","background");
	}
	$mh_fore = Wacs::conf_get_attr("colours","mastheadforeground");
	$mh_back = Wacs::conf_get_attr("colours","mastheadbackground");
	$mh_high = Wacs::conf_get_attr("colours","mastheadhighlight");
	%typenames = Wacs::geticonlist( "types" );

	# start the table
	if( $mhwidth ne "" )
	{
		print "<table border=1 width=".$mhwidth.
			" bgcolor=".$bgcolour.">\n";
	}
	else
	{
		print "<table border=1 bgcolor=".$bgcolour.">\n";
	}
	print " <tr bgcolor=\"".$mh_back."\"><td align=left><font color=\"".
		$mh_fore."\">\n";
	if( int( $mhsetno ) > 0 )
	{
		# now requests a standard size icon rather than whatever
		# size it actually is.
		Wacs::WacsUI::iconlink( 'type'=> $mhstype,
			  'setno'=>$mhsetno,
			  'sarea'=>$mhsarea,
			  'scategory'=>$mhscategory,
			  'sdirectory'=>$mhsdirectory,
			  'resize'=>1 );
	}
	else
	{
		if( $mhuseicon eq "n" )
		{
			print "[No Icon Yet]\n";
		}
		else
		{
			# this thumb thing might work for both as the only
			# jpeg present in unpack for a video is the first
			# icon.
			print "<a href=\"";
			print Wacs::conf_get_attr("server","cgiurl");
			print "/wacsindex/page".$mhsetno.".html\">\n";
			print "<font color=\"".$mh_fore."\">";
			print "<img src=\"";
			print Wacs::conf_get_attr("server","cgiurl");
			print "/wacsthumb/set".$mhsetno."-1.jpg\">";
			print "</font></a>\n";
		}
	}
	print "  </font></td><td align=center>";
	# prepare any modifications we need
	$prettytitle = Wacs::divideup( $mhstitle, '<br>', $thumbsmode );
	# prettifier from GKK
	$prettytitle =~ s,(\w)([A-Z][a-z]),$1 $2,g;
	$prettytitle =~ s,([a-z])([A-Z]),$1 $2,g;
	# set information
	if( $mhsetno > 0 )
	{
		print "<font size=-1 color=\"".$mh_fore."\">Set ".$mhsetno;
	}
	elsif( $mhsetno == 0 && $mhdownloadno > 0 )
	{
		print "<font size=-1 color=\"".$mh_fore."\">Unpacking ".
			"<a href=\"".Wacs::conf_get_attr("server","cgiurl").
			"wacsdnlmgr/".$mhdownloadno."\">Download #".
			$mhdownloadno."</a>\n";
	}
	else
	{
		print "<font size=-1 color=\"".$mh_fore."\">".
			"Unpacking Manually Uploaded Set\n";
	}
	if( $mhstype eq "I" )
	{
		print " - ".$mhsimages." images";
		if( $mhsindexes > 0 )
		{
			print ", ".$mhsindexes." index image(s)";
		}
	}
	elsif( $mhstype eq "V" )
	{
		print " - ";
		if( $mhsaspect ne "" )
		{
			print $mhsaspect." ";
		}
		if( $mhsformat ne "" )
		{
			print $mhsformat." ";
		}
		print "video clip";
		if( $mhsdurhrs > 0 || $mhsdurmin > 0 || $mhsdursec > 0 )
		{
			printf( " (%dh %-2.2dm %-2.2ds)",
				$mhsdurhrs, $mhsdurmin, $mhsdursec );
		}
		if( $mhsbytes > 1 )
		{
			printf ", %5.2fMB",($mhsbytes /(1024*1024));
                }
	}
	elsif( $mhstype eq "D" )
	{
		print " - dvd scene";
	}
	else
	{
		print "file type ".$mhstype;
	}
	if( $mhsphotog ne "" )
	{
		print " by ".$mhsphotog;
	}
	print "<br>\n";
	if( $mhscatflag ne "" )
	{
		print $typenames{$mhscatflag}." ";
		if( $mhstype eq "I" )
		{
			print "image set ";
		}
		elsif( $mhstype eq "V" )
		{
			print "video clip ";
		}
		else
		{
			print "set ";
		}
	}
	if( $mhsfoundry ne "" && $mhsfoundry ne "unknown" )
	{
		print "from ".$mhsfoundry."\n";
	}
	print "</font>\n";
	print "<h3><font color=\"".$mh_high."\">".$prettytitle.
		"</font></h3>\n";
	if( $mhsofftitle ne "" )
	{
		print "<font color=\"".$mh_fore."\">".$mhsofftitle.
			"</font><br>\n";
	}
	Wacs::WacsUI::addkeyicons( $mhscatinfo, 48 );
	if( $mhaddlinks eq "y" )
	{
		if( $mhstype eq "I" )
		{
			print "<a href=\"".
				Wacs::conf_get_attr("server","cgiurl").
				"/wacsindex/page".$mhsetno.".html\">";
			print "<font color=\"".$mh_fore.
				"\">browse this set (paged)";
			print "</font></a><br>";
			print "<font color=\"".$mh_fore."\">or</font> ";
			print "<a href=\"".
				Wacs::conf_get_attr("server","cgiurl");
			print "/wacsindex/set".$mhsetno.".html\">";
			print "<font color=\"".$mh_fore."\">whole set (index)".
				"</font>";
			print "</a><br>\n";
		}
		elsif( $mhstype eq "V" )
		{
			print "<a href=\"".
				Wacs::conf_get_attr("server","cgiurl");
			# FIXME: work out correct file extension
			print "/wacszip/set".$mhsetno.".mpg\">";
			print "<font color=\"".$mh_fore.
				"\">view this movie</font></a>";
			print "<br><font color=\"".$mh_fore."\">or</font> ";
			print "<a href=\"".
				Wacs::conf_get_attr("server","cgiurl");
			print "/wacsindex/set".$mhsetno.".html\">";
			print "<font color=\"".$mh_fore.
				"\">see details and icon(s)";
			print "</font></a><br>\n";
		}
		else
		{
			print "Unrecognised type: ".$mhstype."\n";
		}
	}
	print "  </td><td align=right>";
	if( $mhsetno > 0 )
	{
		modelheads( 'byset', $mhsetno, $dbhandle );
	}
	elsif( $mhmodelno > 0 )
	{
		modelheads( 'byno', $mhmodelno, $dbhandle );
	}
	print " </td></tr>\n";
}

sub modelheads( $$$ )
{
	my( $method, $setnumber, $mhdbhandle )=@_;
	my( $mhlsql, $mhlcsr, @mhlrec );
	my( $mh_rows, $mh_fore );

	$mh_rows = 0;
	$mh_fore = Wacs::conf_get_attr("colours","mastheadforeground");

	$mhlsql = "select modelno, mname, mimage, mrating ";
	$mhlsql.= "from ".Wacs::conf_get_attr("tables","models")." ";
	if( $method eq "byset" )
	{
		$mhlsql.= ", ".Wacs::conf_get_attr("tables","assoc")." ";
		$mhlsql.= "where modelno = amodelno and asetno = ? ";
	}
	elsif( $method eq "byno" )
	{
		$mhlsql.= "where modelno = ? ";
	}
	else
	{
		# default is todays new models (it's a default!)
		$mhlsql.= "where madded = '".Wacs::gettoday."' ";
	}
	$mhlcsr=$mhdbhandle->prepare( $mhlsql );
	$mhlcsr->execute( $setnumber );
	if( $DBI::errstr ne "" )
	{
		print "SQL Query Failed\n";
                print "Error Was: ".$DBI::errstr."\n";
                print "Query Was: ".$mhlsql."\n";
                return;
        }
	print "<table><tr>";
	while( @mhlrec = $mhlcsr->fetchrow_array )
	{
		print "<td align=center valign=top>";
		print "<a href=\"".Wacs::conf_get_attr("server","cgiurl");
		print "wacsmodelpage/".$mhlrec[0]."\"><font color=\"".
			$mh_fore."\">";
		print "<img src=\"".Wacs::conf_get_attr("server","siteurl");
		print "icons/".$mhlrec[2]."\" ";
		print "alt=[".$mhlrec[1]."]><br>".$mhlrec[1]."<br>";
		print $mhlrec[0]." <font size=-2>rated:";
		print $mhlrec[3]."/5</font></a></td>\n";
		$rows++;
		if( $rows == 3 )
		{
			print "</tr><tr>\n";
			$rows=0;
		}
	}
	print "</tr></table>\n";
}

# findmodel
# 
# puts up a multiple choice form consisting of models who match the
# name given.
sub findmodel( % )
{
	my( %params )=@_;
	my( $fmmname, $fmofferalt, $fmoffervalue, $fmoffercapt, $fmdbhandle );
	my( $fmcgihandle, $fmincsubmit );
	my( $fmshowmodel, $fmcols, $fmcount, @radiobuttons );
	my( $fmmsql, $fmmcsr, @fmmrec );
	my( $fmisql, $fmicsr, @fmirec );
	my( %modellist, @modelarray, $name, $howcome, $fmwhere, $dumy );

	if( $debug > 3 )
	{
		print "<!--- ".Dumper( %params )." --->\n";
	}
	$fmmname = $params{'mname'};
	$fmofferalt = $params{'offeralt'};
	$fmoffervalue = $params{'offervalue'};
	$fmoffercapt = $params{'offercapt'};
	$fmincsubmit = $params{'incsubmit'};
	$fmdbhandle = $params{'dbhandle'};
	$fmcgihandle = $params{'cgihandle'};
	
	$fmwhere = '';
	$fmcols = 0;

	print "<center>\n";
	print "<table border=1 width=100%>\n";
	print " <tr><th colspan=4>";
	print "Find The Models Called ".$fmmname."</th></tr>\n";

	if( $fmmname eq "" )
	{
		print "Can't search by name with no name given!<br>\n";
		return null;
	}

	# primary name
	$fmmsql = "select modelno, mname from ".
			Wacs::conf_get_attr("tables","models")." ";
	$fmmsql.= "where mname like '".$fmmname."%' ";
	$fmmsql.= "or maliases like '%".$fmmname."%' ";
	$fmmcsr = $fmdbhandle->prepare( $fmmsql );
	$fmmcsr->execute;
	if( $DBI::errstr ne "" )
	{
		print "**** ERROR ****<br>\n";
		print "Routine: findmodel (WacsStd/main)<br>\n";
		print "Error  : <tt>".$fmmname."</tt><br>\n";
		print "SQL was: <tt>".$fmmsql."</tt><br>\n";
		return null;
	}
	while( @fmmrec = $fmmcsr->fetchrow_array )
	{
		if( $fmmrec[0] ne "" )
		{
			$modellist{ $fmmrec[0] } = $fmmrec[1]."|P";
		}
	}

	# now check for secondary names
	$fmisql = "select imodelno, iname, isite from ".
			Wacs::conf_get_attr("tables","idmap")." ";
	$fmisql.= "where iname like '".$fmmname."%' ";
	$fmicsr = $fmdbhandle->prepare( $fmisql );
	$fmicsr->execute;
	if( $DBI::errstr ne "" )
        {
                print "**** ERROR ****<br>\n";
                print "Routine: findmodel (WacsStd,idmap)<br>\n";
                print "Error  : <tt>".$fmmname."</tt><br>\n";
                print "SQL was: <tt>".$fmisql."</tt><br>\n";
        }
        while( @fmirec = $fmicsr->fetchrow_array )
        {
                if( $fmirec[0] ne "" )
                {
			if( ! exists ( $modellist{$fmirec[0]} ) )
			{
				$modellist{ $fmirec[0] } = $fmirec[1].
				"|S-".$fmirec[2];
			}
                }
        }
	
	print "<tr>\n";
	# now have a list of models; transfer across
	$fmcount=0;
	foreach $fmshowmodel ( sort { $a<=>$b } keys %modellist )
	{
		$modelarray[$fmcount]=$fmshowmodel;
		$fmcount++;
	}
	# add a "none of the above" option
	if( $fmofferalt eq "y" )
	{
		if( $debug > 2 )
		{
			print "<!--- offering alternative: value=".
				$fmoffervalue." desc=".$fmoffercapt." --->\n";
		}
		$modelarray[$fmcount] = $fmoffervalue;
	}
	@radiobuttons = $fmcgihandle->radio_group(-name=>'modelno',
						-values=>\@modelarray );
	# now display
	$fmcount=0;
	foreach $fmshowmodel ( sort { $a<=>$b } keys %modellist )
	{
		$fmwhere = '';
		( $name, $howcome )= split( /\|/,$modellist{$fmshowmodel},2 );
		if( $howcome =~ /^S-/ )
		{
			( $dumy, $fmwhere ) = split( /-/, $howcome, 2);
			$howcome = 'S';
		}
		print "<td align=center valign=top width=25%>";
		modelheadshot( 'modelno'=>$fmshowmodel,
			       'name'=>$name,
			       'howcome'=>$howcome,
			       'where'=>$fmwhere,
			       'dbhandle'=>$fmdbhandle );
		print $radiobuttons[$fmcount];
		$fmcount++;
		print "</td>\n";
		if( $fmcols > 2 )
		{
			$fmcols = 0;
			print "</tr><tr>\n";
		}
		else
		{
			$fmcols++;
		}
		if( $debug > 4 )
		{
			print STDERR "model ".$fmshowmodel.": ".
				$modellist{$fmshowmodel}.".<br>\n";
		}
	}
	print "</tr>\n";
	print "<tr><th colspan=4>";
	if( $fmofferalt eq "y" )
	{
		print $fmoffercapt.$radiobuttons[$fmcount]."&nbsp;";
	}
	if( $fmincsubmit eq "y" )
	{
		print $fmcgihandle->submit( -name=>'choose',
					  -label=>'Choose This Model' );
	}
	print "</th></tr>\n";
	print "</table>\n";
	print "</center>\n";
}

sub findrecentsets( % )
{
	my( %params )=@_;
	my( $frsoffset, $frscgihandle, $frsdbhandle, $frsdefault );
	my( @setmeth, %settypes );
	my( %wfsetopts )=('specify'=>'Specify Set By Number',
		          'image'=>'Recently Added Image Sets',
			  'video'=>'Recently Added Video Clips' );
	my( $wfimgsql, $wfimgcsr, @wfimgrec, %wfrecimg );
	my( $wfvidsql, $wfvidcsr, @wfvidrec, %wfrecvid );
	$frsoffset = $params{'offset'};
	$frscgihandle = $params{'cgihandle'};
	$frsdbhandle = $params{'dbhandle'};
	$frsdefault = $params{'default'};
	if( int($frsoffset) == 0 )
	{
		$frsoffset = Wacs::conf_get_attr("layout","newperiod");
	}
	%settypes = Wacs::geticonlist( "types" );

	print "<tr bgcolor=\"".Wacs::conf_get_attr("colours","divider").
		"\">\n";
	print "<th width=100%>Please Select A Set:</th></tr>\n";
	
	@setmeth = $frscgihandle->radio_group( -name=>'setmeth',
					  -values=>['specify','image','video'],
				          -labels=>\%wfsetopts,
					  -default=>'specify' );
				
	print "<tr><td><b>".$setmeth[0]."</b><br>Set Number: ".
		$frscgihandle->textfield( -name=>'spec_setno',
				       -length=>6,
				       -default=>$frsdefault )."</td></tr>\n";

	# find recent image sets
	$wfimgsql = "select setno, stitle, scatflag ";
	$wfimgsql.= "from ".Wacs::conf_get_attr("tables","sets")." ";
	$wfimgsql.= "where stype = 'I' and ";
	$wfimgsql.= "sadded between '".Wacs::gettoday( offset=>$frsoffset ).
		"' and '".Wacs::gettoday."' ";
	$wfimgsql.= "order by setno desc ";
	$wfimgcsr = $frsdbhandle->prepare( $wfimgsql );
	$wfimgcsr->execute;
	while( @wfimgrec = $wfimgcsr->fetchrow_array )
	{
		$wfrecimg{$wfimgrec[0]}=$wfimgrec[1]." (".$wfimgrec[0]."-".
			$settypes{$wfimgrec[2]}.")";
	}

	print "<tr><td><b>".$setmeth[1]."</b><br>Choose One:<br>".
		"<font size=-1>".
		$frscgihandle->popup_menu( -name=>'recent_img',
					-values=>
					    [ reverse sort keys %wfrecimg ],
					-labels=>\%wfrecimg ).
				       "</font></td></tr>\n";

	# find recent video sets
	$wfvidsql = "select setno, stitle, scatflag ";
	$wfvidsql.= "from ".Wacs::conf_get_attr("tables","sets")." ";
	$wfvidsql.= "where stype = 'V' and ";
	$wfvidsql.= "sadded between '".Wacs::gettoday( offset=>$frsoffset ).
		"' and '".Wacs::gettoday."' ";
	$wfvidsql.= "order by setno desc";
	$wfvidcsr = $frsdbhandle->prepare( $wfvidsql );
	$wfvidcsr->execute;
	while( @wfvidrec = $wfvidcsr->fetchrow_array )
	{
		$wfrecvid{$wfvidrec[0]}=$wfvidrec[1]." (".$wfvidrec[0]."-".
			$settypes{$wfvidrec[2]}.")";
	}

	print "<tr><td><b>".$setmeth[2]."</b><br>Choose One:<br>".
		"<font size=-1>".
		$frscgihandle->popup_menu( -name=>'recent_vid',
					-values=>
					  [ reverse sort keys %wfrecvid ],
					-labels=>\%wfrecvid ).
				       "</font></td></tr>\n";
}

sub findrecentmodels( % )
{
	my( %params )=@_;
	my( @modmeth, $frmoffset, $frmcgihandle, $frmdbhandle, $frmdefault );
	my( %pussytypes, $wfmodsql, $wfmodcsr, @wfmodrec, %wfrecmod );
	my( %wfmodopts )=('specify'=>'Specify Model By Number',
			  'recent'=>'Recently Added Models',
			  'search'=>'Search By Name' );

	$frmoffset = $params{'offset'};
	$frmcgihandle = $params{'cgihandle'};
	$frmdbhandle = $params{'dbhandle'};
	$frmdefault = $params{'default'};
	%pussytypes = Wacs::geticonlist( "pussy" );
	if( int( $frmoffset ) == 0 )
	{
		$frmoffset = Wacs::conf_get_attr("layout","newperiod");
	}

	print "<tr bgcolor=\"".Wacs::conf_get_attr("colours","divider").
		"\">\n";
	print "<th width=100%>Please Select A Model:</th></tr>\n";

	@modmeth = $frmcgihandle->radio_group( -name=>'modmeth',
					-values=>['specify','recent','search'],
					-labels=>\%wfmodopts,
					-default=>'specify' );
	# find recent models
	$wfmodsql = "select modelno, mname, mhair, mtitsize, mpussy ";
	$wfmodsql.= "from ".Wacs::conf_get_attr("tables","models")." ";
	$wfmodsql.= "where madded between '".Wacs::gettoday(
		 offset=>$frmoffset )."' and '".Wacs::gettoday."' ";
	$wfmodsql.= "order by modelno ";
	$wfmodcsr = $frmdbhandle->prepare( $wfmodsql );
	$wfmodcsr->execute;
	while( @wfmodrec = $wfmodcsr->fetchrow_array )
	{
		$wfrecmod{$wfmodrec[0]}=$wfmodrec[1]." (".$wfmodrec[2].", ".
			$wfmodrec[3]." tits, ".$pussytypes{$wfmodrec[4]}.
			" pussy, no#".$wfmodrec[0].")";
	}

	print "<tr><td><b>".$modmeth[0]."</b><br>Model Number: ".
		$frmcgihandle->textfield( -name=>'spec_modelno',
					  -length=>6,
					  -default=>$frmdefault ).
		"</td></tr>\n";
	print "<tr><td><b>".$modmeth[1]."</b><br>Recently Added: ".
		$frmcgihandle->popup_menu( -name=>'recent_mod',
					-values=>
					     [ sort {$b<=>$a} keys %wfrecmod ],
					-labels=>\%wfrecmod ).
					"</td></tr>\n";
	print "<tr><td><b>".$modmeth[2]."</b><br>Search For: ".
		$frmcgihandle->textfield( -name=>'search',
					-length=>30 )."</td></tr>\n";
}

sub modelheadshot( % )
{
	my( %params )=@_;
	my( $mhs_modelno, $mhs_name, $mhs_howcome, $mhs_where, $mhs_ikey );
	my( $mhs_dbhandle );
	my( $mhs_sql, $mhs_csr, @mhs_rec );
	
	$mhs_modelno = $params{'modelno'};
	$mhs_name = $params{'name'};
	$mhs_howcome = $params{'howcome'};
	$mhs_where = $params{'where'};
	$mhs_ikey = $params{'key'};
	$mhs_dbhandle = $params{'dbhandle'};

	#                   0        1      2      3        4
	$mhs_sql = "select modelno, mname, mhair, mlength, mtitsize, ";
	#            5         6       7      8      9         10
	$mhs_sql.= "mcupsize, mpussy, mrace, meyes, mdisting, mattributes, ";
	#            11        12       13      14      15
	$mhs_sql.= "mcountry, mrating, mimage, musual, maliases ";
	$mhs_sql.= "from ".Wacs::conf_get_attr("tables","models")." ";
	$mhs_sql.= "where modelno = '".$mhs_modelno."' ";
	
	$mhs_csr=$mhs_dbhandle->prepare( $mhs_sql );
	$mhs_csr->execute;

	@mhs_rec = $mhs_csr->fetchrow_array;
	if( $mhs_rec[0] eq "" )
	{
		print STDERR "model fetch failed for ".$mhs_modelno."\n";
		return null;
	}
	print "<!--- begin model head shot for ".$mhs_modelno." --->\n";
	print "<table>\n";
	print "<tr><th>".$mhs_rec[1]."</th></tr>\n";
	print "<tr><td align=center>";
	print "<img src=\"".Wacs::conf_get_attr("server","siteurl");
	print "icons/".$mhs_rec[13]."\" alt=[".$mhs_rec[1]."]></td></tr>\n";
	print "<tr><th>";
	print "<a href=\"".Wacs::conf_get_attr("server","cgiurl");
	print "wacsmpthumbs/".$mhs_rec[0]."\">view ".$mhs_rec[0];
	print "</a></th></tr>\n";
	print "<tr><td align=center>";
	print Wacs::WacsUI::describeher( 
		     'hair'=>$mhs_rec[2], 'length'=>$mhs_rec[3],
		     'titsize'=>$mhs_rec[4], 'cupsize'=>$mhs_rec[5],
		     'pussy'=>$mhs_rec[6], 'race'=>$mhs_rec[7],
		     'eyes'=>$mhs_rec[8], 'distmarks'=>$mhs_rec[9] );
	print "</td></tr>\n";
	if( $mhs_rec[10] ne "" )
	{
		print "<tr><td align=center>\n";
		Wacs::WacsUI::addkeyicons( $mhs_rec[10], 24 );
		print "</td></tr>\n";
	}
	if( $mhs_howcome eq "S" )
	{
		print "<tr><td align=center>Known as ".$mhs_name." at ".
				$mhs_where."</td></tr>\n";
	}
	elsif( $mhs_howcome eq "I" )
	{
		print "<tr><td align=center>".$mhs_name.": ".$mhs_ikey.
				" at ".$mhs_where."</td></tr>\n";
	}
	print "</table>\n";
	print "<!--- end model head shot for ".$mhs_modelno." --->\n";
}

# getgallery
# 
# work out what the next gallery number should be for the specification
# given
sub getgallery( % )
{
	my( %params )=@_;
	my( $ggwhich, $ggsarea, $ggscategory, $ggsdirectory, $ggsubstitute );
	my( $ggstype, $ggdbhandle );
	my( $ggsql, $ggcsr, @ggrec );
	my( $galnumber, $galbit, $ggresult );

	# check if we should be generating extra debug info
	$debug=Wacs::conf_get_attr("debug","module_wacsstd");

	# fetch the parameters
	$ggwhich=$params{'which'};
	$ggstype=$params{'stype'};
	$ggsarea=$params{'sarea'};
	$ggscategory=$params{'scategory'};
	$ggsdirectory=$params{'sdirectory'};
	$ggsubstitute=$params{'substitute'};
	$ggdbhandle=$params{'dbhandle'};

	# write the appropriate SQL query
	if( $ggwhich eq "scategory" )
	{
		$ggsql = "select scategory, count(*) ";
		$ggsql.= "from ".Wacs::conf_get_attr("tables","sets")." ";
		$ggsql.= "where sarea = '".$ggsarea."' ";
		$ggsql.= "and stype = '".$ggstype."' ";
		$ggsql.= "group by scategory ";
		$ggsql.= "order by scategory desc ";
	}
	else
	{
		$ggsql = "select sdirectory, count(*) ";
		$ggsql.= "from ".Wacs::conf_get_attr("tables","sets")." ";
		$ggsql.= "where sarea = '".$ggsarea."' ";
		$ggsql.= "and scategory = '".$ggscategory."' ";
		$ggsql.= "and stype = '".$ggstype."' ";
		$ggsql.= "group by sdirectory ";
		$ggsql.= "order by sdirectory desc ";
	}
	$ggcsr = $ggdbhandle->prepare( $ggsql );
	$ggcsr->execute;
	if( $DBI::errstr ne "" )
	{
		print STDERR "getgallery: SQL Croaked.\n";
		print STDERR "getgallery: Error was: ".$DBI::errstr."\n";
		print STDERR "getgallery: Query was: ".$ggsql."\n";
		return null;
	}
	if( ( @ggrec = $ggcsr->fetchrow_array ) )
	{
		if( $debug > 2 )
		{
			print STDERR "Highest numbered gallery in ".
				$ggsarea." is:\n";
			print STDERR ">".$ggrec[0]."< with ".$ggrec[1].
				" members.\n";
			print STDERR "Limit per gallery is ".
				Wacs::conf_get_attr("layout","setspergallery").
				"\n";
		}
		$galbit = $ggrec[0];
		$galbit =~ s/gallery//g;
		if( $ggrec[1] < Wacs::conf_get_attr("layout","setspergallery") )
		{
			$galnumber = sprintf( "%3.3d", $galbit );
		}
		else
		{
			$galnumber = sprintf( "%3.3d", ($galbit + 1));
		}
	}
	else
	{
		# if the get didn't work, default to gallery 1
		$galnumber = sprintf( "%3.3d", 1 );
	}

	# do the substitution
	if( $ggwhich eq "scategory" )
	{
		$ggresult = $ggscategory;
		$ggresult =~ s/$ggsubstitute/$galnumber/g;
	}
	else
	{
		$ggresult = $ggsdirectory;
		$ggresult =~ s/$ggsubstitute/$galnumber/g;
	}
	return( $ggresult );
}

#
# callframe
#
# create a suitable URL for page based navigation of the specified set
sub callframe( % )
{
	my( %params )=@_;
	my( $cfimgprefix, $cfsetnum, $cfcurimg, $cfmaximg, $cfmodels );
	my( $cfcgiurl, $cfbaseurl, $cfhandler, $cfpagemode, $cfappstem );
	my( $cfresult );

	$cfimgprefix = $params{'imgprefix'};
	$cfsetnum = $params{'setno'};
	$cfcurimg = $params{'curimg'};
	$cfmaximg = $params{'maximg'};
	$cfmodels = $params{'models'};
	$cfpagemode = $params{'pagemode'};
	$cfappstem = $params{'appstem'};

	$cfcgiurl = Wacs::conf_get_attr("server","cgiurl");
	$cfbaseurl = Wacs::conf_get_attr("server","baseurl");
	$cfcgiurl =~ s/^$cfbaseurl//;

	$cfresult =  "wacsframe/".$cfimgprefix.$cfsetnum."=".$cfcurimg."=".
		$cfmaximg."=".$cfmodels;
	if( $cfpagemode == 1 )
	{
		# get the handler for this menu set and then clean it
		$cfhandler = Wacs::WacsUI::menu_get_handler( for=>"models" );
		$cfbaseurl =~ s/^http:\/\///;
		$cfhandler =~ s/^http:\/\///;
		$cfhandler =~ s/^$cfbaseurl//;
		$cfhandler =~ s/\?/&quest;/g;
		$cfhandler =~ s/=/&equal;/g;

		$cfresult .= "=".$cfcgiurl."wacspage/".
			$cfappstem.$cfsetnum.".html=".
			$cfhandler;
	}
	$cfresult .=  "\">";

	return( $cfresult );
}

# kwscore_reset
#
# resets the keyword scoring system back to defaults
sub kwscore_reset( $ )
{
	my( $kws_scope )=@_;

	# reset arrays to a null list
	@kwscore_loc=(); 
	@kwscore_cat=();         
	@kwscore_det=();
	@kwscore_attr=();
	@kwscore_other=();
	$kwsshown=0;
}

# kwscore_process
#
# process the provided string looking for keywords
sub kwscore_process( % )
{
	my( %params )=@_;
	my( $kwsp_stitle, $kwsp_dbhandle );
	my( $kwspsql, $kwspcsr, @kwsprec, $kwspcount, $kwspexclude );
	my( $kwspfoundexcl );

	$debug=Wacs::conf_get_attr("debug","module_wacsstd");
	$kwsp_stitle = $params{'string'};
	$kwsp_dbhandle = $params{'dbhandle'};

	#                   0         1      2      3
	$kwspsql = "select kentryno, kflag, kword, kexclusions, ";
	#            4      5        6      7        8      9
	$kwspsql.= "kiloc, kiscore, kicat, kicscore, kidet, kidscore, ";
	#            10      11        12       13        14
	$kwspsql.= "kiattr, kiascore, kiother, kioscore, knotes, ";
	#            15      16
	$kwspsql.= "kadded, kamended ";
	$kwspsql.= "from ".Wacs::conf_get_attr("tables","keyword")." ";
	$kwspsql.= "where kflag = 'A' ";
	$kwspsql.= "order by kentryno ";
	
	$kwspcsr = $kwsp_dbhandle->prepare( $kwspsql );
	$kwspcsr->execute;

	$_ = $kwsp_stitle;
	$_ =~ y/A-Z/a-z/;
	while( @kwsprec = $kwspcsr->fetchrow_array )
	{
		if( $kwsprec[3] ne "" )
		{
			# case with exclusions 
			if( ! /$kwsprec[2]/ )
			{
				# the keyword isn't even here
				next;
			}
			# now, for the exclusions themselves
			# This tries to iterate for each seperate word
			# in case it's a list of multiple exclusions
			$kwspfoundexcl=0;
			foreach $kwspexclude ( split( /\s/, $kwsprec[3] ) )
			{
				# IS IT EXCLUDED?
				if( /$kwspexclude/ )
				{
					# YES - we have the word and it is
					# excluded
					$kwspfoundexcl++;
					last;
				}
			}
			if( $kwspfoundexcl > 0 )
			{
				# exclusion ruled it out, next time round
				# the while loop
				next;
			}
			if( /$kwsprec[2]/ )
			{
				# location - OVERWRITE AT SAME LEVEL
				if( $kwsprec[4] ne "" )
				{
					$kwscore_loc[($kwsprec[5])] = 
							$kwsprec[4];
				}
				# catflag - OVERWRITE AT SAME LEVEL
				if( $kwsprec[6] ne "" )
				{
					$kwscore_cat[($kwsprec[7])] =
						$kwsprec[6];
				}
				# detail location - OVERWRITE AT SAME
				# 		    LEVEL
				if( $kwsprec[8] ne "" )
				{
					$kwscore_det[($kwsprec[9])] =
						$kwsprec[8];
				}
				# attributes - APPEND AT SAME LEVEL
				if( $kwsprec[10] ne "" )
				{
					$kwscore_attr[($kwsprec[11])] =
						$kwscore_attr[($kwsprec[11])].
						" ".$kwsprec[10];
				}
				# other (now attire) - OVERWRITE AT
				#                      SAME LEVEL
				if( $kwsprec[12] ne "" )
				{
					$kwscore_other[($kwsprec[13])] =
						$kwsprec[12];
				}
			}
		}
		else
		{
			# no exclusions
			if( /$kwsprec[2]/ )
			{
				# location - OVERWRITE AT SAME LEVEL
				if( $kwsprec[4] ne "" )
				{
					$kwscore_loc[($kwsprec[5])] = 
						$kwsprec[4];
				}
				# catflag - OVERWRITE AT SAME LEVEL
				if( $kwsprec[6] ne "" )
				{
					$kwscore_cat[($kwsprec[7])] =
						$kwsprec[6];
				}
				# detail location - OVERWRITE AT SAME LEVEL
				if( $kwsprec[8] ne "" )
				{
					$kwscore_det[($kwsprec[9])] =
						$kwsprec[8];
				}
				# attributes - APPEND AT SAME LEVEL
				if( $kwsprec[10] ne "" )
				{
					$kwscore_attr[($kwsprec[11])] =
						$kwscore_attr[($kwsprec[11])].
						" ".$kwsprec[10];
				}
				# other (now attire) - OVERWRITE AT SAME LEVEL
				if( $kwsprec[12] ne "" )
				{
					$kwscore_other[($kwsprec[13])] =
						$kwsprec[12];
				}
			}
		}
	}

	if( $debug > 0 && $kwsshown==0 )
	{
		print "Results for:\n";
		print $kwsp_stitle."\n";
		print "Rank: Flag: Location:        Detailed:   Attributes:";
		print "                      Attire\n";
		print "====  ====  ========         ========    ========== ";
		print "                      ======\n";
		for( $kwspcount=10; $kwspcount>0 ; $kwspcount-- )
		{
			printf( "%4d  ", $kwspcount );
			printf( "%-4.4s  ", $kwscore_cat[$kwspcount] );
			printf( "%-15.15s  ", $kwscore_loc[$kwspcount] );
			printf( "%-10.10s  ", $kwscore_det[$kwspcount] );
			printf( "%-30.30s  ", $kwscore_attr[$kwspcount] );
			printf( "%-10.10s\n", $kwscore_other[$kwspcount] );
		}
		$kwsshown++;
	}
}

# kwscore_get
# 
# get the specified result from the processing of the strings provided
# previously
sub kwscore_get( % )
{
	my( %params )=@_;
	my( $kwsgwhat, $kwsgdefault );
	my( $kwsgresult, $kwsgcount );

	$kwsgwhat=$params{'what'};
	$kwsgdefault=$params{'default'};
	$kwsgresult='';

	# set default in all cases
	if( $kwsgdefault ne "" )
	{
		$kwsgresult=$kwsgdefault;
		if( $kwsgwhat eq "attributes" || $kwsgwhat eq "other" )
		{
			$kwsgresult.=" ";
		}
	}

	# loop through results
	for( $kwsgcount=10; $kwsgcount>0 ; $kwsgcount-- )
	{
		if( $kwsgwhat eq "cat" && $kwscore_cat[$kwsgcount] ne "" )
		{
			return( $kwscore_cat[$kwsgcount] );
		}
		if( $kwsgwhat eq "loc" && $kwscore_loc[$kwsgcount] ne "" )
		{
			return( $kwscore_loc[$kwsgcount] );
		}
		if( $kwsgwhat eq "det" && $kwscore_det[$kwsgcount] ne "" )
		{
			return( $kwscore_det[$kwsgcount] );
		}
		if( $kwsgwhat eq "attr" && $kwscore_attr[$kwsgcount] ne "" )
		{
			$kwsgresult.=$kwscore_attr[$kwsgcount]." ";
		}
		if( $kwsgwhat eq "other" && $kwscore_other[$kwsgcount] ne "" )
		{
			return( $kwscore_other[$kwsgcount] );
		}
	}

	# clean off any excess leading or trailing spaces
	$kwsgresult =~ s/^\s//;
	$kwsgresult =~ s/\s$//;

	# check for sensible result
	if( $kwsgwhat eq "attr" )
	{
		if( $kwsgresult ne "" )
		{
			return( $kwsgresult );
		}
	}
	else
	{
		# this kept separate for future better defaulting
		if( $kwsgresult ne "" )
		{
			return( $kwsgresult );
		}
	}
	return( "" );
}

# removedups
#
# clean up a catinfo or other attribute string
sub removedups( $ )
{
	my( $givencat )=@_;
        my( $newcat, $nextone, @catlist );

        $newcat="";
        @catlist = split( /\s+/, $givencat );
        #print "as a list: ",@catlist;
        foreach $nextone ( sort @catlist )
        {
                if( $debug > 3 )
		{
			print STDERR "removedups: considering [$nextone]... ";
		}
                if( $newcat =~ /$nextone/ )
                {
			if( $debug > 3 )
			{
				print STDERR "discarded duplicate $nextone...";
			}
                }
                else
                {
                        $newcat = $nextone ." ".$newcat;
                }
        }
        $newcat =~ s/\s$//;
	$newcat =~ s/^\s//;
	if( $debug > 2 )
	{
		print STDERR "removedups: catinfo is now: [$newcat]\n";
	}
        return( $newcat );
}

# removeconflicts
# 
# remove items that contradict the set attributes from the model attributes
sub removeconflicts( % )
{
	my( %params )=@_;
	my( $rcmodel, $rcexisting, $haspussy );
	$rcmodel = $params{'model'};
	$rcexisting = $params{'existing'};
	$haspussy=0;

	# check existing for model pussy attributes
	if( $rcexisting =~ /shaven/ )
	{
		$haspussy++;
	}
	if( $rcexisting =~ /brazilian/ )
	{
		$haspussy++;
	}
	if( $rcexisting =~ /trimmed/ )
	{
		$haspussy++;
	}
	if( $rcexisting =~ /hairy/ )
	{
		$haspussy++;
	}

	# state decision
	if( $haspussy > 0 )
	{
		# pussy type is already flagged so we're going to 
		# remove it
		if( $debug > 1 )
		{
			print STDERR 
				"removeconflicts: set already has pussy info\n";
		}
		$rcmodel=~ s/shaven//g;
		$rcmodel=~ s/brazilian//g;
		$rcmodel=~ s/trimmed//g;
		$rcmodel=~ s/hairy//g;
	}
	# return possibly modified version of model attributes
	return( $rcmodel );
}

# addassoc
#
# add a new association (simple version)
sub addassoc( % )
{
	my( %params ) = @_;
	my( $setno, $modelno, $asstype, $aadbhandle );
	my( $chksql, $chkcsr, @chkrec );
	my( $insass, $assno, $today );

	$setno = 0;
	$modelno = 0;
	$asstype = 'G';

	$setno = $params{'setno'};
	$modelno = $params{'modelno'};
	$asstype = $params{'asstype'};
	$aadbhandle = $params{'dbhandle'};

	# sanity check time (turns out wacsxmlin was insane!...)
	if( $modelno == 0 )
	{
		print STDERR "addassoc: trying to addassoc to model 0!\n";
		return -1;
	}
	if( $setno == 0 )
	{
		print STDERR "addassoc: trying to addassoc to set 0!\n";
		return -1;
	}
	if( $asstype eq "" )
	{
		$asstype = 'G';
	}

        # guard against existing entry
        $chksql = "select assocno, amodelno, asetno ";
	$chksql .= "from ".Wacs::conf_get_attr("tables","assoc")." ";
        $chksql .= "where amodelno = '".$modelno."' and asetno = '";
        $chksql .= $setno ."' ";
        $chkcsr=$aadbhandle->prepare( $chksql );
        $chkcsr->execute;
        if( @chkrec = $chkcsr->fetchrow_array )
        {
                print STDERR
		"addassoc: This association already exists as $chkrec[0]\n";
                print STDERR
		"addassoc: Model: $modelno  Set No: $chkrec[2]\n";
                return -1;
        }

	$today = Wacs::gettoday;
	$assno = alloc_nextkey( Wacs::conf_get_attr("tables","assoc"),
			"assocno", $aadbhandle );
	if( $assno eq 0 )
	{
		print STDERR
			"addassoc: Assignment of number failed for assoc\n";
		exit -1;
	}

	$insass = "insert into ".Wacs::conf_get_attr("tables","assoc");
	$insass .= " ( assocno, amodelno, asetno, ";
	$insass .= "astatus, aadded ) values ( ";
	$insass .= $assno.",".$modelno.",".$setno.",'".$asstype."','".
		$today."' ) ";

	if( $debug > 1 )
	{
		print STDERR "addassoc: SQL Query: $insass\n";
	}
        $aadbhandle->do( $insass );
        if( $DBI::errstr ne "" )
        {
		print STDERR "addassoc: SQL Choaked\n";
		print STDERR "addassoc: Query was: $insass\n";
		print STDERR "addassoc: Error was: $DBI::errstr\n";
		exit -1;
        }
	return 0;
}

# alloc_nextkey
#
# allocate next primary key number
sub alloc_nextkey( $$$ )
{
        my( $tablename, $prikey, $andbhandle )=@_;
        my( $highest, $allocsql, $alloccsr, @allocrec );
	my( $dchksql, $dchkcsr, @dchkrec );

	# find out the higher number used so far
	# NB: (null) + 1 is not equal to 1!
	$allocsql = "select max(".$prikey.") from ".$tablename." ";
	$alloccsr=$andbhandle->prepare( $allocsql );
	$alloccsr->execute;
	@allocrec = $alloccsr->fetchrow_array;
	if( $DBI::errstr ne "" )
	{
		print STDERR "SQL Croaked.\n";
		print STDERR "Error was: ".$DBI::errstr."\n";
		print STDERR "Query was: ".$allocsql."\n";
	}
	if( $allocrec[0] ne "" )
	{
		return ( ($allocrec[0])+1 );
	}

	# ok, could be an empty table or error - ask for record count to
	# confirm it's an empty table
	$dchksql = "select count(*) from ".$tablename;
	$dchkcsr = $andbhandle->prepare( $dchksql );
	$dchkcsr->execute;
	if( $DBI::errstr ne "" )
	{
		print STDERR "SQL Croaked.\n";
		print STDERR "Error was: ".$DBI::errstr."\n";
		print STDERR "Query was: ".$dchksql."\n";
		return( 0 );
	}
	if( ! (@dchkrec = $dchkcsr->fetchrow_array ) )
	{
		# assignment failed for some reason
		return( 0 );
	}
	# the correct answer should be the string "0".
	if( $dchkrec[0] eq "0" )
	{
		if( $debug > 0 )
		{
			print STDERR "alloc_nextkey: table ".$tablename.
					" appears empty - allocating 1.\n";
		}
		return( 1 );
	}
	# fall through with a failure.
	return( 0 );
}

# show that we have parsed correctly
1;
__END__

=head1 NAME

Wacs - Web Adult Content Server - Standard Building Blocks

=head1 NOTICE

THIS PROJECT DOES NOT CONTAIN ANY ADULT MATERIAL.  Rather it is a 
toolkit for presenting and managing your own collections by enabling
you to access those collections in a way similar to that provided by
subscription web sites.  Hopefully it contains many of the necessary
tools to create a good-looking site with best-of-breed facilities.

The idea of the project is to make a system for personal use, but Wacs
is quite capable of hosting an internet accessible site.  Before doing
so however, the author strongly recommends you consider the legal and
copyright infringement issues that could arise out of such an action. 
Those of you in the USA definitely need to consider the implications of
18 USC 2257 carefully.

=head1 SYNOPSIS

masthead() - creates a heading describing the basic details of a set.
kwscore_reset() - reset the keyword scoring system
kwscore_process() - process the provided string for keywords
kwscore_get() - return the requested result
findrecentsets() - allow user to select from recent sets or specify number
findrecentmodels() - allow user to select from recent models or specify number
removedups() - clean up a catinfo string
removeconflicts() - do some sanity checking on a catinfo string
addassoc() - add an association between set and model
alloc_nextkey() - allocate the next free primary key number for named table
